<!--
/*
 * b1gMail7 client library
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: autocomplete.js,v 1.11 2008/07/04 12:41:39 patrick Exp $
 *
 */

var acFields = [];

function __acKeyUp(e, f)
{
	return acFields[f.id].onKeyUp(e);
}

function __acKeyDown(e, f)
{
	return acFields[f.id].onKeyDown(e);
}

function __acHide(f)
{
	acFields[f.id].hideSuggestions();
}

function __acClick(f)
{
	var fields = f.id.split('_'),
		acField = fields[3],
		acID = fields[4];
	acFields[acField].deactivateItem(acFields[acField].activeItem);
	acFields[acField].activateItem(acID);
	acFields[acField].onEnter();
	return(false);
}

function Autocomplete()
{
	this.mode = 'plain';
	this.field = false;
	this.suggestions = [];
	this.suggestionPos = 0;
	this.text = '';
	this.activeItem = 0;
	this.setSearchFunction = function(f)
	{
		this.searchAddresses = f;
	}
	this.setMode = function(mode)
	{
		this.mode = mode;
	}
	this.setField = function(field)
	{
		acFields[field] = this;
		this.field = EBID(field);
		this.field.onkeypress = function(e) { if(!e && event) e = event; return e.keyCode!=13; }
		this.field.onkeyup = function(e) { if(!e && event) e = event; return __acKeyUp(e, this); }
		this.field.onkeydown = function(e) { if(!e && event) e = event; return __acKeyDown(e, this); }
		this.field.onblur = function(e) { __acHide(this); }
	}
	this.setSuggestions = function(suggestions)
	{
		this.suggestions = [];
		for(var i=0; i<suggestions.length; i++)
		{
			var item = [];
			item['text'] = suggestions[i];
			this.suggestions.push(item);
		}
		this.renderSuggestions();
	}
	this.renderSuggestions = function()
	{
		if(this.suggestions.length > 0
			&& !(this.suggestions.length==1 && this.suggestions[0]['text'].length < 2))
		{
			// remove old data
			var child;
			while(child = this.completeDiv.firstChild)
				this.completeDiv.removeChild(child);
			
			// add new data
			for(var i=this.suggestions.length-1; i>=0; i--)
			{
				var suggestion = this.suggestions[i]['text'];
				suggestion = HTMLEntities(suggestion);
				suggestion = suggestion.replace(this.text, '<b>' + this.text + '</b>');
									
				var item = document.createElement('div');
				item.setAttribute('id', '__suggestionItem_' + this.field.id + '_' + i);
				item.onmousedown = function() { return __acClick(this); }
				item.className = 'suggestionInactive';
				item.innerHTML = suggestion;
				this.completeDiv.insertBefore(item, this.completeDiv.firstChild);
				
				this.suggestions[i]['div'] = item;
			}
			
			this.activateItem(0);
			this.showSuggestions();
		}
		else
		{
			this.hideSuggestions();
		}
	}
	this.deactivateItem = function(i)
	{
		this.suggestions[i]['div'].className = 'suggestionInactive';
	}
	this.activateItem = function(i)
	{
		this.suggestions[i]['div'].className = 'suggestionActive';
		this.activeItem = i;
	}
	this.onKeyUp = function(e)
	{
		if(e.isChar || (e.keyCode != 13 && e.keyCode != 38 && e.keyCode != 40))
		{
			var text = this.field.value;
			
			this.suggestionPos = 0;
			if(this.mode == 'semicolonSeparated')
			{
				var semicolon = text.lastIndexOf(',');
				if(semicolon >= 0)
				{
					this.suggestionPos = semicolon+1;
					text = text.substring(semicolon+1);
				}
			}
			
			text = trim(text);
			text = text.replace('"', '');
			text = text.replace('<', '');
			text = text.replace('>', '');
			
			if(text.length >= 1)
			{
				this.text = text;
				this.searchAddresses(this, text);
			}
			else
			{
				this.hideSuggestions();
			}
		}
		
		if(e.keyCode == 13 || e.keyCode == 38 || e.keyCode == 40)
			return(false);
		else
			return(true);
	}
	this.onKeyDown = function(e)
	{			
		if(e.keyCode == 13)
		{
			this.onEnter();
			return(false);
		}
		else if(e.keyCode == 38)
		{
			this.deactivateItem(this.activeItem);
			if(this.activeItem > 0)
			{
				this.deactivateItem(this.activeItem);
				this.activeItem--;
			}
			this.activateItem(this.activeItem);
			return(false);
		}
		else if(e.keyCode == 40)
		{
			if(this.activeItem < this.suggestions.length-1)
			{
				this.deactivateItem(this.activeItem);
				this.activeItem++;
			}
			this.activateItem(this.activeItem);
			return(false);
		}
			
		return(true);
	}
	this.onEnter = function()
	{
		var newValue = this.suggestions[this.activeItem]['text'];
		if(this.suggestionPos > 0)
			newValue = ' ' + newValue;
		this.field.value = this.field.value.substring(0, this.suggestionPos) + newValue;
		this.hideSuggestions();
	}
	this.hideSuggestions = function()
	{
		this.completeDiv.style.display = 'none';
	}
	this.showSuggestions = function()
	{
		this.completeDiv.style.display = 'block';
	}
	this.setUp = function()
	{
		var body = document.getElementsByTagName('body').item(0);
		
		this.completeDiv = document.createElement('div');
		this.completeDiv.setAttribute('id', '__acCompleteDiv_' + this.field.id);
		this.completeDiv.style.display = 'none';
		this.completeDiv.style.position = 'absolute';
		this.completeDiv.style.zIndex = 500;
		this.completeDiv.style.width = getElementMetrics(this.field, 'w') + 'px';
		this.completeDiv.style.left = (getElementMetrics(this.field, 'x')-1) + 'px';
		this.completeDiv.style.top = (getElementMetrics(this.field, 'y')+getElementMetrics(this.field, 'h')+1) + 'px';
		this.completeDiv.className = 'completeDiv';
		
		body.insertBefore(this.completeDiv, body.firstChild);
	}
}

//-->