<?php
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: webdav.php,v 1.17 2008/10/24 12:36:06 patrick Exp $
 *
 */

define('INTERFACE_MODE', true);
include('../serverlib/init.inc.php');
include('../serverlib/webdisk.class.php');
include('../serverlib/webdav_server.class.php');

/**
 * WebDAV interface implementation
 *
 */
class BMWebDAV_Server extends WebDAV_Server 
{
	var $_userObject;
	var $_userRow;
	var $_groupObject;
	var $_groupRow;
	var $_webdisk;
	
	/**
	 * check user login, initiate session
	 *
	 * @return bool
	 */
	function CheckLogin()
	{
		if(empty($this->_user) || empty($this->_pass))
			return(false);
		
		// login
		list($result, $userID) = BMUser::Login($this->_user, $this->_pass, false, false);
		
		// login OK?
		if($result == USER_OK)
		{
			// get user and group
			$this->_userObject = _new('BMUser', array($userID));
			$this->_groupObject = $this->_userObject->GetGroup();
			$this->_userRow = $this->_userObject->Fetch();
			$this->_groupRow = $this->_groupObject->Fetch();
			
			// check privileges
			if($this->_groupRow['webdav'] == 'yes')
			{
				$this->_webdisk = _new('BMWebdisk', array($this->_userRow['id']));
				return(true);
			}
			else 
			{
				// log
				PutLog(sprintf('WebDAV login as <%s> failed (disallowed by group settings)',
					$this->_user),
					PRIO_NOTE,
					__FILE__,
					__LINE__);
				return(false);
			}
		}
		
		return(false);
	}
	
	/**
	 * check if a lock for $path exists
	 *
	 * @param string $path Path
	 * @return array
	 */
	function CheckLock($path)
	{
		global $db;
		
		$result = false;
		$res = $db->Query('SELECT owner,token,created,modified,expires,scope FROM {pre}disklocks WHERE path=? AND user=?',
			$path,
			$this->_userRow['id']);
		if($res->RowCount() != 0)
			$result = $res->FetchArray(MYSQL_ASSOC);
		$res->Free();
		
		if(is_array($result) && $result['expires'] > 0 && $result['expires'] < time())
			$result = false;
		
		return($result);
	}
	
	/**
	 * PROPPATCH handler
	 *
	 * @return array
	 */
	function Handler_Proppatch()
	{
		return(false);
	}
	
	function Handler_Unlock($lockToken)
	{
		global $db;
		
		$db->Query('DELETE FROM {pre}disklocks WHERE user=? AND path=? AND token=?',
			$this->_userRow['id'],
			$this->_path,
			$lockToken);
		
		if($db->AffectedRows() > 0)
		{
			$this->_headers('204 No content');
			return(true);
		}
		else 
		{
			$this->_headers('409 Conflict');
			return(false);
		}
	}
	
	/**
	 * LOCK handler
	 *
	 * @param string $lockScope Scope
	 * @param string $lockType Type
	 * @param string $lockOwner Owner
	 * @param string $lockToken Token
	 * @param int $lockExpires Expire time
	 * @param bool $update Update?
	 */
	function Handler_Lock($lockScope, $lockType, $lockOwner, $lockToken, $lockExpires, $update = false)
	{
		global $db;
		
		$response = false;
		
		if($update)
		{
			// get lock row
			$res = $db->Query('SELECT owner,scope FROM {pre}disklocks WHERE path=? AND token=? AND user=?',
				$this->_path,
				$lockToken,
				$this->_userRow['id']);
			if($res->RowCount() == 0)
				return(false);
			$row = $res->FetchArray(MYSQL_ASSOC);
			$res->Free();
			
			// update
			$db->Query('UPDATE {pre}disklocks SET expires=?,modified=? WHERE path=? AND token=? AND user=?',
				$lockExpires,
				time(),
				$this->_path,
				$lockToken,
				$this->_userRow['id']);
			$response = $this->CheckLock($this->_path);
		}
		
		else 
		{
			$db->Query('INSERT INTO {pre}disklocks(`user`,`path`,`token`,`type`,`created`,`modified`,`expires`,`scope`,`owner`) '
				.	'VALUES(?,?,?,?,?,?,?,?,?)',
				$this->_userRow['id'],
				$this->_path,
				$lockToken,
				$lockType,
				time(),
				time(),
				$lockExpires,
				$lockScope,
				$lockOwner);
			if($db->AffectedRows() == 1)
				$response = $this->CheckLock($this->_path);
		}
		
		return($response);
	}
	
	/**
	 * PROPFIND handler
	 *
	 * @param array $requested Requested props
	 * @param string $depth Depth
	 * @return array
	 */
	function Handler_Propfind($requested, $depth)
	{
		$result = $this->_webdisk->ParsePath($this->_path, true);
		if($result === false)
			return(false);
		
		// get item ID and item type
		list($itemID, $itemType) = $result;
		
		// folder?
		if($itemType == WEBDISK_ITEM_FOLDER)
		{
			$result = array();
			
			// self
			$this->_propfindItem($this->_webdisk->GetStructFolderInfo($itemID), $requested, $result, true);
			
			// contents
			if($depth == 'infinity' || $depth > 0)
			{
				$contents = $this->_webdisk->GetFolderContent($itemID);
				foreach($contents as $dirItem)
					$this->_propfindItem($dirItem, $requested, $result, false);
			}
			
			return($result);
		}
		
		else if($itemType == WEBDISK_ITEM_FILE)
		{
			$result = array();
			
			// self
			$this->_propfindItem($this->_webdisk->GetStructFileInfo($itemID), $requested, $result, true);
			
			return($result);
		}
		
		return(false);
	}
	
	/**
	 * MKCOL handler
	 *
	 * @param string $parent Dirname
	 * @param string $dir Basename
	 * @return bool
	 */
	function Handler_MKCol($parent, $dir)
	{
		if($parent != '/')
		{
			$result = $this->_webdisk->ParsePath($parent, true);
			if($result === false)
			{
				$this->_headers('409 Parent path does not exist');
				return(false);
			}
			
			list($parentID, $parentType) = $result;
		}
		else 
		{
			$parentID = 0;
			$parentType = WEBDISK_ITEM_FOLDER;
		}
		
		if($parentType == WEBDISK_ITEM_FILE
			|| $this->_webdisk->FolderExists($parentID, $dir)
			|| $this->_webdisk->FileExists($parentID, $dir))
		{
			$this->_headers('405 Method not allowed');
			return(false);
		}
		
		if($this->_webdisk->CreateFolder($parentID, $dir))
		{
			$this->_headers('201 Created');
			return(true);
		}
		else 
		{
			$this->_headers('403 Forbidden');
			return(false);
		}
	}
	
	/**
	 * DELETE handler
	 *
	 * @param string $parent Dirname
	 * @param string $name Basename
	 * @param bool $isDir Is directory?
	 */
	function Handler_Delete($parent, $name, $isDir)
	{
		$result = $this->_webdisk->ParsePath($parent . (substr($parent, -1) != '/' ? '/' : '')  . $name, !$isDir);
		if($result === false)
		{
			$this->_headers('404 Not found');
			exit();
		}
		else 
		{
			list($fileID, $fileType) = $result;
			
			/*if($fileType == ($isDir ? WEBDISK_ITEM_FOLDER : WEBDISK_ITEM_FILE))
			{*/
				if(($fileType == WEBDISK_ITEM_FILE && $this->_webdisk->DeleteFile($fileID))
					|| ($fileType == WEBDISK_ITEM_FOLDER && $this->_webdisk->DeleteFolder($fileID)))
				{
					$this->_headers('204 No content');
					exit();
				}
				else 
				{
					$this->_headers('500 Internal server error');
					exit();
				}
			/*}
			else 
			{
				$this->_headers('400 Bad request');
				exit();
			}*/
		}
	}
	
	/**
	 * GET handler for files and directories
	 *
	 * @param array $ranges Ranges to get
	 * @return bool
	 */
	function Handler_Get($ranges)
	{
		global $db;
		
		$result = $this->_webdisk->ParsePath($this->_path, true);
		if($result === false)
			return(false);
		
		// file?
		list($fileID, $fileType) = $result;
		if($fileType == WEBDISK_ITEM_FOLDER)
		{
			$this->_directoryListing($fileID);
			exit();
		}
		
		// send
		$fileInfo = $this->_webdisk->GetFileInfo($fileID);
		if($fileInfo !== false)
		{
			if($this->_groupRow['traffic'] <= 0 || ($this->_userRow['traffic_down']+$this->_userRow['traffic_up']+$fileInfo['size']) <= $this->_groupRow['traffic'])
			{
				$speedLimit = $this->_groupRow['wd_member_kbs'] <= 0 ? -1 : $this->_groupRow['wd_member_kbs'];
				$db->Query('UPDATE {pre}users SET traffic_down=traffic_down+? WHERE id=?',
					$fileInfo['size'],
					$this->_userRow['id']);
				
				// send file
				header('Pragma: public');
				header('Last-Modified: ' . gmdate('D, d M Y H:i:s', $fileInfo['modified']) . ' GMT');
				header('Content-Type: ' . $fileInfo['contenttype']);
				header('Content-Length: ' . $fileInfo['size']);
				header('Content-Disposition: attachment; filename="' . addslashes($fileInfo['dateiname']) . '"');
				Add2Stat('wd_down', ceil($fileInfo['size']/1024));
				SendFile(DataFilename($fileInfo['id'], 'dsk'), $speedLimit);
				exit();
			}
			else 
			{
				$this->_headers('509 Bandwith limit exceeded');
				exit();
			}
		}
		
		return(false);
	}
	
	/**
	 * PUT handler
	 *
	 * @param int $fileSize File size
	 * @param string $contentType Content type
	 * @return bool
	 */
	function Handler_Put($fileSize, $contentType)
	{
		global $db;
		
		if(substr($this->_path, -1) == '/')
			$this->_path = substr($this->_path, 0, -1);
		
		$parent = dirname($this->_path);
		$name = basename($this->_path);
		
		// get parent
		if($parent == '/')
		{
			$parentID = 0;
			$parentType = WEBDISK_ITEM_FOLDER;
		}
		else 
		{
			$result = $this->_webdisk->ParsePath($parent, false);
			if($result === false)
				return(false);
			list($parentID, $parentType) = $result;
			if($parentType != WEBDISK_ITEM_FOLDER)
				return(false);
		}
		
		// limit!
		$spaceLimit = $this->_groupRow['webdisk'];
		$usedSpace = $this->_webdisk->GetUsedSpace();
		
		if($this->_groupRow['traffic'] <= 0 || ($this->_userRow['traffic_down']+$this->_userRow['traffic_up']+$fileSize) <= $this->_groupRow['traffic'])
		{
			if($spaceLimit == -1 || ($usedSpace+$fileSize) <= $spaceLimit)
			{
				// exists => delete
				if($fileID = $this->_webdisk->FileExists($parentID, $name))
				{
					$this->_webdisk->DeleteFile($fileID);
					$result = '204 No content';
				}
				
				// create file
				$fileID = $this->_webdisk->CreateFile($parentID, $name, $contentType, $fileSize);
				if($fileID)
				{
					$filename = DataFilename($fileID, 'dsk');
					
					$copiedBytes = 0;
					$sourceFP = @fopen('php://input', 'rb');
					$fp = @fopen($filename, 'wb');
					
					while(!feof($sourceFP) && $copiedBytes <= $fileSize)
					{
						$buf = fread($sourceFP, 4096);
						fwrite($fp, $buf);
						$copiedBytes += strlen($buf);
					}
					
					fclose($sourceFP);
					fclose($fp);
					
					$db->Query('UPDATE {pre}users SET traffic_up=traffic_up+? WHERE id=?',
						$fileSize,
						$this->_userRow['id']);
					Add2Stat('wd_up', ceil($fileSize/1024));
					
					$result = '201 Created';
				}
			}
		}
		else 
		{
			$this->_headers('509 Bandwith limit exceeded');
			exit();
		}
		
		if(isset($result))
		{
			$this->_headers($result);
			exit();
		}
			
		return(false);
	}
	
	/**
	 * COPY handler
	 *
	 * @param string $depth Depth
	 * @param string $dest Destination
	 * @return bool
	 */
	function Handler_Copy($depth, $dest)
	{
		$overwrite = !isset($_SERVER['HTTP_OVERWRITE'])
		 				|| $_SERVER['HTTP_OVERWRITE'] == 'T';
		 
		// parse source path
		$result = $this->_webdisk->ParsePath($this->_path, true);
		if($result === false)
		{
			$this->_headers('404 Not found');
			exit();
		}
		list($sourceID, $sourceType) = $result;
		if($sourceType == WEBDISK_ITEM_FOLDER && $depth == 1)
		{
			$this->_headers('400 Bad request');
			exit();
		}
		
		// pase dest path
		if(substr($dest, -1) == '/')
		{
			$isDir = true;
			$dest = substr($dest, 0, -1);
		}
		else 
			$isDir = false;
			
		$destParent = dirname($dest);
		$destName = basename($dest);
		$result = $this->_webdisk->ParsePath($destParent, false);
		if($result === false)
		{
			$this->_headers('409 Conflict');
			exit();
		}
		list($destParentID, $destParentType) = $result;
		if($destParentType != WEBDISK_ITEM_FOLDER)
		{
			$this->_headers('409 Conflict');
			exit();
		}
		
		// exists?
		if(!$overwrite && ($this->_webdisk->FolderExists($destParentID, $destName)
							|| $this->_webdisk->FileExists($destParentID, $destName)))
		{
			$this->_headers('412 Precondition failed');
			exit();
		}
		
		// limits
		$spaceLimit = $this->_groupRow['webdisk'];
		$usedSpace = $this->_webdisk->GetUsedSpace();
		
		// copy file
		if($sourceType == WEBDISK_ITEM_FILE)
		{
			$fileInfo = $this->_webdisk->GetFileInfo($sourceID);
			
			if($spaceLimit == -1 || ($usedSpace+$fileInfo['size']) <= $spaceLimit)
			{
				if($id = $this->_webdisk->FolderExists($destParentID, $destName))
				{
					$destParentID = $id;
					$destName = $fileInfo['dateiname'];
					$ex = true;
				}
					
				if($id = $this->_webdisk->FileExists($destParentID, $destName))
				{
					$this->_webdisk->DeleteFile($id);
					$ex = true;
				}
				
				if($this->_webdisk->CopyFile($destParentID, $sourceID, $destName))
				{
					$this->_headers(isset($ex) ? '204 No content' : '201 Created');
					exit();
				}
			}
			else 
			{
				$this->_headers('403 Forbidden');
				exit();
			}
		}
		
		// copy folder
		else if($sourceType == WEBDISK_ITEM_FOLDER)
		{
			$maxSpace = $spaceLimit == -1 ? -1 : $spaceLimit - $usedSpace;
			
			if($id = $this->_webdisk->FileExists($destParentID, $destName))
			{
				$this->_headers('409 Conflict');
				exit();
			}
			
			if($id = $this->_webdisk->FolderExists($destParentID, $destName))
			{
				if($isDir)
					$destParentID = $id;
				else 
					$this->_webdisk->DeleteFolder($id);
				$ex = true;
			}
			
			if($this->_webdisk->CopyFolder($destParentID, $sourceID, $maxSpace, $destName))
			{
				$this->_headers(isset($ex) ? '204 No content' : '201 Created');
				exit();
			}
			else 
			{
				$this->_headers('403 Forbidden');
				exit();
			}
		}
		
		return(false);
	}
	
	/**
	 * MOVE handler
	 *
	 * @param string $depth Depth
	 * @param string $dest Destination
	 * @return bool
	 */
	function Handler_Move($depth, $dest)
	{
		$overwrite = !isset($_SERVER['HTTP_OVERWRITE'])
		 				|| $_SERVER['HTTP_OVERWRITE'] == 'T';
		 
		// parse source path
		$result = $this->_webdisk->ParsePath($this->_path, true);
		if($result === false)
		{
			$this->_headers('404 Not found');
			exit();
		}
		list($sourceID, $sourceType) = $result;
		if($sourceType == WEBDISK_ITEM_FOLDER && $depth == 1)
		{
			$this->_headers('400 Bad request');
			exit();
		}
		
		// pase dest path
		if(substr($dest, -1) == '/')
		{
			$isDir = true;
			$dest = substr($dest, 0, -1);
		}
		else 
			$isDir = false;
			
		$destParent = dirname($dest);
		$destName = basename($dest);
		$result = $this->_webdisk->ParsePath($destParent, false);
		if($result === false)
		{
			$this->_headers('409 Conflict');
			exit();
		}
		list($destParentID, $destParentType) = $result;
		if($destParentType != WEBDISK_ITEM_FOLDER)
		{
			$this->_headers('409 Conflict');
			exit();
		}
		
		// exists?
		if(!$overwrite && ($this->_webdisk->FolderExists($destParentID, $destName)
							|| $this->_webdisk->FileExists($destParentID, $destName)))
		{
			$this->_headers('412 Precondition failed');
			exit();
		}
		
		// move file
		if($sourceType == WEBDISK_ITEM_FILE)
		{
			$fileInfo = $this->_webdisk->GetFileInfo($sourceID);
			
			if($id = $this->_webdisk->FolderExists($destParentID, $destName))
			{
				$destParentID = $id;
				$destName = $fileInfo['dateiname'];
				$ex = true;
			}
				
			if($id = $this->_webdisk->FileExists($destParentID, $destName))
			{
				$this->_webdisk->DeleteFile($id);
				$ex = true;
			}
			
			if($this->_webdisk->MoveFile($destParentID, $sourceID, $destName))
			{
				$this->_headers(isset($ex) ? '204 No content' : '201 Created');
				exit();
			}
		}
		
		// move folder
		else if($sourceType == WEBDISK_ITEM_FOLDER)
		{
			if($id = $this->_webdisk->FileExists($destParentID, $destName))
			{
				if($overwrite)
					$this->_webdisk->DeleteFile($id);
				else 
				{
					$this->_headers('409 Conflict');
					exit();
				}
			}
			
			if($id = $this->_webdisk->FolderExists($destParentID, $destName))
			{
				if($isDir)
					$destParentID = $id;
				else 
					$this->_webdisk->DeleteFolder($id);
				$ex = true;
			}
			
			if($this->_webdisk->MoveFolder($destParentID, $sourceID, $destName))
			{
				$this->_headers(isset($ex) ? '204 No content' : '201 Created');
				exit();
			}
			else 
			{
				$this->_headers('403 Forbidden');
				exit();
			}
		}
		
		return(false);
	}
	
	/**
	 * propfind helper
	 *
	 * @param array $dirItem Dir item array
	 * @param array $requested Requested props
	 * @param array $result Result array
	 * @param bool $self Item = self?
	 */
	function _propfindItem($dirItem, $requested, &$result, $self = false)
	{
		$item = array();
		
		$href = !$self
			? $_SERVER['SCRIPT_NAME'] . $this->_path . (substr($this->_path, -1) != '/' ? '/' : '') . $dirItem['title']
			: $_SERVER['SCRIPT_NAME'] . $this->_path;
		if($dirItem['type'] == WEBDISK_ITEM_FOLDER && substr($href, -1) != '/')
			$href .= '/';
		
		$dbProps = $this->_webdisk->GetDBProps($this->_path, $this->_userRow['id']);
			
		foreach($requested as $prop)
		{
			$uProp = strtoupper($prop);
			
			switch($uProp)
			{
			case 'GETCONTENTLENGTH':
				if($dirItem['type'] == WEBDISK_ITEM_FILE)
					$item[$prop] = $dirItem['size'];
				break;
				
			case 'GETLASTMODIFIED':
				$item[$prop] = gmdate('D, d M Y H:i:s', $dirItem['modified']) . ' GMT';
				break;
			
			case 'CREATIONDATE':
				$item[$prop] = gmdate(WEBDAV_DATE_FORMAT, $dirItem['created']);
				break;
				
			case 'LASTACCESSED':
				$item[$prop] = gmdate('D, d M Y H:i:s', $dirItem['accessed']);
				break;
				
			case 'ISHIDDEN':
				$item[$prop] = substr($dirItem['title'], 0, 1) == '.' || $self
					? 'true'
					: 'false';
				break;
				
			case 'GETCONTENTTYPE':
				if($dirItem['type'] == WEBDISK_ITEM_FILE)
					$item[$prop] = $dirItem['ctype'];
				else 
					$item[$prop] = 'httpd/unix-directory';
				break;
				
			case 'RESOURCETYPE':
				$item[$prop] = $dirItem['type'] == WEBDISK_ITEM_FOLDER ? '<D:collection/>' : '';
				break;
			
			case 'ISREADONLY':
				$item[$prop] = 'false';
				break;
				
			case 'HREF':
				$item[$prop] = $href;
				break;
			
			case 'ISCOLLECTION':
				$item[$prop] = $dirItem['type'] == WEBDISK_ITEM_FOLDER ? 'true' : 'false';
				break;
			
			case 'NAME':
			case 'DISPLAYNAME':
				$item[$prop] = $dirItem['title'] == '' ? '/' : $dirItem['title'];
				break;
				
			case 'ISROOT':
				$item[$prop] = $dirItem['id'] == 0 ? 'true' : 'false';
				break;
			
			default:
				foreach($dbProps as $dbProp)
					if(strtoupper($dbProp['name']) == $prop)
						$item['prop'] == $dbProp['value'];
				break;
			}
		}
		
		$result[$href] = $item;
	}
	
	/**
	 * send a directory listing for non-WebDAV-clients
	 *
	 * @param int $folderID Folder ID
	 */
	function _directoryListing($folderID)
	{
		$this->_headers('200 OK');
		$columnFormat = '%-20s %-10s %-s' . $this->_crlf;
		$contents = $this->_webdisk->GetFolderContent($folderID);
		
		echo '<html>' . $this->_crlf;
		echo '<head>' . $this->_crlf;
		echo '	<title>Index of ' . htmlentities($this->_path) . '</title>' . $crlf;
		echo '</head>' . $this->_crlf;
		echo '<body>' . $this->_crlf;
		echo '	<h1>Index of ' . htmlentities($this->_path) . '</h1>' . $this->_crlf;
		echo '	<pre>' . $this->_crlf;
		printf($columnFormat, 'Last modified', 'Size', 'Name');
		echo '<hr />';
		
		if($this->_path != '/')
		{
			printf($columnFormat,
				'',
				'-',
				'<a href="' . $_SERVER['SCRIPT_NAME'] . $this->_path . (substr($this->_path, -1) != '/' ? '/' : '') . '../">Parent directory</a>');
		}
		
		foreach($contents as $item)
		{
			printf($columnFormat,
				date('Y-m-d H:i:s', $item['modified']),
				$item['type'] == WEBDISK_ITEM_FOLDER ? '-' : $item['size'],
				'<a href="' . $_SERVER['SCRIPT_NAME'] . $this->_path . (substr($this->_path, -1) != '/' ? '/' : '') . urlencode($item['title']) . ($item['type'] == WEBDISK_ITEM_FOLDER ? '/' : '') . '">' . htmlentities($item['title']) . '</a>');
		}
		
		echo '<hr /></pre>' . $this->_crlf;
		echo '	<address>b1gMail/' . B1GMAIL_VERSION . '</address>' . $this->_crlf;
		echo '</body>' . $this->_crlf;
		echo '</html>' . $this->_crlf;
		
		exit();
	}
}

/**
 * debug?
 */
function webDAVLog($msg)
{
	if(!DEBUG)
		return;
	if($fp = fopen(B1GMAIL_DIR . 'logs/webdav.log', 'a'))
	{
		fwrite($fp, sprintf("[%s] %s\n",
			date('r'),
			$msg));
		fclose($fp);
	}
}
function webDAVErrorHandler($errNo, $errStr, $errFile, $errLine)
{
	if($fp = fopen(B1GMAIL_DIR . 'logs/webdav.log', 'a'))
	{
		fwrite($fp, sprintf("\n--\n[%s] PHP error:\n\tNo: %d\n\tStr: %s\n\tFile: %s@ %d\n--\n\n",
			date('r'),
			$errNo,
			$errStr,
			$errFile,
			$errLine));
		fclose($fp);
	}
}
if(DEBUG)
	set_error_handler('webDAVErrorHandler');
	
/**
 * run!
 */
$srv = _new('BMWebDAV_Server', array(substr(__FILE__, strlen(dirname(__FILE__))+1), $bm_prefs['selfurl']));
$srv->ProcessRequest();
?>