<?php 
/*
 * b1gMail7
 * (c) 2002-2006 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: search.plugin.php,v 1.8 2009/02/18 10:38:02 patrick Exp $
 *
 */

/**
 * b1gMail base search provider plugin
 *
 */
class B1GMailSearchProvider extends BMPlugin 
{
	/**
	 * constructor
	 *
	 * @return B1GMail_SearchProvider
	 */
	function B1GMailSearchProvider()
	{
		global $lang_admin;
		
		// extract version
		sscanf('$Revision: 1.8 $', chr(36) . 'Revision: %d.%d ' . chr(36),
			$vMajor,
			$vMinor);
		
		// plugin info
		$this->type					= BMPLUGIN_DEFAULT;
		$this->name					= 'b1gMail Search Provider';
		$this->author				= 'B1G Software';
		$this->version				= sprintf('%d.%d', $vMajor, $vMinor);
		
		// admin pages
		$this->admin_pages			= true;
		$this->admin_page_title		= $lang_admin['searchprovider'];
	}
	
	/**
	 * perform search
	 *
	 * @param string $query Query
	 * @return array Results
	 */
	function OnSearch($query)
	{
		global $bm_prefs, $userRow, $groupRow, $thisUser, $lang_user, $db;
		
		// prepare
		$results = array();
		$q = '\'%' . $db->Escape($query) . '%\'';
		
		// prefs
		$searchIn = @unserialize($bm_prefs['search_in']);
		if(!is_array($searchIn))
			$searchIn = array();
			
		//
		// mails
		//
		if(isset($searchIn['mails']))
		{
			$thisResults = array();
			$res = $db->Query('SELECT id,betreff FROM {pre}mails WHERE userid=? AND (betreff LIKE ' . $q . ' OR von LIKE ' . $q . ' OR an LIKE ' . $q . ') ORDER BY betreff ASC',
				$thisUser->_id);
			while($row = $res->FetchArray(MYSQL_ASSOC))
				$thisResults[] = array(
					'icon'		=> 'mail_markunread',
					'title'		=> $row['betreff'],
					'link'		=> sprintf('email.read.php?id=%d&', $row['id'])
				);
			$res->Free();
			
			if(count($thisResults) > 0)
				$results[] = array(
					'title'		=> $lang_user['mails'],
					'results'	=> $thisResults
				);
		}
		//
		// sms outbox
		//
		if(isset($searchIn['sms']))
		{
			$thisResults = array();
			$res = $db->Query('SELECT id,text FROM {pre}smsend WHERE isSMS=1 AND user=? AND (text LIKE ' . $q . ' OR `from` LIKE ' . $q . ' OR `to` LIKE ' . $q . ') ORDER BY text ASC',
				$thisUser->_id);
			while($row = $res->FetchArray(MYSQL_ASSOC))
				$thisResults[] = array(
					'icon'		=> 'ico_smsoutbox',
					'title'		=> $row['text'],
					'link'		=> sprintf('sms.php?action=outbox&show=%d&', $row['id'])
				);
			$res->Free();
			
			if(count($thisResults) > 0)
				$results[] = array(
					'title'		=> $lang_user['smsoutbox'],
					'results'	=> $thisResults
				);
		}
			
		//
		// calendar
		//
		if(isset($searchIn['calendar']))
		{
			$thisResults = array();
			$res = $db->Query('SELECT id,title FROM {pre}dates WHERE user=? AND (title LIKE ' . $q . ' OR text LIKE ' . $q . ') ORDER BY title ASC',
				$thisUser->_id);
			while($row = $res->FetchArray(MYSQL_ASSOC))
				$thisResults[] = array(
					'icon'		=> 'ico_calendar',
					'title'		=> $row['title'],
					'link'		=> sprintf('organizer.calendar.php?action=editDate&id=%d&', $row['id'])
				);
			$res->Free();
			
			if(count($thisResults) > 0)
				$results[] = array(
					'title'		=> $lang_user['dates2'],
					'results'	=> $thisResults
				);
		}
			
		//
		// tasks
		//
		if(isset($searchIn['tasks']))
		{
			$thisResults = array();
			$res = $db->Query('SELECT id,titel FROM {pre}tasks WHERE user=? AND (titel LIKE ' . $q . ' OR comments LIKE ' . $q . ') ORDER BY titel ASC',
				$thisUser->_id);
			while($row = $res->FetchArray(MYSQL_ASSOC))
				$thisResults[] = array(
					'icon'		=> 'ico_todo',
					'title'		=> $row['titel'],
					'link'		=> sprintf('organizer.todo.php?action=editTask&id=%d&', $row['id'])
				);
			$res->Free();
			
			if(count($thisResults) > 0)
				$results[] = array(
					'title'		=> $lang_user['tasks'],
					'results'	=> $thisResults
				);
		}
			
		//
		// addressbook
		//
		if(isset($searchIn['addressbook']))
		{
			$thisResults = array();
			$res = $db->Query('SELECT id,vorname,nachname FROM {pre}adressen WHERE user=? AND (CONCAT(vorname,\' \',nachname) LIKE ' . $q . ' OR CONCAT(nachname,\', \',vorname) LIKE ' . $q . ') ORDER BY nachname,vorname ASC',
				$thisUser->_id);
			while($row = $res->FetchArray(MYSQL_ASSOC))
				$thisResults[] = array(
					'icon'		=> 'ico_addressbook',
					'title'		=> $row['nachname'] . ', ' . $row['vorname'],
					'link'		=> sprintf('organizer.addressbook.php?action=editContact&id=%d&', $row['id'])
				);
			$res->Free();
			
			if(count($thisResults) > 0)
				$results[] = array(
					'title'		=> $lang_user['contacts'],
					'results'	=> $thisResults
				);
		}
			
		//
		// notes
		//
		if(isset($searchIn['notes']))
		{
			$thisResults = array();
			$res = $db->Query('SELECT id,text FROM {pre}notes WHERE user=? AND text LIKE ' . $q . ' ORDER BY text ASC',
				$thisUser->_id);
			while($row = $res->FetchArray(MYSQL_ASSOC))
				$thisResults[] = array(
					'icon'		=> 'ico_notes',
					'title'		=> $row['text'],
					'link'		=> sprintf('organizer.notes.php?show=%d&', $row['id'])
				);
			$res->Free();
			
			if(count($thisResults) > 0)
				$results[] = array(
					'title'		=> $lang_user['notes'],
					'results'	=> $thisResults
				);
		}
			
		//
		// webdisk
		//
		if(isset($searchIn['webdisk']))
		{
			// files
			$thisResults = array();
			$res = $db->Query('SELECT ordner,dateiname FROM {pre}diskfiles WHERE user=? AND dateiname LIKE ' . $q . ' ORDER BY dateiname ASC',
				$thisUser->_id);
			while($row = $res->FetchArray(MYSQL_ASSOC))
				$thisResults[] = array(
					'icon'		=> 'webdisk_file',
					'title'		=> $row['dateiname'],
					'link'		=> sprintf('webdisk.php?folder=%d&', $row['ordner'])
				);
			$res->Free();
			
			// folders
			$res = $db->Query('SELECT id,titel FROM {pre}diskfolders WHERE user=? AND titel LIKE ' . $q . ' ORDER BY titel ASC',
				$thisUser->_id);
			while($row = $res->FetchArray(MYSQL_ASSOC))
				$thisResults[] = array(
					'icon'		=> 'webdisk_folder',
					'title'		=> $row['titel'],
					'link'		=> sprintf('webdisk.php?folder=%d&', $row['id'])
				);
			$res->Free();
			
			if(count($thisResults) > 0)
				$results[] = array(
					'title'		=> $lang_user['webdisk'],
					'results'	=> $thisResults
				);
		}
		
		//
		// web
		//
		if($bm_prefs['search_engine'] != '')
		{
			$results[] = array(
				'title'		=> $lang_user['websearch'],
				'results'	=>  array(
					array(
						'icon'		=> 'ico_download',
						'title'		=> $query,
						'extLink'	=> sprintf($bm_prefs['search_engine'], urlencode($query))
					)
				)
			);
		}
		
		// return
		return($results);
	}
	
	/**
	 * admin handler
	 *
	 */
	function AdminHandler()
	{
		global $tpl, $plugins, $lang_admin;
		
		if(!isset($_REQUEST['action']))
			$_REQUEST['action'] = 'prefs';
		
		$tabs = array(
			0 => array(
				'title'		=> $lang_admin['prefs'],
				'link'		=> $this->_adminLink() . '&',
				'active'	=> $_REQUEST['action'] == 'prefs'
			)
		);

		$tpl->assign('tabs', $tabs);
		
		if($_REQUEST['action'] == 'prefs')
			$this->_prefsPage();
	}
	
	/**
	 * admin prefs page
	 *
	 */
	function _prefsPage()
	{
		global $tpl, $db, $bm_prefs;
		
		// save?
		if(isset($_REQUEST['do']) && $_REQUEST['do'] == 'save')
		{
			if(isset($_REQUEST['searchIn']) && is_array($_REQUEST['searchIn']))
				$searchIn = $_REQUEST['searchIn'];
			else 
				$searchIn = array();
			$db->Query('UPDATE {pre}prefs SET search_in=?',
				serialize($searchIn));
			ReadConfig();
		}
		
		// unserialize
		$searchIn = @unserialize($bm_prefs['search_in']);
		if(!is_array($searchIn))
			$searchIn = array();
			
		// assign
		$tpl->assign('searchIn', $searchIn);
		$tpl->assign('pageURL', $this->_adminLink());
		$tpl->assign('page', $this->_templatePath('search.plugin.prefs.tpl'));
	}
}

/**
 * register plugin
 */
$plugins->registerPlugin('B1GMailSearchProvider');
?>