<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: common.inc.php,v 1.141 2008/12/19 13:24:47 patrick Exp $
 *
 */

/**
 * process store time row
 *
 * @param array $row Row
 */
function ProcessStoreTimeRow($row)
{
	global $db;
	
	$userID 	= $row['userID'];
	$mailIDs 	= explode(',', $row['mailIDs']);
	$freedSpace = $row['mailSizes'];
	
	foreach($mailIDs as $mailID)
	{
		$mailFileName = DataFilename($mailID, 'msg');
		if(file_exists($mailFileName))
			if(!@unlink($mailFileName))
				PutLog(sprintf('Failed to delete message file <%s> of message <%d>',
							$mailFileName,
							$mailID),
						PRIO_WARNING,
						__FILE__,
						__LINE__);
	}
	
	$db->Query('DELETE FROM {pre}mails WHERE `id` IN ?',
		$mailIDs);
	$db->Query('DELETE FROM {pre}certmails WHERE `mail` IN ?',
		$mailIDs);
	$db->Query('UPDATE {pre}users SET `mailspace_used`=`mailspace_used`-?,`mailbox_generation`=`mailbox_generation`+1 WHERE `id`=?',
		$freedSpace,
		$userID);
}

/**
 * enforce store times
 *
 * @return int Number of deleted mails
 */
function StoreTimeCron()
{
	global $db, $cacheManager;
	
	// user folders
	$res = $db->Query('SELECT GROUP_CONCAT({pre}mails.`id` SEPARATOR ?) AS mailIDs,SUM({pre}mails.`size`) AS mailSizes,{pre}mails.`userid` AS userID FROM {pre}mails,{pre}folders '
		. 'WHERE {pre}mails.folder>0 '
		. 'AND {pre}folders.id={pre}mails.folder '
		. 'AND {pre}folders.storetime>0 '
		. 'AND {pre}mails.datum<?-{pre}folders.storetime '
		. 'GROUP BY {pre}mails.`userid`',
		',',
		time());
	while($row = $res->FetchArray(MYSQL_ASSOC))
		ProcessStoreTimeRow($row);
	$res->Free();
	
	// system folders
	$res = $db->Query('SELECT `userid` AS userID,GROUP_CONCAT(`key` SEPARATOR ?) AS `keys`,GROUP_CONCAT(`value` SEPARATOR ?) AS `values` FROM {pre}userprefs WHERE `key` LIKE ? AND `value`>0 GROUP BY `userid`',
		',', ',',
		'storetime_%');
	while($row = $res->FetchArray(MYSQL_ASSOC))
	{
		$storeTimes = array();
		$keys = explode(',', $row['keys']);
		$values = explode(',', $row['values']);
		
		foreach($keys as $index=>$key)
			$storeTimes[(int)substr($key, 10)] = $values[$index];
		
		$cond = array();
		foreach($storeTimes as $folder=>$time)
			$cond[] = sprintf('(`folder`=%d AND `datum`<%d)', $folder, time()-$time);
		$cond = '(' . implode(' OR ', $cond) . ')';
		
		$res2 = $db->Query('SELECT `userid` AS userID,GROUP_CONCAT(`id` SEPARATOR ?) AS mailIDs,SUM(`size`) AS mailSizes,`userid` FROM {pre}mails '
			. 'WHERE ' . $cond . ' '
			. 'AND `userid`=? '
			. 'GROUP BY `userid`',
			',',
			$row['userID']);
		while($row2 = $res2->FetchArray(MYSQL_ASSOC))
			ProcessStoreTimeRow($row2);
		$res2->Free();
	}
	$res->Free();
}

/**
 * XOR-encrypt a string
 *
 * @param string $str String to encrypt
 * @param string $key Key, should be at least as long as $str
 * @return string Encrypted string
 */
function XORCrypt($str, $key)
{
	if(strlen($key) < strlen($str))
		$key = str_repeat($key, ceil(strlen($str)/strlen($key)));
	
	for($i=0; $i<strlen($str); $i++)
		$str[$i] = chr(ord($str[$i]) ^ ord($key[$i]));
	
	return($str);
}

/**
 * get b1gMail root certificates as hash => pemdata array
 *
 * @return array
 */
function GetRootCertificates()
{
	global $db;
	
	$certs = array();
	$res = $db->Query('SELECT `hash`,`pemdata` FROM {pre}certificates WHERE `userid`=? AND `type`=?',
		0,
		CERTIFICATE_TYPE_ROOT);
	while($row = $res->FetchArray(MYSQL_ASSOC))
		$certs[$row['hash']] = $row['pemdata'];
	$res->Free();
	
	return($certs);
}

/**
 * crypt private key passphrase
 *
 * @param string $str Input
 * @return string
 */
function CryptPKPassPhrase($str)
{
	return(XORCrypt($str, md5(time())));
}

/**
 * format a string for html output
 *
 * @param string $in
 * @return string
 */
function HTMLFormat($in)
{
	global $currentCharset;
	
	if(PHPNumVersion() >= 523)
	{
		$in = @htmlspecialchars($in, ENT_COMPAT, $currentCharset, false);
	}
	else
	{
		$in = @htmlspecialchars($in, ENT_COMPAT, $currentCharset);
		$in = preg_replace('/\&amp;#([x0-9]*);/', '&#$1;', $in);
	}
	
	return($in);
}

/**
 * generate mail trust token
 *
 * @param string $messageID
 * @param string $from
 * @param string $to
 * @param string $subject
 * @return string
 */
function GenerateTrustToken($messageID, $from, $to, $subject)
{
	global $bm_prefs;
	
	return(sprintf('<%s@%s>',
		md5(sprintf('%s:%s:%s:%s',
			$messageID,
			EncodeMailHeaderField($from),
			EncodeMailHeaderField($to),
			EncodeMailHeaderField($subject))),
		$bm_prefs['b1gmta_host']));
}

/**
 * delete an user and associated data
 *
 * @param int $userID
 */
function DeleteUser($userID)
{
	global $db;
	
	// get mail address
	$res = $db->Query('SELECT email FROM {pre}users WHERE id=?',
		$userID);
	if($res->RowCount() == 0)
		return(false);
	list($userMail) = $res->FetchArray(MYSQL_NUM);
	$res->Free();
	
	// module handler
	ModuleFunction('OnDeleteUser', array($userID));
	
	// delete group<->member associations + groups
	$groupIDs = array(); 
	$res = $db->Query('SELECT id FROM {pre}adressen_gruppen WHERE user=?',
		$userID);
	while($row = $res->FetchArray(MYSQL_ASSOC))
		$groupIDs[] = $row['id'];
	$res->Free();
	if(count($groupIDs) > 0)
	{
		$db->Query('DELETE FROM {pre}adressen_gruppen_member WHERE gruppe IN(' . implode(',', $groupIDs) . ')');
		$db->Query('DELETE FROM {pre}adressen_gruppen WHERE user=?',
			$userID);
	}
	
	// delete addresses
	$db->Query('DELETE FROM {pre}adressen WHERE user=?',
		$userID);
	
	// delete aliases
	$db->Query('DELETE FROM {pre}aliase WHERE user=?',
		$userID);
	
	// delete autoresponder
	$db->Query('DELETE FROM {pre}autoresponder WHERE userid=?',
		$userID);
	
	// delete calendar dates
	$dateIDs = array();
	$res = $db->Query('SELECT id FROM {pre}dates WHERE user=?',
		$userID);
	while($row = $res->FetchArray(MYSQL_ASSOC))
		$dateIDs[] = $row['id'];
	$res->Free();
	if(count($dateIDs) > 0)
	{
		$db->Query('DELETE FROM {pre}date_attendees WHERE date IN(' . implode(',', $dateIDs) . ')');
		$db->Query('DELETE FROM {pre}dates WHERE user=?',
			$userID);
	}
	
	// delete calendar groups
	$db->Query('DELETE FROM {pre}dates_groups WHERE user=?',
		$userID);
	
	// delete disk props
	$db->Query('DELETE FROM {pre}diskprops WHERE user=?',
		$userID);
		
	// delete disk locks
	$db->Query('DELETE FROM {pre}disklocks WHERE user=?',
		$userID);
		
	// delete disk folders
	$db->Query('DELETE FROM {pre}diskfolders WHERE user=?',
		$userID);
	
	// delete disk files
	$res = $db->Query('SELECT id FROM {pre}diskfiles WHERE user=?',
		$userID);
	while($row = $res->FetchArray(MYSQL_ASSOC))
	{
		$fileName = DataFilename($row['id'], 'dsk');
		if(file_exists($fileName))
			@unlink($fileName);
	}
	$res->Free();
	
	// delete cert mails
	$db->Query('DELETE FROM {pre}certmails WHERE user=?',
		$userID);
	
	// delete filters
	$filterIDs = array();
	$res = $db->Query('SELECT id FROM {pre}filter WHERE userid=?',
		$userID);
	while($row = $res->FetchArray(MYSQL_ASSOC))
		$filterIDs[] = $row['id'];
	$res->Free();
	if(count($filterIDs) > 0)
	{
		$db->Query('DELETE FROM {pre}filter_actions WHERE filter IN(' . implode(',', $filterIDs) . ')');
		$db->Query('DELETE FROM {pre}filter_conditions WHERE filter IN(' . implode(',', $filterIDs) . ')');
		$db->Query('DELETE FROM {pre}filter WHERE userid=?',
			$userID);
	}
	
	// delete folder conditions + folders
	$folderIDs = array();
	$res = $db->Query('SELECT id FROM {pre}folders WHERE userid=?',
		$userID);
	while($row = $res->FetchArray(MYSQL_ASSOC))
		$folderIDs[] = $row['id'];
	$res->Free();
	if(count($folderIDs) > 0)
	{
		$db->Query('DELETE FROM {pre}folder_conditions WHERE folder IN(' . implode(',', $folderIDs) . ')');
		$db->Query('DELETE FROM {pre}folders WHERE userid=?',
			$userID);
	}
	
	// delete mails
	$res = $db->Query('SELECT id FROM {pre}mails WHERE userid=? AND body=?',
		$userID,
		'file');
	while($row = $res->FetchArray(MYSQL_ASSOC))
	{
		$fileName = DataFilename($row['id']);
		if(file_exists($fileName))
			@unlink($fileName);
	}
	$res->Free();
	$db->Query('DELETE FROM {pre}mails WHERE userid=?',
		$userID);
	
	// delete notes
	$db->Query('DELETE FROM {pre}notes WHERE user=?',
		$userID);
	
	// uid index + ext. pop3s
	$pop3IDs = array();
	$res = $db->Query('SELECT id FROM {pre}pop3 WHERE user=?',
		$userID);
	while($row = $res->FetchArray(MYSQL_ASSOC))
		$pop3IDs[] = $row['id'];
	$res->Free();
	if(count($pop3IDs) > 0)
	{
		$db->Query('DELETE FROM {pre}uidindex WHERE pop3 IN(' . implode(',', $pop3IDs) . ')');
		$db->Query('DELETE FROM {pre}pop3 WHERE user=?',
			$userID);
	}
		
	// sigs
	$db->Query('DELETE FROM {pre}signaturen WHERE user=?',
		$userID);
	
	// sent sms
	$db->Query('DELETE FROM {pre}smsend WHERE user=?',
		$userID);
	
	// spam index
	$db->Query('DELETE FROM {pre}spamindex WHERE userid=?',
		$userID);
	
	// tasks
	$db->Query('DELETE FROM {pre}tasks WHERE user=?',
		$userID);
		
	// workgroup memberships
	$db->Query('DELETE FROM {pre}workgroups_member WHERE user=?',
		$userID);
	
	// user prefs
	$db->Query('DELETE FROM {pre}userprefs WHERE userid=?',
		$userID);
	
	// finally, the user record itself
	$db->Query('DELETE FROM {pre}users WHERE id=?',
		$userID);
		
	// log
	PutLog(sprintf('User <%s> (%d) deleted',
		$userMail,
		$userID),
		PRIO_NOTE,
		__FILE__,
		__LINE__);
		
	return(true);
}

/**
 * generate session token
 *
 * @return string
 */
function SessionToken()
{
	global $bm_prefs;
	
	$token = $_SERVER['HTTP_USER_AGENT'];
	if($bm_prefs['ip_lock'] == 'yes')
		$token .= $_SERVER['REMOTE_ADDR'];
	if($bm_prefs['cookie_lock'] == 'yes')
		if(isset($_COOKIE['sessionSecret_' . session_id()]))
			$token .= $_COOKIE['sessionSecret_' . session_id()];
		else if(strpos(strtolower($_SERVER['PHP_SELF']), 'webdisk.php') !== false
			&& isset($_POST['key']))
			$token .= $_POST['key'];
	return(md5($token));
}

/**
 * decode HTML entities
 *
 * @param string $text Input
 * @return string
 */
function DecodeHTMLEntities($text)
{
	if(PHPNumVersion() >= 430 && function_exists('html_entity_decode'))
		$text = html_entity_decode($text);
	else 
	{
		$text = preg_replace('~&#x([0-9a-f]+);~ei', 'chr(hexdec("\\1"))', $text);
		$text = preg_replace('~&#([0-9]+);~e', 'chr("\\1")', $text);
		$transTable = get_html_translation_table(HTML_ENTITIES);
		$transTable = array_flip($transTable);
		$text = strtr($text, $transTable);
	}
	
	return($text);
}

/**
 * convert HTML text to plain text
 *
 * @param string $str Input
 * @return string
 */
function htmlToText($str)
{
	$str = str_replace(array("\r", "\n"), '', $str);
	$str = preg_replace('/\<p([^\>]*)\>/i', "\r\n", $str);
	$str = preg_replace('/\<br([^\>]*)\>/i', "\r\n", $str);
	$str = strip_tags(str_replace('>', '> ', $str));
	$str = DecodeHTMLEntities($str);
	return($str);
}

/**
 * get all mail service domains
 *
 * @param bool $includeGroupDomains Include group domains?
 * @return array
 */
function MyDomains($includeGroupDomains = true)
{
	global $bm_prefs;
	
	if($includeGroupDomains)
		return(array_merge(explode(':', strtolower($bm_prefs['domains'])), BMGroup::GetGroupDomains()));
	else
		return(explode(':', strtolower($bm_prefs['domains'])));
}

/**
 * check POP3 login
 *
 * @param string $host
 * @param int $port
 * @param string $user
 * @param string $pass
 * @return bool
 */
function CheckPOP3Login($host, $port, $user, $pass)
{
	$ok = false;
	
	if(!class_exists('BMPOP3'))
		include(B1GMAIL_DIR . 'serverlib/pop3.class.php');

	$pop3 = _new('BMPOP3', array($host, $port));
	if($pop3->Connect())
	{
		if($pop3->Login($user, $pass))
		{
			$ok = true;
		}
		$pop3->Disconnect();
	}
	
	return($ok);
}

/**
 * get microtime as float value
 *
 * @return float
 */
function microtime_float()
{
    list($usec, $sec) = explode(' ', microtime());
    return((float)$usec + (float)$sec);
}

/**
 * get postmaster mail address
 *
 * @return string
 */
function GetPostmasterMail()
{
	global $bm_prefs;

	list($firstDomain) = explode(':', $bm_prefs['domains']);
	$mail = 'postmaster@' . $firstDomain;
	
	return($mail);
}

/**
 * build signature string
 *
 * @param string $mode Mode (text/html)
 * @return string
 */
function GetSigStr($mode = 'text')
{
	global $groupRow;
	
	$sigStr = '';
	if(trim($groupRow['signatur']) != '')
	{
		if($mode == 'html')
		{
			$sigStr = "\r\n<br /><br />\r\n"
						. str_repeat(SIGNATURE_LINE_CHAR, SIGNATURE_LINE_LENGTH)
						. "<br />\r\n"
						. nl2br(htmlentities($groupRow['signatur']));
		}
		else if($mode == 'text')
		{
			$sigStr = "\r\n\r\n"
						. str_repeat(SIGNATURE_LINE_CHAR, SIGNATURE_LINE_LENGTH)
						. "\r\n"
						. $groupRow['signatur'];
		}
	}
	
	return($sigStr);
}
	
/**
 * generate a random secret key
 *
 * @param string $str Some prefix string/salt
 * @return string
 */
function GenerateRandomKey($str)
{
	return(md5(microtime().(function_exists('posix_getpid') ? posix_getpid() : mt_rand(0, 1000)).uniqid($str)));
}

/**
 * synchronize DB structure against an DB structure array
 *
 * @param array $databaseStructure (New/correct) DB structure
 * @return array
 */
function SyncDBStruct($databaseStructure)
{
	global $db;
	
	// queries to execute
	$syncQueries = array();
	
	// get tables
	$defaultTables = array();
	$res = $db->Query('SHOW TABLES');
	while($row = $res->FetchArray(MYSQL_NUM))
		$myTables[] = $row[0];
	$res->Free();
		
	// compare tables
	foreach($databaseStructure as $tableName=>$tableInfo)
	{
		$tableFields = $tableInfo['fields'];
		$tableIndexes = $tableInfo['indexes'];
		
		//
		// table exists => compare fields and indexes
		//
		if(in_array($tableName, $myTables))
		{
			// get my fields
			$myFields = array();
			$res = $db->Query('SHOW FIELDS FROM ' . $tableName);
			while($row = $res->FetchArray(MYSQL_ASSOC))
			{
				if($row['Null'] == '') $row['Null'] = 'NO';
				$myFields[$row['Field']] = array($row['Field'], $row['Type'], $row['Null'], $row['Key'], $row['Default'], $row['Extra']);
			}
			$res->Free();
			
			// get my indexes
			$myIndexes = array();
			$res = $db->Query('SHOW INDEX FROM ' . $tableName);
			while($row = $res->FetchArray(MYSQL_ASSOC))
				if(isset($myIndexes[$row['Key_name']]))
					$myIndexes[$row['Key_name']][] = $row['Column_name'];
				else 
					$myIndexes[$row['Key_name']] = array($row['Column_name']);
			$res->Free();
			
			// compare fields
			foreach($tableFields as $field)
			{
				$op = false;
				
				if(!isset($myFields[$field[0]]))
				{
					$op = 'ADD';
				}
				else
				{
					$myField = $myFields[$field[0]];
					if($myField[1] != $field[1]
						|| $myField[2] != $field[2]
						|| ($myField[4] != $field[4] && !(($myField[4]==0 && $field[4]=='') || ($myField[4]=='' && $field[4]==0)))
						|| $myField[5] != $field[5])
					{
						$op = 'MODIFY';
					}
				}
				
				if($op !== false)
				{
					$syncQueries[] = sprintf('ALTER TABLE %s %s `%s` %s%s%s%s',
						$tableName,
						$op,
						$field[0],
						$field[1],
						$field[2] == 'NO' ? ' NOT NULL' : '',
						$field[4] == 'NULL' ? ' DEFAULT NULL' : ($field[4] != ''
							? (is_numeric($field[4]) 
									? ' DEFAULT ' . $field[4]
									: ' DEFAULT \'' . mysql_escape_string($field[4]) . '\'')
							: ''),
						$field[5] != '' ? ' ' . $field[5] : '');
				}
			}
			
			// compare indexes
			foreach($tableIndexes as $indexName=>$indexFields)
			{
				// keys
				if($indexName != 'PRIMARY')
				{
					$op = false;
					
					if(!isset($myIndexes[$indexName]))
					{
						$op = true;
					}
					else if($myIndexes[$indexName] != $indexFields)
					{
						$op = true;
						$syncQueries[] = sprintf('ALTER TABLE %s DROP KEY `%s`',
							$tableName,
							$indexName);
					}
					
					if($op)
					{
						$syncQueries[] = sprintf('ALTER TABLE %s ADD KEY `%s`(%s)',
							$tableName,
							$indexName,
							'`' . implode('`,`', $indexFields) . '`');
					}
				}
				
				// primary keys
				else 
				{
					if(!isset($myIndexes[$indexName]))
					{
						// add
						$syncQueries[] = sprintf('ALTER TABLE %s ADD PRIMARY KEY(%s)',
							$tableName,
							'`' . implode('`,`', $indexFields) . '`');
					}
					else if($myIndexes[$indexName] != $indexFields)
					{
						// drop, add
						$syncQueries[] = sprintf('ALTER TABLE %s DROP PRIMARY KEY, ADD PRIMARY KEY(%s)',
							$tableName,
							'`' . implode('`,`', $indexFields) . '`');
					}
				}
			}
		}
		
		//
		// table does not exist => create
		//
		else 
		{
			$stmt = sprintf('CREATE TABLE %s(' . "\n",
				$tableName);
			
			// fields
			foreach($tableFields as $field)
			{
				$stmt .= sprintf(' `%s` %s%s%s%s,' . "\n",
					$field[0],
					$field[1],
					$field[2] == 'NO' ? ' NOT NULL' : '',
					$field[4] == 'NULL' ? ' DEFAULT NULL' : ($field[4] != ''
						? (is_numeric($field[4]) 
								? ' DEFAULT ' . $field[4]
								: ' DEFAULT \'' . mysql_escape_string($field[4]) . '\'')
						: ''),
					$field[5] != '' ? ' ' . $field[5] : '');
			}
			
			// indexes
			foreach($tableIndexes as $indexName=>$indexFields)
			{
				if($indexName == 'PRIMARY')
					$stmt .= sprintf(' PRIMARY KEY (%s),' . "\n",
						'`' . implode('`,`', $indexFields) . '`');
				else 
					$stmt .= sprintf(' KEY `%s`(%s),' . "\n",
						$indexName,
						'`' . implode('`,`', $indexFields) . '`');
			}
			
			$stmt = substr($stmt, 0, -2) . "\n" . ')';
				
			$syncQueries[] = $stmt;
		}
	}
	
	// execute queries
	$result = array();
	foreach($syncQueries as $query)
		if($db->Query($query))
			$result[$query] = true;
		else 
			$result[$query] = false;
		
	// return
	return($result);
}

/**
 * send system mail
 *
 * @param string $from Sender
 * @param string $to Recipient
 * @param string $subject Subject
 * @param string $templateName Name of e-mail template (from lang_custom)
 * @param array $vars Variables (key => value)
 * @param int $forUser User ID for localized phrases
 * @return bool
 */
function SystemMail($from, $to, $subject, $templateName, $vars, $forUser = -1)
{
	global $lang_custom, $currentCharset;
	
	// load class, if needed
	if(!class_exists('BMMailBuilder'))
		include(B1GMAIL_DIR . 'serverlib/mailbuilder.class.php');
	
	// create text
	if($forUser != -1)
		$text = GetPhraseForUser($forUser, 'lang_custom', $templateName);
	else
		$text = $lang_custom[$templateName];
	foreach($vars as $key=>$val)
	{
		$text 		= str_replace('%%'.$key.'%%', $val, $text);
		$subject 	= str_replace('%%'.$key.'%%', $val, $subject);
	}
	
	// create mail
	$mail = _new('BMMailBuilder', array(true));
	$mail->AddHeaderField('From',		$from);
	$mail->AddHeaderField('To',			$to);
	$mail->AddHeaderField('Subject',	$subject);
	$mail->AddText($text, 'plain', $currentCharset);
	$result = $mail->Send() !== false;
	$mail->CleanUp();
	
	// stats, log
	if($result)
	{
		Add2Stat('sysmail');
		PutLog(sprintf('Sent system mail <%s> from <%s> to <%s>',
			$templateName,
			ExtractMailAddress($from),
			ExtractMailAddress($to)),
			PRIO_NOTE,
			__FILE__,
			__LINE__);
	}
	else
	{
		PutLog(sprintf('Failed to send system mail <%s> from <%s> to <%s>',
			$templateName,
			ExtractMailAddress($from),
			ExtractMailAddress($to)),
			PRIO_WARNING,
			__FILE__,
			__LINE__);		
	}
	
	// return
	return($result);
}

/**
 * get a phrase of user's language file
 *
 * @param int $userID User ID
 * @param string $language Language
 * @param string $var Phrase pool (lang_main, lang_admin, lang_client, lang_custom)
 * @param string $phrase Phrase key
 * @return string
 */
function GetPhraseForUser($userID, $var, $phrase)
{
	global $db, $bm_prefs;
	
	// language?
	$res = $db->Query('SELECT language FROM {pre}users WHERE id=?',
		(int)$userID);
	list($language) = $res->FetchArray(MYSQL_NUM);
	$res->Free();
	if(trim($language) == '')
		$language = $bm_prefs['language'];
	
	// return phrase
	return(GetPhraseForLanguage($language, $var, $phrase));
}

/**
 * get phrase of a certain language file
 *
 * @param string $language Language
 * @param string $var Phrase pool (lang_main, lang_admin, lang_client, lang_custom)
 * @param string $phrase Phrase key
 * @return string
 */
function GetPhraseForLanguage($language, $var, $phrase)
{
	global $bm_prefs, $cacheManager;
	
	// already loaded?
	if($bm_prefs['language'] == $language)
		return($GLOBALS[$var][$phrase]);
	
	// avail languages?
	$availableLanguages = GetAvailableLanguages();
	if(isset($availableLanguages[$language]))
	{
		if($var == 'lang_custom' && ($lang_custom = $cacheManager->Get('langCustom:' . $language)))
			return($lang_custom[$phrase]);
		
		$lang_custom = $lang_client = $lang_user = $lang_admin = array();
		@include(B1GMAIL_DIR . 'languages/' . $language . '.lang.php');
		
		if(!MAINTENANCE_MODE)
		{
			// module handler
			ModuleFunction('OnReadLang', array(&$lang_user, &$lang_client, &$lang_custom, &$lang_admin, $language));
		}
		
		return(${$var}[$phrase]);
	}
	
	// not found
	return('#UNKNOWN_PHRASE(' . $phrase . ')#');
}

/**
 * update statistics
 *
 * @param string $type
 * @param int $count
 */
function Add2Stat($type, $count = 1)
{
	global $db;
	
	$d = date('d');
	$m = date('m');
	$y = date('Y');
	$id = 0;
	
	// is there already a row for this?
	$res = $db->Query('SELECT id FROM {pre}stats WHERE d=? AND m=? AND y=? AND typ=? LIMIT 1',
		$d,
		$m,
		$y,
		$type);
	if($res->RowCount() == 1)
	{
		list($id) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
	}
	
	// update existing row...
	if($id != 0)
	{
		$db->Query('UPDATE {pre}stats SET anzahl=anzahl+'.(int)$count.' WHERE id=?',
			$id);
	}
	
	// ...or insert new row
	else
	{
		$db->Query('INSERT INTO {pre}stats(d,m,y,typ,anzahl) VALUES (?,?,?,?,?)',
			$d,
			$m,
			$y,
			$type,
			$count);
	}
}

/**
 * fgets wrapper
 *
 * @param resource $handle
 * @param int $length
 * @return string
 */
function fgets2($handle, $length = 40960)
{
	if(PHPNumVersion() >= 430)
		return(fgets($handle));
	else 
		return(fgets($handle, $length));
}

/**
 * validate mail address against MX server
 *
 * @param string $address Address
 */
function ValidateMailAddress($address)
{
	global $bm_prefs;
	
	list($userName, $hostName) = explode('@', $address);
	$errNo = $errStr = '';
	$mailFrom = GetPostmasterMail();
	
	// windows does not support this
	if(!function_exists('getmxrr'))
		return(true);
	
	// get mx recors
	$mxHosts = array();
	$mxHosts[] = $hostName;
	@getmxrr($hostName, $mxHosts);
	
	// check against MX servers
	foreach($mxHosts as $host)
	{
		$ok = $break = false;
		
		$sock = @fsockopen($host, 25, $errNo, $errStr, SOCKET_TIMEOUT);
		if($sock && is_resource($sock))
		{
			$result = @fgets2($sock);
			if(substr($result, 0, 3) == '220'
				&& @fwrite($sock, sprintf('HELO <%s>', $bm_prefs['b1gmta_host']) . "\r\n")
				&& substr(fgets2($sock), 0, 3) == '250')
			{
				// VRFY is the more nice way, but not always supported
				if(@fwrite($sock, sprintf('VRFY <%s>', $address) . "\r\n"))
				{
					$result = @fgets2($sock);
					
					// VRFY worked
					if(substr($result, 0, 3) == '250')
					{
						$ok = true;
					}
					
					// VRFY did not work, try the RCPT-TO-way
					else if(substr($result, 0, 2) != '55')
					{
						if(@fwrite($sock, sprintf('MAIL FROM:<%s>', $mailFrom) . "\r\n"))
						{
							$result = @fgets2($sock);
							if(substr($result, 0, 3) == '250')
							{
								if(@fwrite($sock, sprintf('RCPT TO:<%s>', $address) . "\r\n"))
								{
									$result = @fgets2($sock);
									if(substr($result, 0, 3) == '250')
										$ok = true;
									$break = true;
								}
							}
						}
					}
				}
				
				if(@fwrite($sock, 'QUIT' . "\r\n"))
					$result = @fgets2($sock);
				@fclose($sock);
			}
		}
		
		if($ok)
			return(true);
		else if($break)
			return(false);
	}
	
	return(false);
}

/**
 * query helper for lazy people ;)
 *
 * @param string $table Table (without prefix)
 * @return string
 */
function giveMeAnUpdateStmtFor($table)
{
	global $db;
	
	$fields = array();
	$res = $db->Query('SHOW FIELDS FROM {pre}'.$table);
	while($row = $res->FetchArray(MYSQL_ASSOC))
	{
		$fields[$row['Field']] = $row;
	}
	$res->Free();
	
	$stmt = 'UPDATE {pre}'.$table.' SET';
	$paramList = '';
	
	foreach($_REQUEST as $key=>$val)
	{
		if(isset($fields[$key]) && $fields[$key]['Key'] != 'PRI')
		{
			$stmt .= ' '.$key.'=?,';
			$paramList .= ",\n\t\$_REQUEST['$key']";
		}
	}
	
	$stmt = substr($stmt, 0, -1);
	
	foreach($fields as $key=>$field)
		if($field['Key'] == 'PRI')
		{
			$stmt .= ' WHERE '.$key.'=?';
			$paramList .= ",\n\t\$_REQUEST['$key']";
		}
	
	$stmt = sprintf('$db->Query(\'%s\'%s);',
		$stmt,
		$paramList);
	
	return($stmt);
}

/**
 * class factory
 *
 * @param string $class Class name
 * @param array $args Arguments, if any (as an array)
 * @return Object
 */
function _new($class, $args = array())
{
	global $plugins;
		
	// overridden?
	$origClass = $class;
	if(isset($plugins) && is_object($plugins))
		foreach($plugins->_plugins as $key=>$value)
			if(($replacement = $plugins->callFunction('getClassReplacement', $key, false, array($class))) !== false)
			{
				$replacingModule = $key;
				$class = $replacement;
				break;
			}
			
	// check if class exists
	if(!class_exists($class))
	{
		DisplayError(isset($replacingModule) ?  0x13 : 0x12, 'Class not found',
			'The requested class cannot be found.' . (isset($replacingModule) ? ' An installed plugin seems to be faulty.' : ''),
			isset($replacingModule)
				? sprintf("Original class name:\n%s\n\nOverridden class name:\n%s\n\nOverridden by plugin:\n%s", $origClass, $class, $replacingModule)
				: sprintf("Class name:\n%s", $class),
			__FILE__,
			__LINE__);
		die();
	}
	
	// build arg string
	$argStr = '';
	for($key=0; $key<count($args); $key++)
		$argStr .= '$args[' . $key . '],';
	
	// call!
	return eval('return new ' . $class . '(' . substr($argStr, 0, -1) . ');');
}

/**
 * get day of week (1-7)
 *
 * @return int
 */
function RealW()
{
	$a = date('w');
	if($a==0) {
		$a = 7;
	}
	return($a);
}

/**
 * put a given date into a date category
 *
 * @param int $date Date (timestamp)
 * @return array
 */
function categorizeDate($date)
{
	global $lang_user;
	
	// generate date cats
	$ts = array();
	$ts[0] = array(
		'from' 	 => mktime(0,0,0,date('m'),date('d'),date('Y')),
		'to'	 => mktime(24,0,0,date('m'),date('d'),date('Y')),
		'text'	 => $lang_user['today'],
		'date'   => mktime(0,0,0,date('m'),date('d'),date('Y'))
	);
	$last = mktime(0,0,0,date('m'),date('d'),date('Y'));
	for($i=1; $i<RealW(); $i++)
	{
		$a = RealW() - $i;
		$last -= 86400;
		$ts[$a] = array(
			'from' 	=> $last,
			'to'	=> $last+86400,
			'text' 	=> strftime('%A', $last),
			'date' 	=> $last
		);
	}
	$ts[-2] = array(
		'from'	=> $last-(7 * 86400),
		'to'	=> $last,
		'text'	=> $lang_user['lastweek'],
		'date'	=> -1
	);
	$last -= 7 * 86400;
	$ts[-1] = array(
		'from'	=> -1,
		'to'	=> $last,
		'text'	=> $lang_user['later'],
		'date'	=> -1
	);
	
	// where do we fit in?
	$myCat = -1;
	foreach($ts as $key=>$value)
		if($date >= $value['from'] && $date <= $value['to'])
			$myCat = $key;
	
	// return
	return(array($myCat, $ts[$myCat]));
}

/**
 * format compose text (reply, redirect, forward)
 *
 * @param string $text Text
 * @param string $textMode Text mode (html, text)
 * @param string $action Action (reply, redirect, forward)
 * @param BMMail $mail Mail object
 * @return string
 */
function formatComposeText($text, $textMode, $action, &$mail)
{
	global $lang_user;

	if($textMode == 'html')
	{
		if($action == 'redirect')
		{
			$result = $text;
		}
		else 
		{
			$result = sprintf('<font face="arial" size="2"><br />'
								. '<blockquote style="border-left: 2px solid rgb(0, 0, 0); padding-right: 0px; padding-left: 5px; margin-left: 5px; margin-right: 0px;" dir="ltr">'
								. '--- %s ---<br />'
								. '<b>%s:</b> %s<br />'
								. '<b>%s:</b> %s<br />'
								. '<b>%s:</b> %s<br /><br />',
						$lang_user['srcmsg'],
						$lang_user['from'], 	htmlentities(htmlentities($mail->GetHeaderValue('from'))),
						$lang_user['to'], 		htmlentities(htmlentities($mail->GetHeaderValue('to'))),
						$lang_user['subject'],  htmlentities(htmlentities($mail->GetHeaderValue('subject'))))
						. $text
						. '</blockquote></font>';
		}
	}
	else 
	{
		if($action == 'redirect')
		{
			$result = $text;
		}
		else 
		{
			$result = sprintf("\n\n"
								. '--- %s ---' . "\n"
								. '%s: %s' . "\n"
								. '%s: %s' . "\n"
								. '%s: %s' . "\n"
								. "\n",
						$lang_user['srcmsg'],
						$lang_user['from'], 	$mail->GetHeaderValue('from'),
						$lang_user['to'], 		$mail->GetHeaderValue('to'),
						$lang_user['subject'], 	$mail->GetHeaderValue('subject'))
						. preg_replace('/^/m', '> ', wordwrap($text, 72, "\r\n"));
		}
	}
	
	return(isset($result) ? $result : '');
}

/**
 * format html e-mail text
 *
 * @param string $in Input
 * @param bool $showExternal Enable external objects and scripts?
 * @param array $attachments Attachment list (for CID replacement)
 * @param int $mailID Mail ID
 * @return string
 */
function formatEMailHTMLText($in, $showExternal = false, $attachments = array(), $mailID = -1)
{
	if(!$showExternal)
	{
		$in = eregi_replace("(src|href|background)=([\"']*)([h])", "blocked=\\2\\3", $in);
		$in = eregi_replace("<([\/]{0,1})(object|script|embed)([^>]*)>", "<\\1null>", $in);
	}
	else 
	{
		$in = eregi_replace("href=\"([a-zA-Z]{3,6})://", 'target="_blank" href="deref.php?\\1://', $in);	
	}
	
	$in = eregi_replace("href=\"mailto\:([a-zA-Z0-9\.\_-]*\@[a-zA-Z0-9\.\_-]*\.[a-zA-Z0-9\.\_-]*)([\?]*)", 'target="_top" href="email.compose.php?sid=' . session_id() . '&to=\\1&', $in);
	
	if(count($attachments) > 0)
	{
		foreach($attachments as $key=>$value)
		{
			if(isset($value['cid']))
			{
				$cid = trim(str_replace(array('<', '>'), '', $value['cid']));
				if($cid != '')
				{
					$in = str_replace('cid:' . $cid,
										'email.read.php?action=downloadAttachment&view=true&id=' . $mailID . '&attachment=' . $key . '&sid=' . session_id(),
										$in);
				}
			}
		}
	}
	
	return($in);
}

/**
 * format e-mail text
 *
 * @param string $in Input
 * @param bool $html For HTML output?
 * @return string
 */
function formatEMailText($in, $html = true)
{
	// html entities
	$in = HTMLFormat($in);
	
	// for HTML output?
	if($html)
	{
		$in = nl2br($in);
		
		// tabs
		$in = str_replace("\t", '&nbsp;&nbsp;&nbsp;&nbsp;', $in);
		
		// e-mail addresses
		$in = ereg_replace("[a-zA-Z0-9\.\_-]*\@[a-zA-Z0-9\.\_-]*\.[a-zA-Z0-9\.\_-]*", '<a target="_top" href="email.compose.php?to=\0&sid=' . session_id() . '">\\0</a>', $in);
		
		// links
		$in = ereg_replace("[[:alpha:]]+://[^<>[:space:]]+[[:alnum:]/=?&@%]", '<a href="deref.php?\\0" target="_blank">\\0</a>', $in);
	}
	
	return($in);
}

/**
 * array_slice with preserve_keys for every php version
 *
 * @param array $array Input array
 * @param int $offset Start offset
 * @param int $length Length
 * @return array
 */
function array_slice_preserve_keys($array, $offset, $length = null)
{
	// PHP >= 5.0.2 is able to do this itself
	if(PHPNumVersion() >= 502)
		return(array_slice($array, $offset, $length, true));
	
	// prepare input variables
	if($offset < 0)
		$offset = count($array) + $offset;
	if($length > 0)
		$endOffset = $offset + $length;
	else if($length < 0)
		$endOffset = count($array) + $length;
	else
		$endOffset = count($array);
	
	// collect elements
	$result = array();
	$i = 0;
	foreach($array as $key=>$value)
	{
		if($i >= $offset && $i < $endOffset)
			$result[$key] = $value;
		$i++;
	}
	
	// return
	return($result);
}

/**
 * return numeric php version
 *
 * @return int
 */
function PHPNumVersion()
{
	$ver = phpversion();
	if(strlen($ver) > strlen('0.0.0'))
		$ver = substr($ver, 0, strlen('0.0.0'));
	return((int)str_replace('.', '', $ver));
}

/**
 * get full file contents
 *
 * @param string $fileName Filename
 * @return string
 */
function getFileContents($fileName)
{
	// we should not read too big files at once
	if(@filesize($fileName) > 1024*500)
		PutLog(sprintf('Getting whole file contents of big file <%s> (%.02f KB > 500 KB)',
						$fileName,
						round(filesize($fileName)/1024, 2)),
				PRIO_DEBUG,
				__FILE__,
				__LINE__);
	
	// read contents
	$fp = @fopen($fileName, 'rb');
	if(!$fp)
		return(false);
	$contents = '';
	while(!feof($fp))
		$contents .= fread($fp, 8096);
	fclose($fp);
	
	// return contents
	return($contents);
}

/**
 * get/copy a file uploaded using the local/webdisk upload widget
 *
 * @param string $fieldName Field name
 * @param string $destinationFile Destination path
 * @return array
 */
function getUploadedFile($fieldName, $destinationFile)
{
	global $userRow;
	
	// user row available?
	if(!isset($userRow) || !is_array($userRow))
		return(false);
	
	// get file vars
	$localFile = $_FILES['localFile_' . $fieldName];
	$webdiskFile = (int)$_REQUEST['webdiskFile_' . $fieldName . '_id'];
	
	// file uploaded
	if(isset($localFile) && is_array($localFile) && $localFile['error'] == UPLOAD_ERR_OK)
	{
		if(@move_uploaded_file($localFile['tmp_name'], $destinationFile))
		{
			return($localFile);
		}
		else 
		{
			return(false);
		}
	}
	
	// file from webdisk
	else if($webdiskFile > 0)
	{
		if(!class_exists('BMWeddisk'))
			include(B1GMAIL_DIR . 'serverlib/webdisk.class.php');
		$webdisk = _new('BMWebdisk', array($userRow['id']));
		$file = $webdisk->GetFileInfo($webdiskFile);
		
		if($file && @copy(DataFilename($webdiskFile, 'dsk'), $destinationFile))
		{
			$resultArray = array(
				'name'		=> $file['dateiname'],
				'type'		=> $file['contenttype'],
				'size'		=> $file['size'],
				'tmp_name'	=> DataFilename($webdiskFile, 'dsk'),
				'error'		=> 0
			);
			return($resultArray);
		}
		else 
		{
			return(false);
		}
	}
	
	// no file
	else 
	{
		return(false);
	}
}

/**
 * check if $s is equal to $in or $s is in $in
 *
 * @param mixed $in Array or string to check againt
 * @param string $s String
 * @return bool
 */
function eqOrIn($in, $s)
{
	return((!is_array($in) && $in == $s)
			|| (is_array($in) && in_array($s, $in)));
}

/**
 * strip slashes for a whole array (recursively)
 *
 * @param array $in Input array
 * @return array
 */
function stripslashes_array($in)
{
	foreach($in as $key=>$val)
	{
		if(is_array($val))
			$in[$key] = stripslashes_array($val);
		else 
			$in[$key] = stripslashes($val);
	}
	return($in);
}

/**
 * prepare input vars (turn magic_quotes_gpc off, if needed)
 *
 */
function PrepareInputVars()
{
	if(get_magic_quotes_gpc())
	{
		$_GET = stripslashes_array($_GET);				// G
		$_POST = stripslashes_array($_POST);			// P
		$_COOKIE = stripslashes_array($_COOKIE);		// C
		$_REQUEST = stripslashes_array($_REQUEST);		// GPC
	}
}

/**
 * get cellphone number from smarty cellphone number control
 *
 * @param string $field Field name (prefix)
 * @return string
 */
function SmartyCellphoneNo($field)
{
	if(isset($_REQUEST[$field.'_pre']))
	{
		if(trim($_REQUEST[$field.'_no']) == '')
			$no = '';
		else
			$no = trim($_REQUEST[$field.'_pre'])
					. trim($_REQUEST[$field.'_no']);
	}
	else 
	{
		$no = trim($_REQUEST[$field]);
	}
	
	$no = str_replace('+', '00', $no);
	$no = ereg_replace('[^0-9]', '', $no);
	return($no);
}

/**
 * generate timestamp from smarty date/time controls
 *
 * @param string $field Field name (prefix)
 * @return int
 */
function SmartyDateTime($field)
{
	$d = $m = $y = 1;
	$h = $i = $s = 0;
	$y = (int)date('Y');
	
	if(isset($_REQUEST[$field.'Day']))
		$d = (int)$_REQUEST[$field.'Day'];
	if(isset($_REQUEST[$field.'Month']))
		$m = (int)$_REQUEST[$field.'Month'];
	if(isset($_REQUEST[$field.'Year']))
		$y = (int)$_REQUEST[$field.'Year'];
	if(isset($_REQUEST[$field.'Hour']))
		$h = (int)$_REQUEST[$field.'Hour'];
	if(isset($_REQUEST[$field.'Minute']))
		$i = (int)$_REQUEST[$field.'Minute'];
	if(isset($_REQUEST[$field.'Second']))
		$s = (int)$_REQUEST[$field.'Second'];
		
	return(($d+$m+$y+$h+$i+$s) == 0
		? 0
		: mktime($h, $i, $s, $m, $d, $y));
}

/**
 * send a file stream with speed limit
 *
 * @param resource $fp File stream
 * @param int $speed Speed limit (kb/s)
 * @return int Sent bytes
 */
function SendFileFP($fp, $speed = -1)
{
	$sentBytes = 0;
	if($speed == -1)
		$bufferSize = 4096;
	else
		$bufferSize = ($speed*1024) / 100;
	while(!feof($fp))
	{
		$buffer = fread($fp, $bufferSize);
		echo($buffer);
		$sentBytes += strlen($buffer);
		if($speed != -1)
			usleep(10000);
	}
	return($sentBytes);
}

/**
 * send a file with speed limit
 *
 * @param string $file Filename
 * @param int $speed Speed limit (kb/s)
 * @return int Sent bytes
 */
function SendFile($file, $speed = -1)
{
	if($fp = @fopen($file, 'rb'))
	{
		$result = SendFileFP($fp, $speed);
		fclose($fp);
		return($result);
	}
	else 
	{
		PutLog(sprintf('Cannot open file <%s> for sending', $file),
				PRIO_WARNING,
				__FILE__,
				__LINE__);
		return(-1);
	}
}

/**
 * split a list of mail addresses and names into pairs of name and mail address
 *
 * @param string $list List string
 * @return array
 */
function ParseMailList($list)
{
	$result = array();
	
	$parts = ExplodeOutsideOfQuotation($list, array(',', ';'));
	
	foreach($parts as $part)
	{
		$resultName = trim(ExtractMailName($part));
		$resultMail = trim(ExtractMailAddress($part));
		$result[] = array(
			'name'		=> $resultName == '' ? /*$resultMail*/ '' : $resultName,
			'mail'		=> $resultMail
		);
	}
	
	return($result);
}

/**
 * split string by $separator, taking care of "quotations"
 *
 * @param string $string Input
 * @param mixed $separator Separator(s), may be an array
 * @return array
 */
function ExplodeOutsideOfQuotation($string, $separator)
{
	$result = array();
	
	$inEscape = $inQuote = false;
	$tmp = '';
	for($i=0; $i<strlen($string); $i++)
	{
		$c = $string[$i];
		if(((!is_array($separator) && $c == $separator)
			|| (is_array($separator) && in_array($c, $separator)))
			&& !$inQuote
			&& !$inEscape)
		{
			if(trim($tmp) != '')
			{
				$result[] = trim($tmp);
				$tmp = '';
			}
		}
		else if($c == '"' && !$inEscape)
			$inQuote = !$inQuote;
		else if($c == '\\' && !$inEscape)
			$inEscape = true;
		else
		{
			$tmp .= $c;
			$inEscape = false;
		}
	}
	if(trim($tmp) != '')
	{
		$result[] = trim($tmp);
		$tmp = '';
	}
	
	return($result);
}

/**
 * convert string charset to charset specified by language file
 *
 * @param string $text Input
 * @param string $charset Input charset
 * @return string
 */
function CharsetDecode($text, $charset)
{
	global $currentCharset;
	
	$charset = trim(strtolower($charset));
	$myCharset = trim(strtolower($currentCharset));
	
	if(substr($charset, 0, 10) == 'iso-8859-1')
		$charset = 'iso-8859-1';
	if(substr($myCharset, 0, 10) == 'iso-8859-1')
		$myCharset = 'iso-8859-1';
	
	if($charset == $myCharset)
		return($text);
	
	if(function_exists('mb_convert_encoding'))
	{
		if($newText = @mb_convert_encoding($text, $myCharset, $charset))
			$text = $newText;
	}
	else if(function_exists('iconv'))
	{
		if($newText = @iconv($charset, $myCharset, $text))
			$text = $newText;
	}
	else 
	{
		switch($charset)
		{
		case 'iso-8859-1':
			if($myCharset == 'utf8' || $myCharset == 'utf-8')
				$text = utf8_encode($text);
			break;
			
		case 'utf8':
		case 'utf-8':
			if($myCharset == 'iso-8859-1')
				$text = utf8_decode($text);
			break;
		}
	}
	
	return($text);
}

/**
 * get filename of data item
 *
 * @param int $id Item ID
 * @param string $fx Extension
 * @return string
 */
function DataFilename($id, $fx = 'msg')
{
	global $plugins, $bm_prefs;
		
	// plugin?
	foreach($plugins->_plugins as $className=>$pluginInfo)
	{
		if(($result = $plugins->callFunction('DataFilename',  $className, false,
						array($id, $fx))) !== false
			&& $result
			&& strlen($result) > strlen($id . $fx))
		{
			return($result);
		}
	}
	
	// generate filename
	$dir = B1GMAIL_DATA_DIR;

	if(file_exists($dir . $id . '.' . $fx))
		return($dir . $id . '.' . $fx);

	for($i=0; $i<strlen((string)$id); $i++)
	{
		$dir .= substr((string)$id, $i, 1);
		if(($i+1) % 2 == 0)
		{
			$dir .= '/';
			if(!file_exists($dir) && ($i<strlen((string)$id)-1))
			{
				@mkdir($dir, 0777);
				@chmod($dir, 0777);
			}
		}
	}
	
	if(substr($dir, -1) == '/')
		$dir = substr($dir, 0, -1);
	$dir .= '.' . $fx;
		
	if(file_exists($dir) || $bm_prefs['structstorage'] == 'yes')
	{
		// structured storage
		return($dir);
	}
	else 
	{
		// default
		return(B1GMAIL_DATA_DIR . $id . '.' . $fx);
	}	
}

/**
 * extract message ids from string
 *
 * @param string $str
 * @return array
 */
function ExtractMessageIDs($str)
{
	$ret_arr = $result = array();
	preg_match_all('/<([^>]+)>/', $str, $ret_arr);
	foreach($ret_arr[0] as $ret)
		if(!in_array($ret, $result))
			$result[] = $ret;
	return($result);
}

/**
 * extract name from a string
 *
 * @param string $string
 * @return string
 */
function ExtractMailName($string)
{
	$newString = '';
	$string = str_replace(ExtractMailAddress($string), '', $string);
	
	for($i=0; $i<strlen($string); $i++)
	{
		if($string[$i] == '<')
			break;
		if($string[$i] != '"' && $string[$i] != '\'')
			$newString .= $string[$i];
	}
	
	return(trim($newString));
}

/**
 * extract mail address from a string
 *
 * @param string $string
 * @return string
 */
function ExtractMailAddress($string)
{
	$ret = '';
	$ret_arr = array();
	if(preg_match_all('/[a-zA-Z0-9&\'\\.\\-_\\+]+@[a-zA-Z0-9.-]+\\.+[a-zA-Z]{2,6}/', $string, $ret_arr) > 0)
		$ret = $ret_arr[0][0];
	return($ret);
}

/**
 * extract mail addresses from string
 *
 * @param string $string
 * @return array
 */
function ExtractMailAddresses($string)
{
	$result = $ret_arr = array();
	preg_match_all('/[a-zA-Z0-9&\'\\.\\-_\\+]+@[a-zA-Z0-9.-]+\\.+[a-zA-Z]{2,6}/', $string, $ret_arr);
	foreach($ret_arr[0] as $ret)
		if(!in_array($ret, $result))
			$result[] = $ret;
	return($result);
}

/**
 * request privileges
 * 
 * @param int $privileges Bitmask
 */
function RequestPrivileges($privileges, $return = false)
{
	global $db, $tpl, $userRow, $groupRow, $bm_prefs, $thisUser, $thisGroup, $currentLanguage,
			$lang_user, $lang_client, $lang_custom, $lang_admin;
	$ok = true;
	
	// sessions are always needed for more privileges
	@session_start();
	
	// user privileges?
	if(($privileges & PRIVILEGES_USER) != 0)
		if(!isset($_SESSION['bm_userLoggedIn']) || !$_SESSION['bm_userLoggedIn']
			|| !isset($_SESSION['bm_userID']) || !isset($_SESSION['bm_sessionToken'])
			|| $_SESSION['bm_sessionToken'] != SessionToken())
		{
			$ok = false;
		}
		else 
		{
			$thisUser = _new('BMUser', array($_SESSION['bm_userID']));
			$userRow = $thisUser->Fetch();
			
			if(isset($tpl) && is_object($tpl))
				$tpl->assign('sid', session_id());
			
			if($userRow !== false && is_array($userRow) && $userRow['gesperrt'] == 'no')
			{
				if(isset($_SESSION['bm_sessionLanguage']) &&
					($_SESSION['bm_sessionLanguage'] != $currentLanguage))
				{
					ReadLanguage();
					if(!MAINTENANCE_MODE)
					{
						// module handler
						ModuleFunction('OnReadLang', array(&$lang_user, &$lang_client, &$lang_custom, &$lang_admin, $currentLanguage));
					}
					ReadCustomLanguage();
				}
				
				$thisGroup = $thisUser->GetGroup();
				$groupRow = $thisGroup->Fetch();
				if($groupRow === false || !is_array($groupRow))
				{
					PutLog(sprintf('Group <%d> of user <%s> not found!', $userRow['gruppe'], $userRow['email']),
						PRIO_WARNING,
						__FILE__,
						__LINE__);
					$ok = false;
				}
			}
			else 
			{
				$ok = false;
			}
		}
		
	// admin privileges?
	if(($privileges & PRIVILEGES_ADMIN) != 0)
		if(!isset($_SESSION['bm_adminLoggedIn']) || !$_SESSION['bm_adminLoggedIn']
			|| !isset($_SESSION['bm_adminAuth']) || !isset($_SESSION['bm_sessionToken'])
			|| $_SESSION['bm_sessionToken'] != SessionToken())
		{
			$ok = false;
		}
		else 
		{
			if(isset($tpl) && is_object($tpl))
				$tpl->assign('sid', session_id());
			
			if(md5(md5($bm_prefs['adminpw']) . $_SERVER['HTTP_USER_AGENT']) != $_SESSION['bm_adminAuth'])
				$ok = false;
		}
		
	// mobile privileges?
	if(($privileges & PRIVILEGES_MOBILE) != 0 && $ok)
		$ok = $groupRow['wap'] == 'yes';
		
	// client api access privileges?
	if(($privileges & PRIVILEGES_CLIENTAPI) != 0 && $ok)
		$ok = true;

	// requested action not allowed
	if(!$ok && !$return)
	{
		DisplayError(0x02, 'Unauthorized', 'You are not authrized to view or change this dataset or page. Possible reasons are too few permissions or an expired session.',
			sprintf("Requested privileges:\n%d\n\nLogged in:\n%s",
				$privileges,
				isset($_SESSION['bm_userLoggedIn']) && $_SESSION['bm_userLoggedIn'] ? 'Yes' : 'No'),
			__FILE__,
			__LINE__);
		exit();
	}
	else if($return)
		return($ok);
}

/**
 * format date
 *
 * @param int $date
 * @return string
 */
function FormatDate($date = -1)
{
	global $bm_prefs;
	
	if($date == -1)
		$date = time();
	return(date($bm_prefs['datumsformat'], $date));
}

/**
 * check zip/city
 *
 * @param string $plz
 * @param string $ort
 * @param int $staat
 * @return bool
 */
function ZIPCheck($plz, $ort, $staat)
{
	$strip_chars = array(',', ';', '-', '?', ':', '?', '1', ' ', '', '', '', '', '', '', "ae", "oe", "ue", "AE", "OE", "UE", "Ae", "Oe","Ue");

	$plz = ereg_replace('^([0]*)', '', $plz);
	$ort = strtolower($ort);							# [ORT_STR] erstellen
	$ort = str_replace($strip_chars, '', $ort);			# [ORT_STR] erstellen
	$hash = $plz . soundex($ort);						# [PLZ][ORT_STR]
	$hash = crc32($hash);								# CRC32
	$hash = pack('i', $hash);							# In einen Binaerstring packen

	$plzfile = B1GMAIL_DIR . 'plz/' . $staat . '.plz';

	if(file_exists($plzfile))
	{
		$fp = fopen($plzfile, 'r');
		$inh = fread($fp, filesize($plzfile));
		fclose($fp);

		$pos = strpos($inh, $hash);						# In der PLZ-Datei nach dem PLZ/Ort-Paar suchen
		$pos2 = strpos($inh, $hash[3] . $hash[2] . $hash[1] . $hash[0]);
		unset($inh);

		if($pos === false && $pos2 === false)
		{
			return(false);
		}
		else
		{
			return(true);
		}
	}
	else
	{
		return(true);
	}
}

/**
 * utf8 encode
 * 
 */
function EncodeUTF8($str)
{
	return(utf8_encode($str));
}

/**
 * xml encode
 * 
 */
function XMLEncode($str)
{
	$newstr = '';
	for($i=0; $i<strlen($str); $i++)
	{
		$c = substr($str, $i, 1);
		if(ord($c) < 32 || ord($c) == 127 || $c == '<' || $c == '>' || $c == '"' || $c == '&')
			$newstr .= '&#' . ord($c) . ';';
		else 
			$newstr .= $c;
	}
	
	return($newstr);
}

/**
 * country list
 * 
 */
function CountryList()
{
	global $db, $cacheManager;
	
	if(!($laender = $cacheManager->Get('countryList')))
	{
		$laender = array();
		$land_sql = $db->Query('SELECT id,land FROM {pre}staaten ORDER BY land ASC');
		while($row = $land_sql->FetchArray())
			$laender[$row['id']] = $row['land'];
		$land_sql->Free();
		
		$cacheManager->Set('countryList', $laender);
	}
	
	return($laender);
}

/**
 * check if data is binary
 *
 * @param string $string String
 * @return bool
 */
function isBinary($string)
{
	for($i=0; $i<min(strlen($string), 32*1024); $i++)
	{
		$c = $string[$i];
		$cc = ord($c);
		
		//if(($cc < 32 || $cc > 126) && $cc != 13 && $cc != 10 && $cc != 9)
		if(($cc < 32 || $cc == 127) && ($cc != 13 && $cc != 10 && $cc != 9))
			return(true);
	}
	
	return(false);
}

/**
 * Base64 ouput incl. word wrap
 *
 * @param string $string String
 */
function base64Output($string)
{
	echo wordwrap(base64_encode($string), 72, "\n", true);
}

/**
 * generate xml respone from any array
 * 
 */
function NormalArray2XML($array, $responseString = 'response', $first = true)
{
	global $currentCharset;
	
	// header
	if($first)
	{
		header('Content-Type: text/xml; charset=' . $currentCharset);
		header('Pragma: no-cache');
		header('Cache-Control: no-cache');
		echo '<?xml version="1.0" encoding="' . $currentCharset . '" ?>' . "\n";
		echo '<' . $responseString . '>' . "\n";
	}
	else if(is_array($array))
		echo '<array>' . "\n";
	
	if(is_array($array))
	{
		foreach($array as $key=>$val)
		{
			if(is_numeric($key))
				echo '<item key="' . $key . '">';
			else
				echo '<' . $key . '>';
			
			NormalArray2XML($val, $responseString, false);
			
			if(is_numeric($key))
				echo '</item>' . "\n";
			else 
				echo '</' . $key . '>'. "\n";
		}
	}
	else 
	{
		if($array !== '')
		{
			if(is_numeric($array) || is_int($array))
			{
				echo $array;
			}
			else if(is_bool($array))
			{
				echo $array ? 1 : 0;
			}
			else if(!isBinary($array) && strpos($array, "\n") === false && strpos($array, "\r") === false)
			{
				echo XMLEncode($array);
			}
			else 
			{
				echo '<![CDATA[';
				if(isBinary($array))
				{
					echo '[[BASE64]]';
					base64Output($array);
				}
				else 
					echo $array;
				echo ']]>' . "\n";
			}
		}
	}
	
	// footer
	if($first)
	{
		echo '</' . $responseString . '>' . "\n";
	}
	else if(is_array($array))
		echo '</array>' . "\n";
}

/**
 * generate xml response from specialy prepared array
 * 
 */
function Array2XML($array, $responseString = 'response', $first = true)
{
	global $currentCharset;
	
	// header
	if($first)
	{
		header('Content-Type: text/xml; charset=' . $currentCharset);
		header('Pragma: no-cache');
		header('Cache-Control: no-cache');
		echo '<?xml version="1.0" encoding="' . $currentCharset . '" ?>' . "\n";
		echo '<' . $responseString . '>' . "\n";
	}
	
	// array => node output
	if(is_array($array))
		foreach($array as $key=>$value)
		{
			// custom node name?
			if(is_array($value) && isset($value['#nodeName']))
				$key = $value['#nodeName'];
			
			// ignore special children (starting with #)
			if(!is_array($key) && strlen($key) > 0 && $key[0] == '#')
				continue;
				
			// node opening
			echo '<' . $key . ' ';
	
			// node parameters
			if(isset($value['#nodeParam']) && is_array($value['#nodeParam']))
				foreach($value['#nodeParam'] as $paramKey=>$paramVal)
					printf(' %s="%s"', $paramKey, addslashes(XMLEncode($paramVal)));
			
			echo '>';
			
			if(isset($value['#nodeCDATA']))
			{
				// cdata
				echo '<![CDATA[' . ($value['#nodeCDATA']) . ']]>';
			}
			else 
			{
				// node data / children
				Array2XML($value, $responseString, false);
			}
			
			// node ending
			echo '</' . $key . '>' . "\n";
		}
		
	// non-array => string output
	else 
		echo XMLEncode($array);
	
	// footer
	if($first)
	{
		echo '</' . $responseString . '>' . "\n";
	}
}

/**
 * execute a module function
 * 
 */
function ModuleFunction($function, $args = false)
{
	global $plugins;
	
	if(isset($plugins) && is_object($plugins))
	{
		if($args === false)
			$args = array();
		$args = array($function, false, false, $args);
		call_user_func_array(array(&$plugins, 'callFunction'), $args);
	}
	else 
	{
		if(MAINTENANCE_MODE)
			return(false);
		
		// abort
		DisplayError(0x17, 'Illegal plugin handler call', 'b1gMail tried to call a plugin handler before the plugin system was intialized.',
			sprintf("Function:\n%s", $function),
			__FILE__,
			__LINE__);
		die();
	}
}

/**
 * get language file info
 * 
 */
function GetLanguageInfo($file)
{
	global $bm_prefs, $cacheManager;
	
	$fileName = B1GMAIL_DIR . 'languages/' . $file . '.lang.php';
	$cacheKey = 'langInfo:' . $file;
	
	// in cache?
	$cacheData = $cacheManager->Get($cacheKey);
	if($cacheData && $cacheData['ctime'] == filectime($fileName))
	{
		$cacheData['default'] = $bm_prefs['language'] == $file;
		return($cacheData);
	}
	
	// no -> read from file
	$result = array();
	$fp = @fopen($fileName, 'r');
	if(is_resource($fp))
	{
		while($line = fgets($fp))
		{
			if(substr($line, 0, strlen('// b1gMailLang::')) == '// b1gMailLang::')
			{
				list(, $langTitle,
						$langAuthor,
						$langAuthorMail,
						$langAuthorWeb,
						$langCharset,
						$langLocale) = explode('::', trim($line));
				$result['ctime'] = filectime($fileName);
				$result['title'] = $langTitle;
				$result['author'] = $langAuthor;
				$result['authorMail'] = $langAuthorMail;
				$result['authorWeb'] = $langAuthorWeb;
				$result['charset'] = $langCharset;
				$result['locale'] = $langLocale;
				$result['writeable'] = is_writeable(B1GMAIL_DIR . 'languages/' . $file . '.lang.php');
				$result['default'] = $bm_prefs['language'] == $file;
				break;
			}
		}
		
		fclose($fp);
	}
	
	// put to cache
	$cacheManager->Set($cacheKey, $result);
	
	return($result);
}

/**
 * get available templates
 *
 * @return array
 */
function GetAvailableTemplates()
{
	$result = array();
	
	$dir = @dir(B1GMAIL_DIR . 'templates/');
	if(is_object($dir))
	{
		while($file = $dir->read())
			if($file != '.'
				&& $file != '..'
				&& is_dir(B1GMAIL_DIR . 'templates/' . $file)
				&& file_exists(B1GMAIL_DIR . 'templates/' . $file . '/cache/'))
				$result[] = $file;
		$dir->close();		
	}
	
	return($result);
}

/**
 * get available languages
 * 
 * @return array
 */
function GetAvailableLanguages()
{
	global $bm_prefs;
	
	$result = array();
	
	$dir = @dir(B1GMAIL_DIR . 'languages/');
	if(is_object($dir))
	{
		while($file = $dir->read())
			if($file != '.' && $file != '..' && substr($file, -9) == '.lang.php')
				if($info = GetLanguageInfo(substr($file, 0, -9)))
				{
					if(strtolower($bm_prefs['language']) == strtolower(substr($file, 0, -9)))
						$info['default'] = true;
					$result[substr($file, 0, -9)] = $info;
				}
		$dir->close();
	}
	
	return($result);
}

/**
 * read language file
 *
 */
function ReadLanguage()
{
	global $db, $currentCharset, $lang_admin, $lang_user, $lang_client, $lang_custom, $bm_prefs, $currentLanguage, $lang_info, $cacheManager;
	
	// initialize arrays
	$lang_admin = $lang_user = $lang_client = $lang_custom = array();
	
	// session-defined language?
	if(isset($_SESSION['bm_sessionLanguage']))
	{
		$language = $_SESSION['bm_sessionLanguage'];
	}
	
	// request-defined language?
	else if(isset($_GET['language']))
	{
		$language = $_GET['language'];
	}
	
	// login-defined language?
	else if(isset($_POST['language']) && isset($_REQUEST['action']) && $_REQUEST['action']=='login')
	{
		$language = $_POST['language'];
	}
	
	// default language
	else
	{
		$language = $bm_prefs['language'];
	}
	
	// remove bad characters
	$language = ereg_replace('([^a-zA-Z0-9\-\_]*)', '', $language);
	
	// exists?
	if(!file_exists(B1GMAIL_DIR . 'languages/' . $language . '.lang.php'))
	{
		// fall back?
		if(file_exists(B1GMAIL_DIR . 'languages/' . $bm_prefs['language'] . '.lang.php'))
		{
			// yes, default language exists
			$language = $bm_prefs['language'];
		}
		else 
		{
			// no, abort
			DisplayError(0x03, 'Language file not found', 'The requested language file and the default language file do not exist.',
				sprintf("Language file:\n%s", $bm_prefs['language'] . '.lang.php'),
				__FILE__,
				__LINE__);
			exit();
		}
	}
	
	// include
	$lang_info = GetLanguageInfo($language);
	$langCTime = $lang_info['ctime'];
	
	if(($langData = $cacheManager->Get('langData:' . $language))
		&& $langData['ctime'] == $langCTime)
	{
		$lang_admin		= $langData['admin'];
		$lang_user 		= $langData['user'];
		$lang_client 	= $langData['client'];
		$lang_custom 	= $langData['custom'];
		unset($langData);
	}
	else 
	{
		if(!@include(B1GMAIL_DIR . 'languages/' . $language . '.lang.php'))
		{
			// parse error
			DisplayError(0x04, 'Language file invalid', 'Parse error while including requested language file (corrupt file?).',
				sprintf("Language file:\n%s", $language . '.lang.php'),
				__FILE__,
				__LINE__);
			exit();
		}
		
		$cacheManager->Set('langData:' . $language, array(
			'ctime'		=> $langCTime,
			'admin'		=> $lang_admin,
			'user'		=> $lang_user,
			'client'	=> $lang_client,
			'custom'	=> $lang_custom
		));
		$cacheManager->Delete('langCustom:' . $language);
	}
	
	// get info
	$currentLanguage = $language;
	$currentCharset = $lang_info['charset'];
	
	// locale
	$localeOK = false;
	$locales = explode('|', $lang_info['locale']);
	
	if(PHPNumVersion() >= 430)
	{
		setlocale(LC_ALL, $locales);
	}
	else 
	{
		foreach($locales as $locale)
			if(setlocale(LC_ALL, $locale))
			{
				$localeOK = true;
				break;
			}
	}
	
	// charset
	header('Connection: close');
	header('Content-Type: text/html; charset=' . $currentCharset);
	return(true);
}

/**
 * read custom language phrases
 *
 */
function ReadCustomLanguage()
{
	global $db, $currentLanguage, $lang_custom, $cacheManager;
	
	// get custom texts
	if(!($lang_custom_cmgr = $cacheManager->Get('langCustom:' . $currentLanguage)))
	{
		$res = $db->Query('SELECT `key`,`text` FROM {pre}texts WHERE language=?',
			$currentLanguage);
		while($row = $res->FetchArray(MYSQL_ASSOC))
			$lang_custom[$row['key']] = $row['text'];
		$res->Free();
		
		$cacheManager->Set('langCustom:' . $currentLanguage, $lang_custom);
	}
	else 
		foreach($lang_custom_cmgr as $key=>$val)
			$lang_custom[$key] = $val;
}

/**
 * decode mail header field
 *
 * @param string $text Text
 * @return string
 */
function DecodeMailHeaderField($text)
{
	preg_match_all('/(=\?([^?]+)\?(Q|B)\?([^?]*)\?=)/i', $text, $ret_arr);
	if(!is_array($ret_arr) || !isset($ret_arr[3]))
		return($text);
	foreach($ret_arr[3] as $k=>$ret)
	{
		$str = $ret_arr[4][$k];
		switch(strtoupper($ret))
		{
		case 'Q':
			$str = quoted_printable_decode(str_replace('_', ' ', $str));
			break;
		case 'B':
			$str = base64_decode($str);
			break;
		default:
			break;
		}
		$str = CharsetDecode($str, $ret_arr[2][$k]);
		$text = str_replace($ret_arr[0][$k], $str, $text);
	}
	return($text);
}

/**
 * should $text be encoded?
 *
 * @param string $text Text
 * @return bool
 */
function ShouldEncodeMailHeaderFieldText($text)
{
	for($i = 0; $i < strlen($text); $i++)
	{
		$dec = ord($text[ $i ]);
		if(($dec < 32) || ($dec > 126))
			return(true);
	}
	return(false);
}

/**
 * encode mail header field
 *
 * @param string $text Text
 * @param string $charset Charset
 * @return string
 */
function EncodeMailHeaderField($text, $charset = '')
{
	global $currentCharset;
	
	// get charset, replace line feeds and line breaks
	if($charset == '')
		$charset = $currentCharset;
	$text = str_replace(array("\r", "\n"), '', $text);
	
	// check if string is 8bit or contains non-printable characters
	$encode = ShouldEncodeMailHeaderFieldText($text);
	
	// encode, if needed
	if($encode)
	{
		$fieldParts = array();
		$words = ExplodeOutsideOfQuotation($text, ' ');
		$i = 0;
		foreach($words as $word)
		{
			$encode = ShouldEncodeMailHeaderFieldText($word);
			
			if(isset($fieldParts[$i]))
			{
				if($fieldParts[$i][0] == $encode)
					$fieldParts[$i][1] .= ' ' . $word;
				else 
					$fieldParts[++$i] = array($encode, $word);
			}
			else 
				$fieldParts[$i] = array($encode, $word);
		}
		
		$encodedText = '';
		foreach($fieldParts as $fieldPart)
		{
			if($fieldPart[0])
				$encodedText .= ' ' . sprintf('=?%s?B?%s?=',
					$charset,
					base64_encode(trim($fieldPart[1])));
			else 
				$encodedText .= ' ' . trim($fieldPart[1]);
		}
		
		return(trim($encodedText));
	}
	else 
		return($text);
}

/**
 * write to log table
 *
 * @param string $entry
 * @param int $prio
 * @param string $at_file
 * @param int $at_line
 */
function PutLog($entry, $prio = PRIO_NOTE, $at_file = __FILE__, $at_line = __LINE__)
{
	global $db;
	
	if($prio == PRIO_DEBUG && !DEBUG)
		return;
	
	$sep = (strpos($at_file, '\\') !== false) ? '\\' : '/';
	$at_file = explode($sep, $at_file);
	$at_file = $at_file[ count($at_file)-1 ];

	$db->Query('INSERT INTO {pre}logs(eintrag, zeitstempel, prio) VALUES(?, UNIX_TIMESTAMP(), ?)',
		"(${at_file}:${at_line}) $entry",
		$prio);
}

/**
 * initialize modules
 * 
 */
function InitializePlugins()
{
	global $plugins;
	
	$plugins = _new('BMPluginInterface');
	$plugins->loadPlugins();
}

/**
 * shutdown
 * 
 */
function b1gMailShutdown()
{
	global $db, $tempFilesToReleaseAtShutdown;
	
	foreach($tempFilesToReleaseAtShutdown as $item)
		ReleaseTempFile($item[0], $item[1]);
	
	@mysql_close($db->_handle);
}

/**
 * display error
 *
 * @param int $number
 * @param string $title
 * @param string $description
 * @param string $text
 * @param string $file
 * @param int $line
 */
function DisplayError($number, $title, $description, $text = false, $file, $line)
{
	if(INTERFACE_MODE)
	{
		if(isset($_SERVER['HTTP_USER_AGENT']))
			echo '<pre>';
		
		printf('%s (Error 0x%02X)' . "\n\n", $title, $number);
		printf('%s' . "\n\n", $description);
		
		if(DEBUG)
		{
			printf('Module: %s' . "\n" . 'Line: %s' . "\n" . '%s',
				str_replace(B1GMAIL_DIR, '', $file),
				$line,
				$text !== false && $text != '' ? "\n" . $text . "\n\n" : '');
		}
		
		printf('%s' . "\n\n", date('r'));
		
		if(isset($_SERVER['HTTP_USER_AGENT']))
			echo '</pre>';
	}
	else
	{
	?>
<html>
<head>
	<title>b1gMail: Error <?php echo(sprintf('0x%02X', $number)); ?></title>
	<style>
	<!--
		*			{ font-family: tahoma, arial, verdana; font-size: 12px; }
		H1			{ font-size: 16px; font-weight: bold; border-bottom: 1px solid #DDDDDD; }
		H2			{ font-size: 14px; font-weight: normal; }
		.addInfo	{ font-family: courier, courier new; font-size: 10px; height: 100px; overflow: auto;
						border: 1px solid #DDDDDD; padding: 5px; }
	//-->
	</style>
</head>
<body bgcolor="#F1F2F6">
	
	<br /><br />
	<center>
		<table width="600" cellspacing="0" cellpadding="0" bgcolor="#FFFFFF">
			<tr>
				<td width="14" height="14"><img src="<?php echo(B1GMAIL_REL); ?>res/msg_edge_left_top.png" border="0" alt="" /></td>
				<td height="14" background="<?php echo(B1GMAIL_REL); ?>res/msg_top.png"><img src="<?php echo(B1GMAIL_REL); ?>res/msg_top.png" border="0" alt="" /></td>
				<td width="14" height="14"><img src="<?php echo(B1GMAIL_REL); ?>res/msg_edge_right_top.png" border="0" alt="" /></td>
			</tr>
			<tr>
				<td width="14" background="<?php echo(B1GMAIL_REL); ?>res/msg_left.png"><img src="<?php echo(B1GMAIL_REL); ?>res/msg_left.png" border="0" alt="" /></td>
				<td>
					<div style="padding: 10px;">
					<table width="100%">
						<tr>
							<td align="center" width="80" valign="top"><img src="<?php echo(B1GMAIL_REL); ?>res/error_icon.png" border="0" alt="" /></td>
							<td valign="top" align="left">
							
								<h1><?php echo(sprintf('%s (Error 0x%02X)', HTMLFormat($title), $number)); ?></h1>
								<h2><?php echo(HTMLFormat($description)); ?></h2>
								
								<hr size="1" color="#DDDDDD" width="100%" noshade="noshade" />
								
								<?php if(DEBUG) { ?>
								Additional information:<br /><br />
								
								<div class="addInfo">
									Module:<br /><?php echo(str_replace(B1GMAIL_DIR, '', $file)); ?><br /><br />
									Line:<br /><?php echo($line); ?><br /><br />
									<?php if($text !== false && $text != '') echo(nl2br(HTMLFormat($text))); ?>
								</div>
								
								<hr size="1" color="#DDDDDD" width="100%" noshade="noshade" />
								<?php } ?>
								A notification about this error has been sent to the administrator. The problem
								will be fixed as soon as possible.
								
								<br /><br />
								<input type="button" value="&nbsp; Try again &nbsp;" onclick="document.location.reload()" style="padding: 1px;" />
								<input type="button" value="&nbsp; Start page &nbsp;" onclick="document.location.href='<?php echo(B1GMAIL_REL); ?>';" style="padding: 1px;" />
								
							</td>
						</tr>
					</table>
					</div>
				</td>
				<td width="14" background="<?php echo(B1GMAIL_REL); ?>res/msg_right.png"><img src="<?php echo(B1GMAIL_REL); ?>res/msg_right.png" border="0" alt="" /></td>
			</tr>
			<tr>
				<td width="14" height="14"><img src="<?php echo(B1GMAIL_REL); ?>res/msg_edge_left_bottom.png" border="0" alt="" /></td>
				<td height="14" background="<?php echo(B1GMAIL_REL); ?>res/msg_bottom.png"><img src="<?php echo(B1GMAIL_REL); ?>res/msg_bottom.png" border="0" alt="" /></td>
				<td width="14" height="14"><img src="<?php echo(B1GMAIL_REL); ?>res/msg_edge_right_bottom.png" border="0" alt="" /></td>
			</tr>
		</table>
	</center>

</body>
</html>
	<?php 
	}
}

/**
 * connect to db
 * 
 */
function ConnectDB()
{
	global $db, $mysql;
	
	// try to connect
	$mysqlHandle = @mysql_connect($mysql['host'], $mysql['user'], $mysql['pass']);
	if($mysqlHandle)
	{
		if(@mysql_select_db($mysql['db'], $mysqlHandle))
		{
			$db = _new('DB', array($mysqlHandle));
			return(true);
		}
		else 
		{
			$error = 1;
		}
	}
	else 
	{
		$error = 0;
	}
	
	// failed
	DisplayError(0x01, 'Database error', 'Failed to connect to MySQL database backend.',
		sprintf("Process:\n%s\n\nError number:\n%d\n\nError description:\n%s",
			$error == 1 ? 'Database selection' : 'Connect',
			mysql_errno(),
			mysql_error()),
		__FILE__,
		__LINE__);
	exit();
}

/**
 * fetch configuration from db
 *
 */
function ReadConfig()
{
	global $bm_prefs, $db;
	
	$res = $db->Query('SELECT * FROM {pre}prefs LIMIT 1');
	if($res->RowCount() == 1)
		$bm_prefs = $res->FetchArray(MYSQL_ASSOC);	
	$res->Free();
}

/**
 * reset webdisk traffic
 *
 */
function ResetWebdiskTraffic()
{
	global $db;
	
	$db->Query('UPDATE {pre}users SET traffic_down=0,traffic_up=0,traffic_status=? WHERE traffic_status!=?',
		(int)date('m'),
		(int)date('m'));
}

/**
 * clean up webdisk locks
 *
 */
function CleanupWebdiskLocks()
{
	global $db;
	
	$db->Query('DELETE FROM {pre}disklocks WHERE (expires>0 AND expires<?) OR (expires=0 AND modified<?)',
		time(),
		time()-7*TIME_ONE_DAY);
}

/**
 * clean up expired cert mails
 *
 */
function CleanupCertMails()
{
	global $db, $bm_prefs;
	
	$date = time() - $bm_prefs['einsch_life'];
	
	$res = $db->Query('SELECT DISTINCT(mail) AS mailID FROM {pre}certmails WHERE date<' . $date);
	while($row = $res->FetchArray(MYSQL_ASSOC))
	{
		$db->Query('UPDATE {pre}mails SET flags=flags^'.FLAG_CERTMAIL.' WHERE id=?',
			$row['mailID']);
	}
	$res->Free();
	
	$db->Query('DELETE FROM {pre}certmails WHERE date<' . $date);
}

/**
 * clean up expired safe codes
 *
 */
function CleanupSafeCodes()
{
	global $db;
	$db->Query('DELETE FROM {pre}safecode WHERE generation<' . (time()-8*TIME_ONE_HOUR));
}

/**
 * delete outdated, pending aliases
 *
 * @return bool
 */
function CleanupAliases()
{
	global $db;
	
	$db->Query('DELETE FROM {pre}aliase WHERE (type&'.ALIAS_PENDING.')!=0 AND date<'.(time()-TIME_ONE_WEEK));
	return(true);
}

/**
 * generate temp file name by file id
 *
 * @param int $id
 * @return string
 */
function TempFileName($tempID)
{
	return(B1GMAIL_DIR . 'temp/tmpfile_' . $tempID . '.tmp');
}

/**
 * cleanup temp files
 *
 * @return boolean
 */
function CleanupTempFiles()
{
	global $db;
	
	$res = $db->Query('SELECT id,user FROM {pre}tempfiles WHERE expires<'.time());
	while($row = $res->FetchArray())
		ReleaseTempFile($row['user'], $row['id']);
	$res->Free();
	
	return(true);
}

/**
 * check if temp file is valid and belongs to user
 *
 * @param int $userID
 * @param int $tempID
 * @return bool
 */
function ValidTempFile($userID, $tempID)
{
	global $db;
	
	$res = $db->Query('SELECT COUNT(*) FROM {pre}tempfiles WHERE id=? AND user=?',
		$tempID,
		$userID);
	list($count) = $res->FetchArray(MYSQL_NUM);
	$res->Free();
	
	return($count == 1);
}

/**
 * request new temp file id
 *
 * @param int $expires Expire date
 * @return int
 */
function RequestTempFile($userID, $expires = -1)
{
	global $db;
	
	// default expire timestamp (now + one day)
	if($expires == -1)
		$expires = time() + TIME_ONE_DAY;
	
	// register file
	$db->Query('INSERT INTO{pre}tempfiles(expires,user) VALUES(?,?)',
		$expires,
		$userID);
	$tempID = $db->InsertID();
	
	// return filename
	return($tempID);
}

/**
 * release and delete temp file
 *
 * @param int $tempID
 * @param int $userID
 * @return bool
 */
function ReleaseTempFile($userID, $tempID)
{	
	global $db;
	
	if(ValidTempFile($userID, $tempID))
	{
		// release
		$db->Query('DELETE FROM {pre}tempfiles WHERE id=? AND user=?',
			$tempID,
			$userID);
			
		// delete
		$fileName = TempFileName($tempID);
		if(file_exists($fileName))
			@unlink($fileName);
		
		// return
		return(true);
	}
	
	// return
	return(false);
}

if(!function_exists('fprintf'))
{
	/**
	 * fprintf implementation
	 *
	 * @param resource $handle
	 * @param string $format
	 * @return int
	 */
	function fprintf($handle, $format)
	{
		$args = func_get_args();
		$string = vsprintf($format, array_slice($args, 2));
		return(fwrite($handle, $string));
	}
}
?>