<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: dashboard.class.php,v 1.4 2007/12/04 20:31:39 patrick Exp $
 *
 */

/**
 * dashboard class
 */
class BMDashboard
{
	var $_type;
	
	/**
	 * constructor
	 *
	 * @param int $type Dashboard type (BMWIDGET_*-constant)
	 * @return BMDashboard
	 */
	function BMDashboard($type)
	{
		$this->_type = $type;
	}
	
	/**
	 * get array of shown widgets
	 *
	 * @param string $widgetOrder Widget order string
	 * @return array
	 */
	function getWidgetArray($widgetOrder)
	{
		global $plugins;
		
		$widgetList = explode(',', str_replace(';', ',', $widgetOrder));	
		$tplWidgets = array();
		$widgets = $plugins->getWidgetsSuitableFor($this->_type);
		foreach($widgets as $widget)
		{
			if(in_array($widget, $widgetList))
			{
				$plugins->callFunction('renderWidget', $widget);
				$tplWidgets[$widget] = array(
					'template'		=> $plugins->pluginTemplatePath($plugins->getParam('widgetTemplate', $widget), $widget),
					'title'			=> $plugins->getParam('widgetTitle', $widget)
				);
			}
		}
		
		return($tplWidgets);
	}
	
	/**
	 * check widget order string
	 *
	 * @param string $widgetOrder Order string
	 * @return bool
	 */
	function checkWidgetOrder($widgetOrder)
	{
		global $plugins;
			
		$widgets = $plugins->getWidgetsSuitableFor($this->_type);
		$widgetList = explode(',', str_replace(';', ',', $widgetOrder));
		
		foreach($widgetList as $widget)
			if($widget != '' && !in_array($widget, $widgets))
				return(false);

		return(true);	
	}
	
	/**
	 * get all possible widgets suitable for this dashboard
	 *
	 * @param string $widgetOrder Current order string
	 * @return array
	 */
	function getPossibleWidgets($widgetOrder)
	{
		global $plugins;
		
		$possibleWidgets = $plugins->getWidgetsSuitableFor($this->_type);
		$activeWidgets = explode(',', str_replace(';', ',', $widgetOrder));
		
		$myWidgets = array();
		foreach($possibleWidgets as $widget)
		{
			$myWidgets[$widget] = array(
				'title'		=> $plugins->getParam('widgetTitle', $widget),
				'active'	=> in_array($widget, $activeWidgets)
			);
		}
	
		return($myWidgets);
	}
	
	/**
	 * generate new order string from posted customize form
	 *
	 * @param string $widgetOrder Current order string
	 * @return string New order string
	 */
	function generateOrderStringFromPostForm($widgetOrder)
	{
		global $plugins;
		
		$possibleWidgets = $plugins->getWidgetsSuitableFor($this->_type);
	
		// get al activated widgets
		$newOrder = '';
		$newWidgets = array();
		foreach($_POST as $key=>$val)
			if(substr($key, 0, 7) == 'widget_'
				&& in_array(substr($key, 7), $possibleWidgets))
				$newWidgets[] = substr($key, 7);
		
		// explode old order string
		$rows = explode(';', $widgetOrder);
		$newRows = array();
		
		// remove deactivated widgets
		foreach($rows as $row)
		{
			$cols = explode(',', $row);
			
			foreach($cols as $col)
				if($col != '' && !in_array($col, $newWidgets))
					$newOrder .= ',';
				else 
				{
					$newOrder .= $col . ',';
					if($col != '')
						unset($newWidgets[array_search($col, $newWidgets)]);
				}
					
			$newOrder = substr($newOrder, 0, -1) . ';';
		}
		
		// add new activated widgets
		$i = 0;
		foreach($newWidgets as $widget)
		{
			$i++;
			$newOrder .= $widget . ',';
			
			if($i%3 == 0)
				$newOrder = substr($newOrder, 0, -1) . ';';
		}
		
		// trim empty lines and trailing characters
		$newOrder = ereg_replace(';$', '', $newOrder);
		$newOrder = str_replace(';,,;', ';', $newOrder);
		$newOrder = ereg_replace('^,,;', '', $newOrder);
		$newOrder = ereg_replace(';,,$', '', $newOrder);
		
		return($newOrder);
	}
}
?>