<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: mail.class.php,v 1.37 2009/02/16 17:16:50 patrick Exp $
 *
 */

if(!class_exists('BMMailParser'))
	include(B1GMAIL_DIR . 'serverlib/mailparser.inc.php');

/**
 * mail class
 */
class BMMail
{
	var $_userID;
	var $_row;
	var $_fp;
	var $_parsed;
	var $_useCache;
	var $_msgFileName;
	var $trusted;
	var $infection;
	var $id;
	var $flags;
	var $date;
	var $priority;
	var $trained;
	var $storedIn;
	var $smimeStatus = SMIME_UNKNOWN;
	var $smimeCertificateHash = false;
	
	/**
	 * constructor
	 *
	 * @param array $row Message row
	 * @return BMMail
	 */
	function BMMail($userID, $row = false, $fp = false, $useCache = true, $msgFileName = false)
	{
		$this->_useCache = $useCache;
		$this->_userID = $userID;
		$this->_parsed = false;
		if($row !== false)
			$this->_fromRow($row);
		$this->_fp = $fp;
		$this->_msgFileName = $msgFileName;
		
		ModuleFunction('OnGetMail', array($this->id, $this->_userID));
	}
	
	/**
	 * parse mail
	 * 
	 * @return void
	 */
	function Parse()
	{
		global $bm_prefs, $db, $cacheManager;
		
		if($this->_parsed !== false)
			return;
	
		if($this->_fp === false)
			$this->_fp = $this->GetMessageFP();
		
		//
		// mail in cache?
		//
		if($this->_useCache
			&& $this->id > 0)
		{
			// try to find parsed message in cache
			$this->_parsed = $cacheManager->Get('parsedMsg:' . $this->id);
			
			if($this->_parsed)
			{
				$this->_parsed->InheritFP($this->_fp);
				return;
			}
			else 
				$createCacheFile = true;
		}
		
		$this->_parsed = _new('BMMailParser', array($this->_fp));
		$this->_parsed->Parse();
		
		//
		// create cache entry?
		//
		if(isset($createCacheFile) && $createCacheFile)
			$cacheManager->Add('parsedMsg:' . $this->id, $this->_parsed, TIME_ONE_DAY);
	}
	
	/**
	 * get part list
	 * 
	 * @return array
	 */
	function GetPartList()
	{
		$this->Parse();
		return($this->_parsed->GetPartList());
	}
	
	/**
	 * load mail from row
	 *
	 * @param array $row Row (of bm60_mails)
	 */
	function _fromRow($row)
	{
		$this->_row			= $row;
		$this->id			= $row['id'];
		$this->storedIn		= $row['body'] == 'file' ? STORE_FILE : STORE_DB;
		$this->date			= $row['datum'];
		$this->priority		= $row['priority'] == 'high' ? ITEMPRIO_HIGH :
									($row['priority'] == 'low' ? ITEMPRIO_LOW :
									ITEMPRIO_NORMAL);
		$this->flags		= $row['flags'];
		$this->infection	= $row['virnam'];
		$this->trained		= $row['trained'] == 1;
	}
	
	/**
	 * check if mail is trusted
	 *
	 * @return bool
	 */
	function IsTrusted()
	{
		if(($trustToken = $this->GetHeaderValue('x-b1gmail-trusted')) != '')
		{
			$validToken = GenerateTrustToken($this->GetHeaderValue('message-id'),
				$this->GetHeaderValue('from'),
				$this->GetHeaderValue('to'),
				$this->GetHeaderValue('subject'));
			return($validToken == $trustToken);
		}
	}

	/**
	 * check if mail is signed
	 *
	 * @return bool
	 */
	function IsSigned()
	{
		$this->Parse();
		$partList = $this->_parsed->GetPartList();
		
		return(isset($partList[1]) && isset($partList[1]['content-type'])
				&& in_array(strtolower($partList[1]['content-type']), array('multipart/signed'/*, 'multipart/encrypted'*/)));
	}
	
	/**
	 * check if mail is encrypted
	 *
	 * @return bool
	 */
	function IsEncrypted()
	{
		$this->Parse();
		$partList = $this->_parsed->GetPartList();
		
		return(isset($partList[1]) && isset($partList[1]['content-type'])
				&& in_array(strtolower($partList[1]['content-type']), array('application/pkcs7-mime', 'application/x-pkcs7-mime')));
	}
	
	/**
	 * mark mail as trained
	 *
	 */
	function MarkAsTrained()
	{
		global $db;
		
		$db->Query('UPDATE {pre}mails SET trained=1 WHERE id=?',
			$this->_row['id']);
	}
	
	/**
	 * get header value
	 *
	 * @param string $key Field key
	 * @return string
	 */
	function GetHeaderValue($key)
	{
		$this->Parse();
		return($this->_parsed->rootPart->header->GetValueOnly($key));
	}
	
	/**
	 * get text parts
	 * 
	 * @return array
	 */
	function GetTextParts()
	{
		$this->Parse();
		$result = array('html' => '', 'text' => '');
		$parts = $this->GetPartList();
		
		foreach($parts as $id=>$info)
		{
			// we just need inline parts
			if(strtolower($info['content-disposition']) != 'attachment'
				&& (!isset($info['filename']) || trim($info['filename']) == 'unnamed'))
			{
				if(strtolower($info['content-type']) == 'text/plain'
					|| strtolower($info['content-type']) == 'text'
					|| strtolower($info['content-type']) == 'plain'
					|| strtolower($info['content-type']) == '')
				{
					$textBody = &$info['body'];
					$textBody->Init();
					$textPart = $textBody->DecodeBlock(-1);
					$textBody->Finish();
					$result['text'] .= CharsetDecode($textPart, $info['charset']);
				}
				else if(strtolower($info['content-type']) == 'text/html')
				{
					$htmlBody = &$info['body'];
					$htmlBody->Init();
					$htmlPart = $htmlBody->DecodeBlock(-1);
					$htmlBody->Finish();
					$result['html'] .= CharsetDecode($htmlPart, $info['charset']);
				}
			}
		}
		
		if($result['html'] == '')
			unset($result['html']);
		if($result['text'] == '')
			unset($result['text']);

		return($result);
	}
	
	/**
	 * get mail attachments
	 *
	 * @return array
	 */
	function GetAttachments()
	{
		global $VIEWABLE_TYPES;
		
		$this->Parse();
		$result = array();
		$parts = $this->GetPartList();
		$attachmentTypes = array('image', 'application');
		
		foreach($parts as $id=>$info)
		{
			list($primType) = explode('/', $info['content-type']);
			
			if(strtolower($info['content-disposition']) == 'attachment'
				|| in_array($primType, $attachmentTypes)
				|| (isset($info['filename']) && trim($info['filename']) != 'unnamed'))
			{
				$result[$id] = array(
					'filename'		=> $info['filename'],
					'size'			=> $this->EstimatePartSize($info),
					'mimetype'		=> $info['content-type'],
					'cid'			=> $info['content-id'],
					'viewable'		=> in_array(strtolower($info['content-type']), $VIEWABLE_TYPES)
				);
			}
		}
		
		return($result);
	}
	
	/**
	 * estimate part size
	 *
	 * @param array $info
	 * @return int
	 */
	function EstimatePartSize(&$info)
	{
		$result = $info['body']->offsetLength;
		
		switch($info['content-transfer-encoding'])
		{
		case 'base64':
			$result = round(((($result - 2 * ($result / 72)) / 4) * 3), 0);
			break;
		}
		
		return($result);
	}
	
	/**
	 * save attachment to file pointer
	 *
	 * @param string $key
	 * @param resource $fp
	 * @return bool
	 */
	function AttachmentToFP($key, $fp)
	{
		$parts = $this->GetPartList();
		if(isset($parts[$key]))
		{
			$part = $parts[$key];
			$attData = &$part['body'];
			$attData->Init();
			while($block = $attData->DecodeBlock(PART_CHUNK_SIZE))
			{
				fwrite($fp, $block);
			}
			$attData->Finish();
			fseek($fp, 0, SEEK_SET);
			return(true);
		}
		return(false);
	}
	
	/**
	 * get message as XML array
	 *
	 * @return array
	 */
	function GetXML()
	{
		$this->Parse();
		$result = array();
		
		$result['mail'] = array(
				'#nodeParam'		=> array(
					'id'				=> $this->id,
					'flags'				=> $this->flags,
					'from_name'			=> ExtractMailName($this->GetHeaderValue('from')),
					'from_mail'			=> ExtractMailAddress($this->GetHeaderValue('from')),
					'timestamp'			=> $this->date,
					'subject'			=> $this->GetHeaderValue('subject'),
					'priority'			=> $this->priority
				)
		);
		
		$result['recipients'] = array();
		$recps = array(
			'to'	=> ParseMailList($this->GetHeaderValue('to')),
			'cc'	=> ParseMailList($this->GetHeaderValue('cc')),
			'bcc'	=> ParseMailList($this->GetHeaderValue('bcc'))
		);

		foreach($recps as $key=>$val)
			foreach($val as $recp)
				$result['recipients'][] = array(
					'#nodeName'		=> $key,
					'#nodeParam'	=> array(
						'name'			=> $recp['name'],
						'mail'			=> $recp['mail'],
						'inbook'		=> 0
					)
				);
			
		$result['parts'] = array();
		$textParts = $this->GetTextParts();
		$partID = 0;
		foreach($textParts as $type=>$content)
			if($content != '')
				$result['parts'][] = array(
					'#nodeName'			=> 'part',
					'#nodeParam'		=> array(
						'type'				=> PART_TYPE_TEXT,
						'content'			=> $type,
						'id'				=> $partID++
					),
					'#nodeCDATA'		=> $content
				);
				
		$attachments = $this->GetAttachments();
		foreach($attachments as $key=>$att)
			$result['parts'][] = array (
				'#nodeName'				=> 'part',
				'#nodeParam'			=> array(
					'type'					=> PART_TYPE_ATTACHMENT,
					'content'				=> $att['filename'],
					'id'					=> $key,
					'size'					=> $att['size'],
					'mimetype'				=> $att['mimetype']
				)
			);
	 		
		return($result);
	}
	
	/**
	 * get message file pointer
	 * 
	 * @return resource
	 */
	function GetMessageFP($allowOverride = true)
	{
		if($this->storedIn == STORE_FILE)
		{
			if(!($fp = @fopen($this->GetMessageFilename(0, $allowOverride), 'r')))
			{
				PutLog(sprintf('Message file <%s> for message <%d> not found!',
							$this->GetMessageFilename(0, $allowOverride),
							$this->id),
						PRIO_ERROR,
						__FILE__,
						__LINE__);
				return(false);
			}
			return($fp);
		}
		else 
		{
			$tempFileID = RequestTempFile($this->_userID, time() + 5*TIME_ONE_MINUTE);
			$tempFileName = TempFileName($tempFileID);
			
			$fp = fopen($tempFileName, 'wb+');
			fwrite($fp, $this->_row['body']);
			fseek($fp, 0, SEEK_SET);
			
			$GLOBALS['tempFilesToReleaseAtShutdown'][] = array($this->_userID, $tempFileID);
			
			return($fp);
		}
	}
	
	/**
	 * get message size
	 *
	 * @return int
	 */
	function GetMessageSize($allowOverride = true)
	{
		if($this->storedIn == STORE_FILE)
			return(@filesize($this->GetMessageFilename(0, $allowOverride)));
		else 
			return(strlen($this->_row['body']));
	}
	
	/**
	 * get message filename
	 * 
	 * @return string
	 */
	function GetMessageFilename($id = 0, $allowOverride = true)
	{
		if($this->_msgFileName !== false && $allowOverride)
			return($this->_msgFileName);
		if($id == 0)
			$id = $this->id;
		return(DataFilename($id, 'msg'));
	}
	
	/**
	 * get message references
	 *
	 * @return array
	 */
	function GetReferences()
	{
		// get header values
		$references = $this->GetHeaderValue('references');
		$inReplyTo = $this->GetHeaderValue('in-reply-to');
		
		// extract message ids
		return(ExtractMessageIDs($references . ' ' . $inReplyTo));
	}
	
	/**
	 * parse some mail info
	 *
	 */
	function ParseInfo()
	{
		$this->Parse();
		
		// not trained
		$this->trained = false;
		
		// attachments?
		if(count($this->GetAttachments()) > 0
			&& ($this->flags & FLAG_ATTACHMENT) == 0)
			$this->flags |= FLAG_ATTACHMENT;
		
		// date?
		if($this->GetHeaderValue('date') != ''
			&& ($this->date = strtotime($this->GetHeaderValue('date'))) < 1)
			$this->date = time();
		
		// priority?
		switch((int)$this->GetHeaderValue('x-priority'))
		{
		case 1:
			$this->priority = ITEMPRIO_HIGH;
			break;
		
		case 5:
			$this->priority = ITEMPRIO_LOW;
			break;
			
		default:
			$this->priority = ITEMPRIO_NORMAL;
			break;
		}
	}
	
	/**
	 * generate and return mail thread
	 *
	 * @return array
	 */
	function GetThread()
	{
		if(!class_exists('BMMailThreader'))
			include(B1GMAIL_DIR . 'serverlib/mailthreader.class.php');
		
		// get thread
		$threader = _new('BMMailThreader', array($this->_userID));
		$thread = $threader->GetThread($this->id);
		return($thread);
	}
}
?>