<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: mailbox.class.php,v 1.104 2009/02/18 14:46:42 patrick Exp $
 *
 */

if(!class_exists('BMMail'))
	include(B1GMAIL_DIR . 'serverlib/mail.class.php');
if(!class_exists('BMSMS'))
	include(B1GMAIL_DIR . 'serverlib/sms.class.php');

/**
 * mailbox class
 */
class BMMailbox
{
	var $_userID;
	var $_userMail;
	var $_userObject;
	var $_userGroup;
	var $_intelligentFolders;
	var $_lastInsertId;
	var $_mailboxGeneration;
	var $_mailboxStructureGeneration;
	
	/**
	 * constructor
	 * 
	 * @param int $userID User ID
	 * @param string $userMail User's primary mail address
	 * @return BMMailbox
	 */
	function BMMailbox($userID, $userMail, $userObject)
	{
		$this->_userID = $userID;
		$this->_userMail = $userMail;
		$this->_userObject = $userObject;
		$this->_userGroup = false;
		$this->_mailboxGeneration = $userObject->_row['mailbox_generation'];
		$this->_mailboxStructureGeneration = $userObject->_row['mailbox_structure_generation'];
	}
	
	/**
	 * increment mailbox generation
	 *
	 */
	function IncGeneration()
	{
		global $db;
		
		$db->Query('UPDATE {pre}users SET mailbox_generation=mailbox_generation+1 WHERE id=?',
			$this->_userID);
		$this->_mailboxGeneration++;
	}
	
	/**
	 * increment mailbox structure generation
	 *
	 */
	function IncStructureGeneration()
	{
		global $db;
		
		$db->Query('UPDATE {pre}users SET mailbox_structure_generation=mailbox_structure_generation+1 WHERE id=?',
			$this->_userID);
		$this->_mailboxStructureGeneration++;
	}
	
	/**
	 * check if folder is an intelligent folder
	 *
	 * @param int $folderID Folder ID
	 * @return bool
	 */
	function IsIntelligentFolder($folderID)
	{
		global $db;
		
		if(!is_array($this->_intelligentFolders))
		{
			$res = $db->Query('SELECT id FROM {pre}folders WHERE intelligent=1 AND userid=?',
				$this->_userID);
			while($row = $res->FetchArray(MYSQL_ASSOC))
				$this->_intelligentFolders[$row['id']] = true;
			$res->Free();
		}
		
		return(isset($this->_intelligentFolders[$folderID]));
	}
	
	/**
	 * group a mail list
	 *
	 * @param array $mailList Mail list
	 * @param string $groupMode Group mode
	 * @return array
	 */
	function GroupMailList($mailList, $groupMode)
	{
		global $lang_user;
		
		$newList = array();
		
		$lastGroupValue = -1;
		$sepID = 0;
		foreach($mailList as $key=>$value)
		{
			if($groupMode == 'gelesen')
				$groupValue = ($value['row']['flags'] & FLAG_UNREAD) == 0 ? 1 : 0;
			else if($groupMode == 'beantwortet')
				$groupValue = ($value['row']['flags'] & FLAG_ANSWERED) != 0 ? 1 : 0;
			else if($groupMode == 'weitergeleitet')
				$groupValue = ($value['row']['flags'] & FLAG_FORWARDED) != 0 ? 1 : 0;
			else if($groupMode == 'attach')
				$groupValue = ($value['row']['flags'] & FLAG_ATTACHMENT) != 0 ? 1 : 0;
			else if($groupMode == 'flagged')
				$groupValue = ($value['row']['flags'] & FLAG_FLAGGED) != 0 ? 1 : 0;
			else if($groupMode == 'fetched')
				$groupValue = $value['row']['datum'];
			else
				$groupValue = $value['row'][$groupMode];
			
			if($groupMode != 'fetched')
			{
				if($groupValue != $lastGroupValue)
				{
					$text = '?';
					
					if($groupMode == 'gelesen')
					{
						$text = $lang_user['read'] . ': ' 
									. $lang_user[$groupValue ? 'yes' : 'no'];
					}
					else if($groupMode == 'beantwortet')
					{
						$text = $lang_user['answered'] . ': ' 
									. $lang_user[$groupValue ? 'yes' : 'no'];
					}
					else if($groupMode == 'weitergeleitet')
					{
						$text = $lang_user['forwarded'] . ': ' 
									. $lang_user[$groupValue ? 'yes' : 'no'];
					}
					else if($groupMode == 'attach')
					{
						$text = $lang_user['attachment'] . ': ' 
									. $lang_user[$groupValue ? 'yes' : 'no'];
					}
					else if($groupMode == 'flagged')
					{
						$text = $lang_user['flagged'] . ': ' 
									. $lang_user[$groupValue ? 'yes' : 'no'];
					}
					else if($groupMode == 'von')
					{
						$text = htmlentities($groupValue);
					}
					
					$newList[--$sepID] = array(
						'text'	=> $text
					);
				}
			}
			else
			{
				list($cat1) = categorizeDate($lastGroupValue);
				list($cat2, $cat2Array) = categorizeDate($groupValue);
				
				if($lastGroupValue == -1 || $cat1!=$cat2)
				{
					$newList[--$sepID] = array(
						'text'	=> $cat2Array['text'],
						'date'	=> $cat2Array['date']
					);
				}
			}
				
			$newList[$key] = $value;
			
			$lastGroupValue = $groupValue;
		}
		
		return($newList);
	}
	
	/**
	 * get mails per page for folder
	 *
	 * @param int $id Folder ID
	 */
	function GetMailsPerPage($id)
	{
		global $db, $bm_prefs;
		
		if($id <= 0)
		{
			$prefKey = 'perpage_' . $id;
			$value = $this->_userObject->GetPref($prefKey);
			if($value !== false && !empty($value))
				return($value);
			else 
				return((int)$bm_prefs['ordner_proseite']);
		}
		else 
		{
			$res = $db->Query('SELECT perpage FROM {pre}folders WHERE userid=? AND id=?',
				$this->_userID,
				$id);
			$row = $res->FetchArray(MYSQL_NUM);
			$res->Free();
			return(isset($row[0]) ? $row[0] : (int)$bm_prefs['ordner_proseite']);
		}
	}
	
	/**
	 * set mails per page
	 *
	 * @param int $id Folder ID
 	 * @param string $num New Number
	 * @return bool
	 */
	function SetMailsPerPage($id, $num)
	{
		global $db;
		
		$num = min(50, max(5, $num));
		
		if($id <= 0)
		{
			$prefKey = 'perpage_' . $id;
			return($this->_userObject->SetPref($prefKey, $num));			
		}
		else 
		{
			$db->Query('UPDATE {pre}folders SET perpage=? WHERE userid=? AND id=?',
				$num,
				$this->_userID,
				$id);
			return($db->AffectedRows() == 1);
		}
	}
	
	/**
	 * get group mode for folder
	 *
	 * @param int $id Folder ID
	 */
	function GetGroupMode($id)
	{
		global $db;
		
		if($id <= 0)
		{
			$prefKey = 'groupmode_' . $id;
			$value = $this->_userObject->GetPref($prefKey);
			if($value !== false && !empty($value))
				return($value);
			else 
				return('-');
		}
		else 
		{
			$res = $db->Query('SELECT group_mode FROM {pre}folders WHERE userid=? AND id=?',
				$this->_userID,
				$id);
			$row = $res->FetchArray(MYSQL_NUM);
			$res->Free();
			return(isset($row[0]) ? $row[0] : '-');
		}
	}
	
	/**
	 * set group mode
	 *
	 * @param int $id Folder ID
 	 * @param string $mode New mode
	 * @return bool
	 */
	function SetGroupMode($id, $mode)
	{
		global $db;
		
		if($id <= 0)
		{
			$prefKey = 'groupmode_' . $id;
			return($this->_userObject->SetPref($prefKey, $mode));			
		}
		else 
		{
			$db->Query('UPDATE {pre}folders SET group_mode=? WHERE userid=? AND id=?',
				$mode,
				$this->_userID,
				$id);
			return($db->AffectedRows() == 1);
		}
	}
	
	/**
	 * get folder title
	 *
	 * @param int $folderID Folder ID
	 * @return string
	 */
	function GetFolderTitle($folderID)
	{
		global $db;
		
		if($folderID < 1)
			return('');
			
		$res = $db->Query('SELECT titel FROM {pre}folders WHERE id=? AND userid=?',
			(int)$folderID,
			$this->_userID);
		if($res->RowCount() == 0)
			return('');
		list($title) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		return($title);
	}
	
	/**
	 * delete a folder
	 *
	 * @param int $id Folder ID
	 * @return bool
	 */
	function DeleteFolder($id)
	{
		global $db, $cacheManager;
		
		$cacheManager->Delete('folderList:0:' . $this->_userID . ':' . $this->_mailboxStructureGeneration);
		$cacheManager->Delete('folderList:1:' . $this->_userID . ':' . $this->_mailboxGeneration);
		$this->IncStructureGeneration();
		
		$db->Query('UPDATE {pre}folders SET parent=? WHERE parent=? AND userid=?',
			-1,
			(int)$id,
			$this->_userID);
		
		$db->Query('DELETE FROM {pre}folders WHERE id=? AND userid=?',
			(int)$id,
			$this->_userID);
		if($db->AffectedRows() == 1)
		{
			$db->Query('DELETE FROM {pre}folder_conditions WHERE folder=?',
				(int)$id);
			$db->Query('UPDATE {pre}mails SET folder=?,trashstamp=? WHERE userid=? AND folder=?',
				FOLDER_TRASH,
				time(),
				$this->_userID,
				$id);
			return(true);
		}
		
		return(false);
	}
	
	/**
	 * add a folder
	 *
	 * @param string $title Title
	 * @param int $parent Parent
	 * @param bool $subscribed Subscribed?
	 * @param int $storetime Storetime
	 * @param bool $intelligent Intelligent folder?
	 * @param bool $noDefaultCondition Do not create default condition?
	 * @return int
	 */
	function AddFolder($title, $parent, $subscribed, $storetime, $intelligent, $noDefaultCondition = false)
	{
		global $db, $cacheManager;
		
		$db->Query('INSERT INTO {pre}folders(userid,titel,parent,subscribed,storetime,intelligent) VALUES(?,?,?,?,?,?)',
			$this->_userID,
			$title,
			(int)$parent,
			$subscribed ? 1 : 0,
			$intelligent ? -1 : (int)$storetime,
			$intelligent ? 1 : 0);
		$id = $db->InsertID();
		
		if($id && $intelligent && !$noDefaultCondition)
			$this->AddCondition($id);
			
		$cacheManager->Delete('folderList:0:' . $this->_userID . ':' . $this->_mailboxStructureGeneration);
		$cacheManager->Delete('folderList:1:' . $this->_userID . ':' . $this->_mailboxGeneration);
		$this->IncStructureGeneration();
		
		return($id);
	}
	
	/**
	 * update folder
	 *
	 * @param int $folderID Folder ID
	 * @param string $title Title
	 * @param int $parent Parent
	 * @param bool $subscribed Subscribed?
	 * @param int $storetime Storetime
	 * @param int $intelligent_link Intelligent link type
	 * @return bool
	 */
	function UpdateFolder($folderID, $title, $parent, $subscribed, $storetime, $intelligent_link)
	{
		global $db, $cacheManager;
		
		$db->Query('UPDATE {pre}folders SET titel=?, parent=?, subscribed=?, storetime=?, intelligent_link=? WHERE id=? AND userid=?',
			$title,
			(int)$parent,
			$subscribed ? 1 : 0,
			(int)$storetime,
			(int)$intelligent_link,
			(int)$folderID,
			$this->_userID);
		
		$cacheManager->Delete('folderList:0:' . $this->_userID . ':' . $this->_mailboxStructureGeneration);
		$cacheManager->Delete('folderList:1:' . $this->_userID . ':' . $this->_mailboxGeneration);
		$this->IncStructureGeneration();
		
		return($db->AffectedRows() == 1);
	}
	
	/**
	 * get folder dataset
	 *
	 * @param int $folderID Folder ID
	 * @return array
	 */
	function GetFolder($folderID)
	{
		global $db;
			
		$res = $db->Query('SELECT id,titel,parent,subscribed,perpage,storetime,group_mode,intelligent,intelligent_link FROM {pre}folders WHERE id=? AND userid=?',
			(int)$folderID,
			$this->_userID);
		if($res->RowCount() == 0)
			return(false);
		$result = $res->FetchArray(MYSQL_ASSOC);
		$res->Free();
		
		return($result);
	}
	
	/**
	 * get intelligent folder conditions
	 *
	 * @param int $folderID Folder ID
	 * @return array
	 */
	function GetConditions($folderID)
	{
		global $db;
		
		$result = array();
		$res = $db->Query('SELECT id,field,op,val FROM {pre}folder_conditions WHERE folder=? ORDER BY id ASC',
			(int)$folderID);
		while($row = $res->FetchArray())
			$result[$row['id']] = $row;
		$res->Free();
		
		return($result);
	}
	
	/**
	 * delete intelligent folder condition
	 *
	 * @param int $conditionID Condition ID
	 * @param int $folderID Folder ID
	 * @return bool
	 */
	function DeleteCondition($conditionID, $folderID)
	{
		global $db;
		
		$db->Query('DELETE FROM {pre}folder_conditions WHERE id=? AND folder=?',
			(int)$conditionID,
			(int)$folderID);
		return($db->AffectedRows() == 1);
	}
	
	/**
	 * add intelligent folder condition
	 *
	 * @param int $folderID Folder ID
	 * @return int
	 */
	function AddCondition($folderID)
	{
		global $db;
		
		$db->Query('INSERT INTO {pre}folder_conditions(folder,field,op,val) VALUES(?,?,?,?)',
			(int)$folderID,
			1,
			1,
			'');
		return($db->InsertID());
	}
	
	/**
	 * update intelligent folder condition
	 *
	 * @param int $conditionID Condition ID
	 * @param int $folderID Folder ID
	 * @param int $field Field constant
	 * @param int $op Op constant
	 * @param string $val Value
	 * @return bool
	 */
	function UpdateCondition($conditionID, $folderID, $field, $op, $val)
	{
		global $db;
		
		$db->Query('UPDATE {pre}folder_conditions SET field=?,op=?,val=? WHERE id=? AND folder=?',
			(int)$field,
			(int)$op,
			$val,
			(int)$conditionID,
			(int)$folderID);
		return($db->AffectedRows() == 1);
	}
	
	/**
	 * get user folder list
	 *
	 * @param string $sortColumn Sort column
	 * @param string $sortOrder Sort order
	 * @param bool $withParentTitle Get title of parent folder?
	 * @param bool $withStats With stats?
	 * @return array
	 */
	function GetUserFolderList($sortColumn = 'title', $sortOrder = 'ASC', $withParentTitle = false, $withStats = false)
	{
		global $db;
		
		$result = array();
		$res = $db->Query('SELECT id,titel,parent,subscribed,perpage,storetime,group_mode,intelligent,intelligent_link FROM {pre}folders WHERE userid=? ORDER BY ' . $sortColumn . ' ' . $sortOrder,
			$this->_userID);
		while($row = $res->FetchArray(MYSQL_ASSOC))
		{
			if($withParentTitle)
				$row['parent'] = $this->GetFolderTitle($row['parent']);	
			if($withStats)
			{
				$row['allMails'] = $this->GetMailCount($row['id']);
				$row['unreadMails'] = $this->GetMailCount($row['id'], true);
				$row['flaggedMails'] = $this->GetMailCount($row['id'], false, true);
				$row['size'] = $this->GetFolderSize($row['id']);
			}
			$result[$row['id']] = $row;
		}
		$res->Free();
		
		return($result);
	}
	
	/**
	 * get system folder list
	 *
	 * @return array
	 */
	function GetSysFolderList()
	{
		global $db, $lang_user;
		
		$result = array();
		
		// default folders
		$result[FOLDER_INBOX] = array(
			'titel'			=> $lang_user['inbox'],
			'type'			=> 'inbox',
			'size'			=> $this->GetFolderSize(FOLDER_INBOX),
			'allMails'		=> $this->GetMailCount(FOLDER_INBOX),
			'unreadMails'	=> $this->GetMailCount(FOLDER_INBOX, true),
			'flaggedMails'	=> $this->GetMailCount(FOLDER_INBOX, false, true),
		);
		$result[FOLDER_OUTBOX] = array(
			'titel'			=> $lang_user['outbox'],
			'type'			=> 'outbox',
			'size'			=> $this->GetFolderSize(FOLDER_OUTBOX),
			'allMails'		=> $this->GetMailCount(FOLDER_OUTBOX),
			'unreadMails'	=> $this->GetMailCount(FOLDER_OUTBOX, true),
			'flaggedMails'	=> $this->GetMailCount(FOLDER_OUTBOX, false, true),
		);
		$result[FOLDER_DRAFTS] = array(
			'titel'			=> $lang_user['drafts'],
			'type'			=> 'drafts',
			'size'			=> $this->GetFolderSize(FOLDER_DRAFTS),
			'allMails'		=> $this->GetMailCount(FOLDER_DRAFTS),
			'unreadMails'	=> $this->GetMailCount(FOLDER_DRAFTS, true),
			'flaggedMails'	=> $this->GetMailCount(FOLDER_DRAFTS, false, true),
		);
		$result[FOLDER_SPAM] = array(
			'titel'			=> $lang_user['spam'],
			'type'			=> 'spam',
			'size'			=> $this->GetFolderSize(FOLDER_SPAM),
			'allMails'		=> $this->GetMailCount(FOLDER_SPAM),
			'unreadMails'	=> $this->GetMailCount(FOLDER_SPAM, true),
			'flaggedMails'	=> $this->GetMailCount(FOLDER_SPAM, false, true),
		);
		$result[FOLDER_TRASH] = array(
			'titel'			=> $lang_user['trash'],
			'type'			=> 'trash',
			'size'			=> $this->GetFolderSize(FOLDER_TRASH),
			'allMails'		=> $this->GetMailCount(FOLDER_TRASH),
			'unreadMails'	=> $this->GetMailCount(FOLDER_TRASH, true),
			'flaggedMails'	=> $this->GetMailCount(FOLDER_TRASH, false, true),
		);
		
		return($result);
	}
	
	/**
	 * get folder size
	 *
	 * @param int $folderID Folder ID
	 * @return int
	 */
	function GetFolderSize($folderID)
	{
		global $db;
		
		$res = $db->Query('SELECT SUM(size) FROM {pre}mails WHERE folder=? AND userid=?',
			$folderID,
			$this->_userID);
		list($folderSize) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		return($folderSize);
	}

	/**
	 * change subscribed-status of a folder
	 *
	 * @param int $folderID Folder ID
	 * @param bool $subscribe Subscribe?
	 * @return bool
	 */
	function SubscribeFolder($folderID, $subscribe = true)
	{
		global $db, $cacheManager;
		
		$db->Query('UPDATE {pre}folders SET subscribed=? WHERE id=? AND userid=?',
			$subscribe ? 1 : 0,
			(int)$folderID,
			$this->_userID);
			
		$cacheManager->Delete('folderList:0:' . $this->_userID . ':' . $this->_mailboxStructureGeneration);
		$cacheManager->Delete('folderList:1:' . $this->_userID . ':' . $this->_mailboxGeneration);
		$this->IncStructureGeneration();
		
		return($db->AffectedRows() == 1);
	}
	
	/**
	 * fetch structured folder list for use in dropdown
	 * (parameters only used in internal recursion)
	 * 
	 * @return array
	 */
	function GetDropdownFolderList($parent = -1, &$result, $level = 0, $more = 0)
	{
		global $db, $lang_user;
		
		$returnArray = $result == null;
		if($result == null)
			$result = array();
			
		if($parent == -1)
		{
			$result[FOLDER_INBOX]	= $lang_user['inbox'];
			$result[FOLDER_OUTBOX]	= $lang_user['outbox'];
			$result[FOLDER_SPAM]	= $lang_user['spam'];
			$result[FOLDER_TRASH]	= $lang_user['trash'];
		}
		
		$folderCount = 0;
		$res = $db->Query('SELECT titel,id FROM {pre}folders WHERE userid=? AND intelligent=0 AND parent=? ORDER BY titel ASC',	
			$this->_userID,
			$parent);
		$rowCount = $res->RowCount();
		while($row = $res->FetchArray(MYSQL_ASSOC))
		{
			$folderCount++;
			$lastItem = $folderCount == $rowCount;
			
			if($level > 0)
			{
				$indent = str_repeat('&nbsp;&nbsp;&nbsp;', max(0, $more-1))
							. str_repeat('|&nbsp;', $level-$more);
				if($lastItem)
					$indent .= '\-- ';
				else
					$indent .= '|-- ';
			}
			else
				$indent = '';
			
			$result[$row['id']] = trim($indent . HTMLFormat($row['titel']));
			if($this->GetDropdownFolderList($row['id'], $result, $level+1, $lastItem || $level == 0 ? $more+1 : $more) > 0)
				$result[$row['id']] = str_replace('|-- ', '|-+ ', $result[$row['id']]);
		}
		$res->Free();
		
		if($returnArray)
			return($result);
		else
			return($folderCount);
	}
	
	/**
	 * fetch folder list for user
	 * 
	 * @param bool $withUnreadCount Calculate unread count?
	 * @return array
	 */
	function GetFolderList($withUnreadCount = false, $encode = true)
	{
		global $db, $lang_user, $cacheManager;
		
		$cacheKey = sprintf('folderList:%d:%d:%d',
			$withUnreadCount ? 1 : 0,
			$this->_userID,
			$withUnreadCount ? $this->_mailboxGeneration : $this->_mailboxStructureGeneration);
		
		if(!($result = $cacheManager->Get($cacheKey)))
		{
			$result = array();
			
			// default folders
			$result[FOLDER_INBOX] = array(
				'parent'		=> -1,
				'title'			=> $lang_user['inbox'],
				'type'			=> 'inbox',
				'intelligent'	=> false
			);
			$result[FOLDER_OUTBOX] = array(
				'parent'		=> -1,
				'title'			=> $lang_user['outbox'],
				'type'			=> 'outbox',
				'intelligent'	=> false
			);
			$result[FOLDER_DRAFTS] = array(
				'parent'		=> -1,
				'title'			=> $lang_user['drafts'],
				'type'			=> 'drafts',
				'intelligent'	=> false
			);
			$result[FOLDER_SPAM] = array(
				'parent'		=> -1,
				'title'			=> $lang_user['spam'],
				'type'			=> 'spam',
				'intelligent'	=> false
			);
			$result[FOLDER_TRASH] = array(
				'parent'		=> -1,
				'title'			=> $lang_user['trash'],
				'type'			=> 'trash',
				'intelligent'	=> false
			);
			
			// user folders
			$res = $db->Query('SELECT parent,titel,intelligent,id FROM {pre}folders WHERE userid=? AND subscribed=1 ORDER BY titel ASC',
				$this->_userID);
			while($row = $res->FetchArray(MYSQL_ASSOC))
			{
				$result[$row['id']] = array(
					'parent'		=> $row['parent'],
					'title'			=> $row['titel'],
					'type'			=> $row['intelligent'] == 1 ? 'intellifolder' : 'folder',
					'intelligent'	=> $row['intelligent'] == 1
				);
			}
			$res->Free();
			
			// unread count
			if($withUnreadCount)
				foreach($result as $key=>$val)
					$result[$key]['unread'] = $this->GetMailCount($key, true);
					
			$cacheManager->Set($cacheKey, $result, TIME_ONE_DAY);
		}
		else 
		{
			// apply language file
			$result[FOLDER_INBOX]['title'] 	= $lang_user['inbox'];
			$result[FOLDER_OUTBOX]['title'] = $lang_user['outbox'];
			$result[FOLDER_DRAFTS]['title'] = $lang_user['drafts'];
			$result[FOLDER_SPAM]['title'] 	= $lang_user['spam'];
			$result[FOLDER_TRASH]['title'] 	= $lang_user['trash'];
		}
		
		// encode?
		if($encode)
			foreach($result as $key=>$val)
				if($key>0)
					$result[$key]['title'] = htmlentities($result[$key]['title']);
		
		return($result);
	}
	
	/**
	 * rename folder
	 * 
	 * @param int $folderID
	 * @param string $newName
	 * @return int
	 */
	function RenameFolder($folderID, $newName)
	{
		global $db, $cacheManager;
		
		// long enough?
		$newName = trim($newName);
		if(strlen($newName) < 1)
			return(2);
		
		// get folder parent
		$res = $db->Query('SELECT parent FROM {pre}folders WHERE id=? AND userid=?',
			$folderID,
			$this->_userID);
		list($parent) = $res->FetchArray();
		$res->Free();
		
		// check if name is available
		$res = $db->Query('SELECT COUNT(*) FROM {pre}folders WHERE parent=? AND titel=? AND userid=?',
			$parent,
			$newName,
			$this->_userID);
		list($count) = $res->FetchArray();
		$res->Free();
		
		if($count == 0)
		{
			// okay!
			$db->Query('UPDATE {pre}folders SET titel=? WHERE id=? AND userid=?',
				$newName,
				$folderID,
				$this->_userID);
				
			$cacheManager->Delete('folderList:0:' . $this->_userID . ':' . $this->_mailboxStructureGeneration);
			$cacheManager->Delete('folderList:1:' . $this->_userID . ':' . $this->_mailboxGeneration);
			$this->IncStructureGeneration();
		
			return(1);
		}
		else 
		{
			// exists...
			return(0);
		}
	}
	
	/**
	 * get count of mails in folder
	 *
	 * @param int $folderID
	 * @param bool $unread Only count unread mails?
	 * @param bool $flagged Only count flagged mails?
	 * @return int
	 */
	function GetMailCount($folderID, $unread = false, $flagged = false)
	{
		global $db;
		
		$cond = ($folderID == -1)
					? '1'
					: $this->FolderCondition($folderID);
		if($unread)
			$cond .= ' AND (flags&'.FLAG_UNREAD.')!=0';
		if($flagged)
			$cond .= ' AND (flags&'.FLAG_FLAGGED.')!=0';
		
		$res = $db->Query('SELECT COUNT(*) FROM {pre}mails WHERE userid=? AND ' . $cond,
			$this->_userID);
		$row = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		return($row[0]);
	}
	
	/**
	 * get group order by - statement
	 *
	 * @param string $groupMode Group mode
	 * @return string
	 */
	function GetGroupOrderBy($groupMode)
	{
		if($groupMode == 'fetched')
			return('datum');
		else if($groupMode == 'von')
			return('von');
		else if($groupMode == 'gelesen')
			return('(flags&'.FLAG_UNREAD.')=0');
		else if($groupMode == 'beantwortet')
			return('(flags&'.FLAG_ANSWERED.')=0');
		else if($groupMode == 'weitergeleitet')
			return('(flags&'.FLAG_FORWARDED.')=0');
		else if($groupMode == 'flagged')
			return('(flags&'.FLAG_FLAGGED.')=0');
		else if($groupMode == 'attach')
			return('(flags&'.FLAG_ATTACHMENT.')=0');
		else 
			return('datum');
	}
	
	/**
	 * get mail list (IDs only)
	 *
	 * @param int $folderID
	 * @return array
	 */
	function GetMailIDList($folderID)
	{
		global $db;
		$result = array();
		
		$condition = $this->FolderCondition($folderID);
		$res = $db->Query('SELECT id FROM {pre}mails WHERE userid=? AND ' .  $condition,
			$this->_userID);
		while($row = $res->FetchArray(MYSQL_ASSOC))
			$result[] = $row['id'];
		$res->Free();
		
		return($result);
	}
	
	/**
	 * get mail list
	 *
	 * @param int $folderID
	 * @return array
	 */
	function GetMailList($folderID, $page = 1, $mailsPerPage = -1, $sortField = 'fetched', $sortBy = 'DESC', $groupMode = '-')
	{
		global $db;
		$result = array();
		
		if($groupMode == 'fetched')
			$groupMode = '-';
		if($sortField == 'fetched')
			$sortField = 'datum';
		
		ModuleFunction('OnStartMailList', array($this->_userID, $folderID == FOLDER_DRAFTS));
		
		$condition = $this->FolderCondition($folderID);
		$res = $db->Query('SELECT id,von,an,betreff,datum,flags,priority,size FROM {pre}mails WHERE userid=? AND ' . $condition . ' '
					. 'ORDER BY ' . ($groupMode != '-' ? $this->GetGroupOrderBy($groupMode) . ', ' : ''). $sortField . '  ' . $sortBy
					. ($mailsPerPage != -1 ? ' LIMIT ' . (($page-1)*$mailsPerPage) . ','. (int)$mailsPerPage : ''),
					$this->_userID);
		while($row = $res->FetchArray(MYSQL_ASSOC))
		{
			$to = ParseMailList($row['an']);
			if(count($to) == 0)
				$to_name = $to_mail = $row['an'];
			else
			{
				$to_name = (isset($to[0]['name']) && trim($to[0]['name']) != ''
					? $to[0]['name']
					: $to[0]['mail']) . (count($to) > 1 
					? ', ...'
					: '');
				$to_mail = $to[0]['mail'];
			}
			
			$result[$row['id']] = array(
				'subject'	=> $row['betreff'],
				'timestamp'	=> $row['datum'],
				'from_name'	=> ExtractMailName($row['von']),
				'from_mail'	=> ExtractMailAddress($row['von']),
				'to_name'	=> $to_name,
				'to_mail'	=> $to_mail,
				'priority'	=> $row['priority'] == 'high' ? ITEMPRIO_HIGH : ($row['priority'] == 'low' ? ITEMPRIO_LOW : ITEMPRIO_NORMAL),
				'flags'		=> $row['flags'],
				'row'		=> $row,
				'from'		=> $row['von'],
				'to'		=> $row['an'],
				'size'		=> $row['size']
			);
			ModuleFunction('OnGetMail', array($row['id'], $this->_userID));
		}
		$res->Free();
		
		ModuleFunction('OnEndMailList', array($this->_userID, $folderID == FOLDER_DRAFTS));
		
		return($result);
	}
	
	/**
	 * get previous and next mail
	 *
	 * @param int $folderID
	 * @param int $mailID
	 * @return array
	 */
	function GetPrevNextMail($folderID, $mailID)
	{
		global $db;
		
		// prev
		$res = $db->Query('SELECT id FROM {pre}mails WHERE id<? AND userid=? AND folder=? ORDER BY id DESC LIMIT 1',
							$mailID,
							$this->_userID,
							$folderID);
		if($res->RowCount() == 1)
			list($prevID) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		// next
		$res = $db->Query('SELECT id FROM {pre}mails WHERE id>? AND userid=? AND folder=? ORDER BY id ASC LIMIT 1',
							$mailID,
							$this->_userID,
							$folderID);
		if($res->RowCount() == 1)
			list($nextID) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		return(array(isset($prevID) ? $prevID : -1, isset($nextID) ? $nextID : -1));
	}
	
	/**
	 * build SQL condition for intelligent folder
	 *
	 * @param int $folderID Intelligent folder ID
	 * @return string
	 */
	function IntelligentFolderCondition($folderID)
	{
		global $db;
		
		$conditions = array();
		
		// get link mode
		$res = $db->Query('SELECT intelligent_link FROM {pre}folders WHERE id=? AND userid=?',
			(int)$folderID,
			$this->_userID);
		assert('$res->RowCount() != 0');
		list($linkMode) = $res->FetchArray();
		$res->Free();
		
		// get condition
		$res = $db->Query('SELECT field,op,val FROM {pre}folder_conditions WHERE folder=?',
			(int)$folderID);
		while($row = $res->FetchArray(MYSQL_NUM))
		{
			list($field, $op, $val) = $row;
			
			// field & op valid?
			if(($field > 0 && $field <= MAILFIELD_FOLDER)
				&& ($op > 0 && $op <= BMOP_ENDSWITH))
			{
				if(isset($condition))
					unset($condition);
					
				// prepare value
				$sqlVal = mysql_escape_string($val);
				
				// prepare field
				if($field == MAILFIELD_SUBJECT)
					$sqlField = 'betreff';
				else if($field == MAILFIELD_FROM)
					$sqlField = 'von';
				else if($field == MAILFIELD_TO)
					$sqlField = 'an';
				else if($field == MAILFIELD_CC)
					$sqlField = 'cc';
				else if($field == MAILFIELD_READ)
					$condition = '(flags&'.FLAG_UNREAD.')' . ($sqlVal == 'no' ? '!=00' : '=0');
				else if($field == MAILFIELD_ANSWERED)
					$condition = '(flags&'.FLAG_ANSWERED.')' . ($sqlVal == 'no' ? '=0' : '!=0');
				else if($field == MAILFIELD_FORWARDED)
					$condition = '(flags&'.FLAG_FORWARDED.')' . ($sqlVal == 'no' ? '=0' : '!=0');
				else if($field == MAILFIELD_PRIORITY)
					$sqlField = 'priority';
				else if($field == MAILFIELD_ATTACHMENT)
					$condition = '(flags&'.FLAG_ATTACHMENT.')' . ($sqlVal == 'no' ? '=0' : '!=0');
				else if($field == MAILFIELD_FLAGGED)
					$condition = '(flags&'.FLAG_FLAGGED.')' . ($sqlVal == 'no' ? '=0' : '!=0');
				else if($field == MAILFIELD_FOLDER)
					$sqlField = 'folder';
				else 
					$sqlField = '0';
			
				if(!isset($condition))
				{
					// prepare op
					if($op == BMOP_EQUAL)
						$sqlOp = '=';
					else if($op == BMOP_NOTEQUAL)
						$sqlOp = '!=';
					else if($op == BMOP_CONTAINS)
					{
						$sqlOp = ' LIKE ';
						$sqlVal = '%' . $sqlVal . '%';
					}
					else if($op == BMOP_NOTCONTAINS)
					{
						$sqlOp = ' NOT LIKE ';
						$sqlVal = '%' . $sqlVal . '%';
					}
					else if($op == BMOP_STARTSWITH)
					{
						$sqlOp =  ' LIKE ';
						$sqlVal .= '%';
					}
					else if($op == BMOP_ENDSWITH)
					{
						$sqlOp = ' LIKE ';
						$sqlVal = '%' . $sqlVal;
					}
					
					// build condition!
					$condition = sprintf('%s%s\'%s\'',
						$sqlField,
						$sqlOp,
						$sqlVal);
				}
				
				$conditions[] = $condition;
			}
		}
		$res->Free();
		
		// return SQL string
		if(count($conditions) > 0)
			return('(' . implode($linkMode == BMLINK_AND ? ' AND ' : ' OR ', $conditions) . ')');
		else 
			return('(0)');
	}
	
	/**
	 * get SQL condition for folder
	 *
	 * @param int $folderID Folder-ID (special or user)
	 * @param bool $moveCondition Move-Condition?
	 * @return string
	 */
	function FolderCondition($folderID, $moveCondition = false)
	{
		global $db;
		
		$result = 'folder=\'' . (int)$folderID . '\'';
		
		if($folderID > 0)
		{
			// this may be an intelligent folder			
			if(!$moveCondition && $this->IsIntelligentFolder((int)$folderID))
				$result = $this->IntelligentFolderCondition($folderID);
		}
		
		if($moveCondition && $folderID == FOLDER_TRASH)
			$moveCondition .= ',trashstamp=\'' . time() . '\'';
		
		if($moveCondition)
			return($result);
		else
			return('(' . $result . ')');
	}

	/**
	 * check if folder exists
	 *
	 * @param int $folderID
	 * @return bool
	 */
	function FolderExists($folderID)
	{
		global $db;
		
		if(in_array($folderID, array(FOLDER_INBOX, FOLDER_OUTBOX, FOLDER_DRAFTS, FOLDER_SPAM, FOLDER_TRASH)))
			return(true);
			
		$res = $db->Query('SELECT COUNT(*) FROM {pre}folders WHERE userid=? AND id=?',
			$this->_userID,
			$folderID);
		list($count) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		return($count == 1);
	}
	
	/**
	 * set a flag
	 * 
	 * @param int $flag
	 * @param bool $value
	 * @param int $mail
	 * @return int New message flags
	 */
	function FlagMail($flag, $value, $mail)
	{
		global $db;
		
		// get current flags
		$res = $db->Query('SELECT flags FROM {pre}mails WHERE id=? AND userid=? LIMIT 1',
			$mail,
			$this->_userID);
		if($res->RowCount() == 0)
			return(false);
		$row = $res->FetchArray();
		$res->Free();
		$currentFlags = $newFlags = $row['flags'];
		
		// change flag
		if($value && (($currentFlags & $flag) == 0))
			$newFlags |= $flag;
		else if(!$value && (($currentFlags & $flag) != 0))
			$newFlags ^= $flag;
			
		// update needed?
		if($newFlags != $currentFlags
			&& $this->SetMessageFlags($mail, $newFlags))
		{
			return($newFlags);
		}
		else 
		{
			return($currentFlags);
		}
	}
	
	/**
	 * set message flags
	 * 
	 * @param int $mail
	 * @param int $flags
	 * @return bool
	 */
	function SetMessageFlags($mail, $flags)
	{
		global $db, $cacheManager;
		
		// update row
		$db->Query('UPDATE {pre}mails SET flags=? WHERE id=? AND userid=?',
			$flags,
			$mail,
			$this->_userID);
			
		$cacheManager->Delete('folderList:1:' . $this->_userID . ':' . $this->_mailboxGeneration);
		$this->IncGeneration();
		
		// success
		if($db->AffectedRows() == 1)
 		{
 			// plugins
 			ModuleFunction('AfterChangeMailFlags', array($mail, $flags, &$this));
 			
 			return(true);
 		}
 		else
 			return(false);		
	}
	
	/**
	 * move message(s)
	 * 
	 * @param mixed $mails
	 * @param int $destFolder
	 * @return int count of moved mails
	 */
	function MoveMail($mails, $destFolder)
	{
		global $db, $cacheManager;
		
		// array required
		if(!is_array($mails))
			$mails = array($mails);
			
		// int!
		foreach($mails as $key=>$value)	
			$mails[$key] = (int)$value;
		
		// query part
		$queryPart = $this->FolderCondition((int)$destFolder, true);
		
		// execute query
		$db->Query('UPDATE {pre}mails SET ' . $queryPart . ' WHERE userid=? AND id IN (' . implode(',', $mails) . ')',
			$this->_userID);
			
		$cacheManager->Delete('folderList:1:' . $this->_userID . ':' . $this->_mailboxGeneration);
		$this->IncGeneration();
		
		// result count
 		$result = $db->AffectedRows();
 		
 		// plugins
 		ModuleFunction('AfterMoveMails', array($mails, $destFolder, &$this));
 		
 		return($result);		
	}
	
	/**
	 * get BMMail-instance for message $id
	 *
	 * @param int $id ID
	 * @return BMMail
	 */
	function GetMail($id)
	{
		global $db;
		
		$result = false;
		$res = $db->Query('SELECT * FROM {pre}mails WHERE id=? AND userid=? LIMIT 1',
			$id,
			$this->_userID);
		while($row = $res->FetchArray(MYSQL_ASSOC))
			$result = _new('BMMail', array($this->_userID, $row));
		$res->Free();
		
		$group = $this->_userObject->GetGroup();
		
		if($group->_row['smime'] == 'yes')
		{
			$smimeStatus = SMIME_UNKNOWN;
			
			$i = 0;
			while($result->IsSigned() || $result->IsEncrypted())
			{
				if($result->IsSigned())
				{
					if(!class_exists('BMSMIME'))
						include(B1GMAIL_DIR . 'serverlib/smime.class.php');
					$smime = _new('BMSMIME', array($this->_userID, &$this->_userObject));
					$res = $smime->CheckMailSignature($result);
					$smimeStatus |= $res[0];
					if(isset($res[1]) && $res[1] && is_object($res[1]))
						$result = $res[1];
					if(isset($res[2]))
						$result->smimeCertificateHash = $res[2];
				}
				
				if($result->IsEncrypted())
				{
					if(!class_exists('BMSMIME'))
						include(B1GMAIL_DIR . 'serverlib/smime.class.php');
					$smime = _new('BMSMIME', array($this->_userID, &$this->_userObject));
					$res = $smime->DecryptMail($result);
					$smimeStatus |= $res[0];
					if(isset($res[1]) && $res[1] && is_object($res[1]))
						$result = $res[1];
				}
				
				if(++$i == 5)
				{
					PutLog(sprintf('Breaking S/MIME processing loop for message %d (this should not happen)',
						$id),
						PRIO_WARNING,
						__FILE__,
						__LINE__);
					break;
				}
			}
			
			$result->smimeStatus = $smimeStatus;
		}
		
		return($result);
	}
	
	/**
	 * check if mail exists in mailbox
	 *
	 * @param int $id ID
	 * @return bool
	 */
	function MailExists($id)
	{
		global $db;
		
		$res = $db->Query('SELECT COUNT(*) FROM {pre}mails WHERE id=? AND userid=?',
			$id,
			$this->_userID);
		list($mailCount) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		return($mailCount == 1);
	}
	
	/**
	 * get count of recent mails and reset it
	 *
	 * @return int
	 */
	function GetRecentMailCount($folder = FOLDER_ROOT)
	{
		global $db;
		
		$userInfo = BMUser::Fetch($this->_userID);
		
		// select new mails
		$res = $db->Query('SELECT COUNT(*) FROM {pre}mails WHERE '
			. ($folder == FOLDER_ROOT
				? '(folder=' . FOLDER_INBOX . ' OR folder>0)'
				: '(folder=' . (int)$folder . ')')
			. ' AND (flags&'.FLAG_UNREAD.')!=0 AND userid=? AND fetched>=?',
			$userInfo['id'],
			$userInfo['last_notify']);
		$row = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		// update last notify date
		$this->UpdateLastNotifyDate();
		
		// return
		return($row[0]);
	}
	
	/**
	 * update last notify date
	 *
	 */
	function UpdateLastNotifyDate()
	{
		global $db;
		$db->Query('UPDATE {pre}users SET last_notify=? WHERE id=?',
			time(),
			$this->_userID);
	}
	
	/**
	 * update space
	 *
	 * @param int $bytes Bytes (negative or positive)
	 * @return boolean
	 */
	function UpdateSpace($bytes)
	{
		global $db;
		
		if($bytes < 0)
		{
			$db->Query('UPDATE {pre}users SET mailspace_used=mailspace_used-' . abs($bytes) . ' WHERE id=?',
				$this->_userID);
			$this->_userObject->_row['mailspace_used'] -= abs($bytes);
		}
		else if($bytes > 0)
		{
			$db->Query('UPDATE {pre}users SET mailspace_used=mailspace_used+' . abs($bytes) . ' WHERE id=?',
				$this->_userID);
			$this->_userObject->_row['mailspace_used'] += abs($bytes);
		}
		
		return(true);
	}
	
	/**
	 * get user space limit
	 *
	 * @return int
	 */
	function GetSpaceLimit()
	{
		global $userRow, $groupRow;
		
		if(isset($userRow) && $userRow['id'] == $this->_userID)
		{
			return($groupRow['storage']);
		}
		else
		{
			if($this->_userGroup === false)
				$this->_userGroup = $this->_userObject->GetGroup();
			$myGroupRow = $this->_userGroup->Fetch();
			return($myGroupRow['storage']);
		}
	}
	
	/**
	 * get space limit per mail item
	 *
	 * @return int
	 */
	function GetMaxMailSize()
	{
		global $userRow, $groupRow;
		
		if(isset($userRow) && $userRow['id'] == $this->_userID)
		{
			return($groupRow['maxsize']);
		}
		else
		{
			if($this->_userGroup === false)
				$this->_userGroup = $this->_userObject->GetGroup();
			$myGroupRow = $this->_userGroup->Fetch();
			return($myGroupRow['maxsize']);
		}		
	}
	
	/**
	 * get used space
	 * 
	 * @return int
	 */
	function GetUsedSpace()
	{
		return($this->_userObject->_row['mailspace_used']);
	}
	
	/**
	 * empty a folder
	 *
	 * @param int $id Folder ID
	 * @return int Deleted mail count
	 */
	function EmptyFolder($id)
	{
		$deletedMails = 0;
		$mails = $this->GetMailIDList($id);
		foreach($mails as $mailID)
			if($this->DeleteMail($mailID))
				$deletedMails++;
		return($deletedMails);
	}
	
	/**
	 * set mail spam status
	 *
	 * @param int $id
	 * @param bool $spam
	 * @return bool
	 */
	function SetSpamStatus($id, $spam)
	{
		global $bm_prefs;
		
		$mail = $this->GetMail($id);
		
		if($mail !== false)
		{
			if($bm_prefs['use_bayes'] == 'yes'
				&& !$mail->trained)
			{
				if(!class_exists('BMMailFilter_Bayes'))
					include(B1GMAIL_DIR . 'serverlib/filters.inc.php');
				
				if($bm_prefs['bayes_mode'] == 'local')
					$filter = _new('BMMailFilter_Bayes', array($mail, $this->_userID));
				else 
					$filter = _new('BMMailFilter_Bayes', array($mail));
				$filter->Train($spam);
			}
			
			$this->FlagMail(FLAG_SPAM, $spam, $id);
			
			if($spam)
				$this->FlagMail(FLAG_UNREAD, false, $id);
			
			if(!$spam
				&& $mail->_row['folder'] == $this->_userObject->_row['spamaction'])
				$this->MoveMail($id, FOLDER_INBOX);
			else if($spam
					&& $mail->_row['folder'] == FOLDER_INBOX
					&& $this->_userObject->_row['spamaction'] != -1
					&& $this->FolderExists($this->_userObject->_row['spamaction']))
				$this->MoveMail($id, $this->_userObject->_row['spamaction']);
		}
		
		return(false);
	}
	
	/**
	 * delete a mail
	 *
	 * @param int $id Mail ID
	 * @param bool $hard Delete without trashing?
	 * @return bool
	 */
	function DeleteMail($id, $hard = false)
	{
		global $db, $cacheManager;
		
		// get msg state
		$res = $db->Query('SELECT folder,size,body FROM {pre}mails WHERE userid=? AND id=?',
			$this->_userID,
			(int)$id);
		if($res->RowCount() != 1)
			return(false);
		list($folder, $messageSize, $body) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		// trashed?
		if($folder == FOLDER_TRASH || $hard)
		{
			// delete mail
			if($body == 'file')
			{
				$messageFile = DataFilename((int)$id);
				if(!@unlink($messageFile))
					PutLog(sprintf('Failed to delete message file <%s> of message <%d>',
								$messageFile,
								$id),
							PRIO_WARNING,
							__FILE__,
							__LINE__);
			}
			$db->Query('DELETE FROM {pre}certmails WHERE mail=? AND user=?',
				(int)$id,
				$this->_userID);
			$db->Query('DELETE FROM {pre}mails WHERE id=?',
				(int)$id);
			$this->UpdateSpace($messageSize*-1);
			
			$cacheManager->Delete('folderList:1:' . $this->_userID . ':' . $this->_mailboxGeneration);
			$this->IncGeneration();

			// plugins
 			ModuleFunction('AfterDeleteMail', array($id, &$this));

			return(true);
		}
		else 
		{
			// mark mail as trashed
			$this->MoveMail($id, FOLDER_TRASH);
			
			// flag mail as read
			$this->FlagMail(FLAG_UNREAD, false, $id);
			
			$cacheManager->Delete('folderList:1:' . $this->_userID . ':' . $this->_mailboxGeneration);
			$this->IncGeneration();
			return(true);
		}
	}
	
	/**
	 * store mail in mailbox without passing filters etc.
	 *
	 * @param BMMail $mail Mail object
	 * @param int $folder Folder
	 * @return int Store result constant
	 */
	function StoreMail($mail, $folder = FOLDER_INBOX)
	{
		global $db, $bm_prefs, $cacheManager;
		
		// get mail size
		$oldOffset = ftell($mail->_fp);
		fseek($mail->_fp, 0, SEEK_END);
		$mailSize = ftell($mail->_fp);
		fseek($mail->_fp, 0, SEEK_SET);
		
		// space left?
		if($this->GetUsedSpace()+$mailSize > $this->GetSpaceLimit())
		{
			fseek($mail->_fp, $oldOffset, SEEK_SET);
			return(STORE_RESULT_NOTENOUGHSPACE);
		}
		else if($mailSize > $this->GetMaxMailSize())
		{
			fseek($mail->_fp, $oldOffset, SEEK_SET);
			return(STORE_RESULT_MAILTOOBIG);
		}
		
		// get mail body for DB
		if($bm_prefs['storein'] == 'db'
			&& $mailSize <= MAIL_DB_MAX)
		{
			$mailBody = '';
			while(!feof($mail->_fp))
				$mailBody .= fread($mail->_fp, 4096);
			fseek($mail->_fp, 0, SEEK_SET);			
		}
		else 
			$mailBody = 'file';
		
		// extract message ID
		$messageIDs = ExtractMessageIDs($mail->GetHeaderValue('message-id'));
		$messageID = count($messageIDs) > 0
			? $messageIDs[0]
			: '<' . GenerateRandomKey('messageID') . '@' . $bm_prefs['b1gmta_host'] . '>';
		
		// date?
		$date = @strtotime($mail->GetHeaderValue('date'));
		if($date <= 0)
			$date = time();
			
		// insert into DB
		$db->Query('INSERT INTO {pre}mails(userid,betreff,von,an,cc,body,folder,datum,trashstamp,priority,fetched,msg_id,virnam,trained,refs,flags,size) VALUES '
			. '(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)',
			$this->_userID,
			$mail->GetHeaderValue('subject'),
			$mail->GetHeaderValue('from'),
			$mail->GetHeaderValue('to'),
			$mail->GetHeaderValue('cc'),
			$mailBody,
			$folder,
			$date,
			0,
			$mail->priority == ITEMPRIO_LOW
				? 'low'
				: ($mail->priority == ITEMPRIO_HIGH
					? 'high'
					: 'normal'),
			time(),
			$messageID,
			$mail->infection,
			0,
			implode(';;;', $mail->GetReferences()),
			$mail->flags,
			$mailSize);
			
		// get insert id
		$id = $db->InsertId();
		
		// clean up cache
		$cacheManager->Delete('folderList:1:' . $this->_userID . ':' . $this->_mailboxGeneration);
		$this->IncGeneration();
		
		// failed to insert?
		if(!$id)
		{
			fseek($mail->_fp, $oldOffset, SEEK_SET);
			return(STORE_RESULT_INTERNALERROR);
		}
		
		// create file?
		if($bm_prefs['storein'] == 'file'
			|| $mailSize > MAIL_DB_MAX)
		{
			$fileName = DataFilename($id);
			
			// create file
			$fp = @fopen($fileName, 'wb');
			if(!$fp)
			{
				// log
				PutLog(sprintf('Failed to create message file <%s> in folder <%s> - cannot store message for user <%s> (%d)',
					basename($fileName),
					dirname($fileName),
					$this->_userMail,
					$this->_userID),
					PRIO_ERROR,
					__FILE__,
					__LINE__);
				
				// delete message row
				$db->Query('DELETE FROM {pre}mails WHERE id=? AND userid=?',
					$id,
					$this->_userID);
				
				// return
				fseek($mail->_fp, $oldOffset, SEEK_SET);
				return(STORE_RESULT_INTERNALERROR);
			}
			else 
			{
				// copy contents
				while(!feof($mail->_fp))
				{
					$chunk = fread($mail->_fp, 4096);
					fwrite($fp, $chunk);
				}
				fclose($fp);
				
				@chmod($fileName, 0666);
			}
		}
		
		// update space
		$this->UpdateSpace($mailSize);
		
		// okay!
		fseek($mail->_fp, $oldOffset, SEEK_SET);
		$this->_lastInsertId = $id;

		// plugins
 		ModuleFunction('AfterStoreMail',
 			array($id, &$mail, &$this));
		
		return(STORE_RESULT_OK);
	}
	
	/**
	 * store mail in mailbox with passing filters etc.
	 *
	 * @param BMMail $mail Mail object
	 * @return int Store result constant
	 */
	function ReceiveMail($mail, $folder = FOLDER_INBOX)
	{
		global $plugins, $bm_prefs, $db;
		
		// load needed classes
		if(!class_exists('BMMailBuilder'))
			include(B1GMAIL_DIR . 'serverlib/mailbuilder.class.php');
		if(!class_exists('BMFilterEval'))
			include(B1GMAIL_DIR . 'serverlib/filtereval.class.php');
		
		// destination folder and default result
		if($folder != FOLDER_INBOX && !$this->FolderExists($folder))
			$folder = FOLDER_INBOX;
		$storeResult = STORE_RESULT_OK;
		
		// trusted?
		$trusted = false;
		if(($trustToken = $mail->GetHeaderValue('x-b1gmail-trusted')) != '')
		{
			$validToken = GenerateTrustToken($mail->GetHeaderValue('message-id'),
				$mail->GetHeaderValue('from'),
				$mail->GetHeaderValue('to'),
				$mail->GetHeaderValue('subject'));
			if($validToken == $trustToken)
				$trusted = true;
		}
		
		// module functions
		$moduleResults = $plugins->callFunction('OnReceiveMail', false, true, array(&$mail, &$this, &$this->_userObject));
		foreach($moduleResults as $moduleName=>$moduleResult)
			if($moduleResult == BM_BLOCK && !$trusted)
			{
				PutLog(sprintf('Mail to <%s> (%d) from <%s> blocked by plugin <%s>',
					$this->_userMail,
					$this->_userID,
					ExtractMailAddress($mail->GetHeaderValue('from')),
					$moduleName),
					PRIO_NOTE,
					__FILE__,
					__LINE__);
				return(RECEIVE_RESULT_BLOCKED);
			}
			else if($moduleResult == BM_DELETE && !$trusted)
			{
				PutLog(sprintf('Mail to <%s> (%d) from <%s> deleted by plugin <%s>',
					$this->_userMail,
					$this->_userID,
					ExtractMailAddress($mail->GetHeaderValue('from')),
					$moduleName),
					PRIO_NOTE,
					__FILE__,
					__LINE__);
				return(RECEIVE_RESULT_DELETE);
			}
			else if($moduleResult == BM_IS_INFECTED
					&& ($mail->flags&FLAG_INFECTED)==0
					&& !$trusted)
			{
				PutLog(sprintf('Mail to <%s> (%d) from <%s> marked as infected by plugin <%s>',
					$this->_userMail,
					$this->_userID,
					ExtractMailAddress($mail->GetHeaderValue('from')),
					$moduleName),
					PRIO_NOTE,
					__FILE__,
					__LINE__);
				$mail->flags |= FLAG_INFECTED;
			}
			else if($moduleResult == BM_IS_SPAM
					&& ($mail->flags&FLAG_SPAM)==0
					&& !$trusted)
			{
				PutLog(sprintf('Mail to <%s> (%d) from <%s> marked as spam by plugin <%s>',
					$this->_userMail,
					$this->_userID,
					ExtractMailAddress($mail->GetHeaderValue('from')),
					$moduleName),
					PRIO_NOTE,
					__FILE__,
					__LINE__);
				$mail->flags |= FLAG_SPAM;
			}
		
		// duplicate detection
		if($bm_prefs['detect_duplicates'] == 'yes'
			&& count($messageIDs) > 0)
		{
			// extract message ID
			$messageIDs = ExtractMessageIDs($mail->GetHeaderValue('message-id'));
			$messageID = $messageIDs[0];
			
			// check for duplicates
			$res = $db->Query('SELECT COUNT(*) FROM {pre}mails WHERE msg_id=? AND userid=? AND folder!=?',
				$messageID,
				$this->_userID,
				FOLDER_OUTBOX);
			list($duplicateCount) = $res->FetchArray(MYSQL_NUM);
			$res->Free();
			
			// duplicate?
			if($duplicateCount > 0)
			{
				PutLog(sprintf('Message duplicate for <%s> (%d) detected (%s), processing stopped',
					$this->_userMail,
					$this->_userID,
					$messageID),
					PRIO_NOTE,
					__FILE__,
					__LINE__);
				return(RECEIVE_RESULT_DELETE);
			}
		}
		
		// user filter rules
		$filterEval = _new('BMFilterEval', array(&$this->_userObject, &$mail));
		list($filterStoreResult, $filterFolder, $filterActionFlags) = $filterEval->EvalFilters();
		if($folder == FOLDER_INBOX && $filterFolder != -1)
			$folder = $filterFolder;
		if($filterStoreResult != STORE_RESULT_OK)
		{
			$storeResult = $filterStoreResult;
			if($storeResult == RECEIVE_RESULT_BLOCKED)
			{
				PutLog(sprintf('Mail to <%s> (%d) from <%s> blocked by user filter',
					$this->_userMail,
					$this->_userID,
					ExtractMailAddress($mail->GetHeaderValue('from'))),
					PRIO_NOTE,
					__FILE__,
					__LINE__);
				return($storeResult);
			}
			else if($storeResult == RECEIVE_RESULT_DELETE)
			{
				PutLog(sprintf('Mail to <%s> (%d) from <%s> deleted by user filter',
					$this->_userMail,
					$this->_userID,
					ExtractMailAddress($mail->GetHeaderValue('from'))),
					PRIO_NOTE,
					__FILE__,
					__LINE__);
				return($storeResult);
			}
		}
		
		// bayes, if non-spam, local and spam filter enabled
		if($bm_prefs['use_bayes'] == 'yes'
			&& $bm_prefs['bayes_mode'] == 'local'
			&& $this->_userObject->_row['spamfilter'] == 'yes'
			&& !$trusted)
		{
			$filter = _new('BMMailFilter_Bayes', array(&$mail, $this->_userID));
			$filter->Filter();
		}
		
		// virus action
		if(($mail->flags&FLAG_INFECTED) != 0
			&& $this->_userObject->_row['virusfilter'] == 'yes'
			&& !$trusted)
		{
			$virusAction = $this->_userObject->_row['virusaction'];
			if($virusAction == -1)
				$storeResult = RECEIVE_RESULT_BLOCKED;
			else if($virusAction != -256
					&& $folder == FOLDER_INBOX
					&& $this->FolderExists($virusAction))
				$folder = $virusAction;
		}
		
		// remove infected flag, if virus filter is disabled
		if($this->_userObject->_row['virusfilter'] != 'yes'
			&& ($mail->flags&FLAG_INFECTED) != 0)
			$mail->flags ^= FLAG_INFECTED;
		
		// remove spam & infected flag if mail is trusted
		if($trusted)
		{
			if(($mail->flags&FLAG_INFECTED) != 0)
				$mail->flags ^= FLAG_INFECTED;
			if(($mail->flags&FLAG_SPAM) != 0)
				$mail->flags ^= FLAG_SPAM;
		}
		
		// spam action
		if(($mail->flags&FLAG_SPAM) != 0
			&& $this->_userObject->_row['spamfilter'] == 'yes')
		{
			$spamAction = $this->_userObject->_row['spamaction'];
			if($spamAction == -1)
				$storeResult = RECEIVE_RESULT_BLOCKED;
			else if($folder == FOLDER_INBOX && $this->FolderExists($spamAction))
				$folder = $spamAction;
		}
		
		// forward if storeResult == STORE_RESULT_OK
		if($storeResult == STORE_RESULT_OK
			&& $this->_userObject->_row['forward'] == 'yes'
			&& BMUser::AddressValid($forwardTo = ExtractMailAddress($this->_userObject->_row['forward_to']))
			&& ExtractMailAddress($mail->GetHeaderValue('return-path')) != ''
			&& BMUser::GetID($forwardTo) != $this->_userObject->_id)
		{
			$forwardTempID = RequestTempFile($this->_userObject->_id, -1);
			$forwardTempFileName = TempFileName($forwardTempID);
			
			if($fp = fopen($forwardTempFileName, 'wb+'))
			{
				// add mail headers
				fprintf($fp, 'X-Forward-To: <%s>' . "\r\n", $forwardTo);
				fprintf($fp, 'Return-Path: <>' . "\r\n");
				fprintf($fp, 'To: <%s>' . "\r\n", $forwardTo);
				
				// copy original content
				$forwardInHeader = true;
				$forwardOldPos = ftell($mail->_fp);
				fseek($mail->_fp, 0, SEEK_SET);
				while(!feof($mail->_fp))
				{
					$line = rtrim(fgets2($mail->_fp)) . "\r\n";
					
					if($forwardInHeader && $line == "\r\n")
						$forwardInHeader = false;
						
					if(!$forwardInHeader ||
						(strtolower(substr($line, 0, 12)) != 'return-path:'
						&& strtolower(substr($line, 0, 3)) != 'to:'
						&& strtolower(substr($line, 0, 3)) != 'cc:'
						&& strtolower(substr($line, 0, 4)) != 'bcc:'
						&& strtolower(substr($line, 0, 13)) != 'x-forward-to:'))
					{
						fwrite($fp, $line);
					}
				}
				fseek($mail->_fp, $forwardOldPos, SEEK_SET);
				
				// reset stream
				fseek($fp, 0, SEEK_SET);
				
				// send
				$sendMail = _new('BMSendMail');
				$sendMail->SetSender($mail->GetHeaderValue('from'));
				$sendMail->SetRecipients($forwardTo);
				$sendMail->SetSubject(($subject = $mail->GetHeaderValue('subject')) != ''
						? $subject
						: '(no subject)');
				$sendMail->SetBodyStream($fp);
				
				// send, log
				if($sendMail->Send())
				{
					PutLog(sprintf('Forwarded mail for <%s> (%d) to <%s> (forwardDelete: %d)',
						$this->_userMail,
						$this->_userID,
						$forwardTo,
						$this->_userObject->_row['forward_delete'] == 'yes'),
						PRIO_NOTE,
						__FILE__,
						__LINE__);
					if($this->_userObject->_row['forward_delete'] == 'yes')
						return(RECEIVE_RESULT_DELETE);
				}
				else 
				{
					PutLog(sprintf('Failed to forwarded mail for <%s> (%d) to <%s>',
						$this->_userMail,
						$this->_userID,
						$forwardTo),
						PRIO_WARNING,
						__FILE__,
						__LINE__);
				}
				
				// close file
				fclose($fp);
			}
			
			ReleaseTempFile($this->_userObject->_id, $forwardTempID);
		}
		else if($this->_userObject->_row['forward'] == 'yes')
		{
			PutLog(sprintf('Will not forward mail for <%s> (storeResultCheck: %d, addressValidityCheck: %d, returnPathCheck: %d, idCheck: %d)',
				$this->_userMail,
				$storeResult == STORE_RESULT_OK,
				BMUser::AddressValid($forwardTo = ExtractMailAddress($this->_userObject->_row['forward_to'])),
				ExtractMailAddress($mail->GetHeaderValue('return-path')) != '',
				BMUser::GetID($forwardTo) != $this->_userObject->_id),
				PRIO_DEBUG,
				__FILE__,
				__LINE__);
		}
		
		// autoresponder if nonspam and nonjunk
		if(($mail->flags&FLAG_SPAM) == 0
			&& strpos(strtolower($mail->GetHeaderValue('precedence')), 'junk') === false
			&& ($mailFrom = ExtractMailAddress($mail->GetHeaderValue('from'))) != ''
			&& $mail->GetHeaderValue('x-autoresponder') == ''
			&& ExtractMailAddress($mail->GetHeaderValue('return-path')) != ''
			&& !$trusted)
		{
			$autoresponderSettings = $this->_userObject->GetAutoresponder();
			
			// autoresponder active?
			if($autoresponderSettings[0]
				&& $autoresponderSettings[3] != strtolower($mailFrom))
			{
				$arMail = _new('BMMailBuilder');
				$arMail->AddHeaderField('From',				$this->_userMail);
				$arMail->AddHeaderField('To',				$mailFrom);
				$arMail->AddHeaderField('Subject',			$autoresponderSettings[1]);
				$arMail->AddHeaderField('X-Autoresponder',	'yes');
				$arMail->AddHeaderField('Precedence',		'junk');
				$arMail->AddText($autoresponderSettings[2],
					'plain',
					$this->_userObject->_row['charset']);
				$success = $arMail->Send() !== false;
				$arMail->CleanUp();
				
				// log
				if($success)
				{
					$this->_userObject->SetAutoresponderLastSend($mailFrom);
					Add2Stat('sysmail');
					PutLog(sprintf('Sent autoresponder from <%s> (%d) to <%s>',
						$this->_userMail,
						$this->_userID,
						$mailFrom),
						PRIO_DEBUG,
						__FILE__,
						__LINE__);
				}
				else 
				{
					PutLog(sprintf('Failed to send autoresponder from <%s> (%d) to <%s>',
						$this->_userMail,
						$this->_userID,
						$mailFrom),
						PRIO_WARNING,
						__FILE__,
						__LINE__);
				}
			}
		}
		
		// store mail
		if($storeResult == STORE_RESULT_OK)
			$storeResult = $this->StoreMail($mail, $folder);
			
		// spam & infection stats
		if(($mail->flags & FLAG_SPAM) != 0)
			Add2Stat('spam');
		if(($mail->flags & FLAG_INFECTED) != 0)
			Add2Stat('infected');
		
		// stats, logs
		if($storeResult == STORE_RESULT_OK)
		{
			// mail2sms if (nonspam and mail2sms enabled) or (filter action flag is set)
			if((($this->_userObject->_row['mail2sms'] == 'yes' && ($mail->flags&FLAG_SPAM) == 0)
				|| ($filterActionFlags & FILTER_ACTIONFLAG_MAIL2SMS) != 0)
				&& strlen(trim($this->_userObject->_row['mail2sms_nummer'])) > 3)
			{
				$toNo = $this->_userObject->_row['mail2sms_nummer'];
				$smsText = GetPhraseForUser($this->_userID, 'lang_custom', 'mail2sms');
				$smsText = str_replace('%%abs%%', ExtractMailAddress($mail->GetHeaderValue('from')), $smsText);
				$smsText = str_replace('%%betreff%%', $mail->GetHeaderValue('subject'), $smsText);
				if(strlen($smsText) > 160)
					$smsText = substr($smsText, 0, 157) . '...';
				
				$sms = _new('BMSMS', array($this->_userID, &$this->_userObject));
				$sms->Send($bm_prefs['mail2sms_abs'], $toNo, $smsText, 0, true, true);
			}
			
			// module functions
			ModuleFunction('AfterReceiveMail', array(&$mail, &$this, &$this->_userObject));
			Add2Stat('receive');
			$this->_userObject->UpdateLastReceive();
			PutLog(sprintf('<%s> (%d) received mail from <%s>',
				$this->_userMail,
				$this->_userID,
				ExtractMailAddress($mail->GetHeaderValue('from'))),
				PRIO_NOTE,
				__FILE__,
				__LINE__);
		}
		else if($storeResult == STORE_RESULT_INTERNALERROR)
			PutLog(sprintf('Failed to store mail for <%s> (%d) from <%s>: Internal error',
				$this->_userMail,
				$this->_userID,
				ExtractMailAddress($mail->GetHeaderValue('from'))),
				PRIO_WARNING,
				__FILE__,
				__LINE__);
		else if($storeResult == STORE_RESULT_MAILTOOBIG)
			PutLog(sprintf('Failed to store mail for <%s> (%d) from <%s>: Mail too big',
				$this->_userMail,
				$this->_userID,
				ExtractMailAddress($mail->GetHeaderValue('from'))),
				PRIO_NOTE,
				__FILE__,
				__LINE__);
		else if($storeResult == STORE_RESULT_NOTENOUGHSPACE)
			PutLog(sprintf('Failed to store mail for <%s> (%d) from <%s>: Not enough space left in user mailbox',
				$this->_userMail,
				$this->_userID,
				ExtractMailAddress($mail->GetHeaderValue('from'))),
				PRIO_NOTE,
				__FILE__,
				__LINE__);
		
		return($storeResult);
	}
	
	/**
	 * send cert mail
	 *
	 * @param int $mailID Mail ID
	 * @param BMMail $mailObj Mail object
	 * @return int
	 */
	function SendCertMail($mailID, $mailObj)
	{
		global $db, $bm_prefs;
		
		$count = 0;
		$recipients = ExtractMailAddresses(implode(' ', array(
				$mailObj->GetHeaderValue('to'),
				$mailObj->GetHeaderValue('cc'),
				$mailObj->GetHeaderValue('bcc')
			)));
		
		foreach($recipients as $to)
		{
			$db->Query('INSERT INTO {pre}certmails(mail,recipient,user,code,date) VALUES(?,?,?,?,?)',
				$mailID,
				$to,
				$this->_userID,
				$certMailCode = GenerateRandomKey('certMail:' . $to),
				time());
			
			if($certMailID = $db->InsertId())
			{
				$vars = array(
					'user_name'		=> $this->_userObject->_row['vorname'] . ' ' . $this->_userObject->_row['nachname'],
					'user_mail'		=> ExtractMailAddress($mailObj->GetHeaderValue('from')),
					'date'			=> date('d.m.Y, H:i', time()+$bm_prefs['einsch_life']),
					'url'			=> sprintf('%sindex.php?action=readCertMail&id=%d&key=%s',
						$bm_prefs['selfurl'],
						$certMailID,
						$certMailCode)
				);
				
				if(SystemMail($mailObj->GetHeaderValue('from'),
					$to,
					$mailObj->GetHeaderValue('subject'),
					'certmail',
					$vars))
				{
					$count++;
				}
				else 
				{
					$db->Query('DELETE FROM {pre}certmails WHERE id=?',
						$certMailID);
				}
			}
		}
		
		if($count)
			$this->FlagMail(FLAG_CERTMAIL, true, $mailID);
		
		return($count);
	}
	
	/**
	 * get cert mail
	 *
	 * @param int $id
	 * @param string $key
	 * @return BMMail
	 */
	function GetCertMail($id, $key)
	{
		global $db, $bm_prefs;
		
		$id = (int)$id;
		$key = trim($key);
		
		if(strlen($key) == 32)
		{
			$res = $db->Query('SELECT id,mail,recipient,user,date,code,confirmation_sent FROM {pre}certmails WHERE id=? AND code=?',
				$id,
				$key);
			if($res->RowCount() != 1)
				return(false);
			$row = $res->FetchArray(MYSQL_ASSOC);
			
			if($row['code'] == $key)
			{
				$userID = $row['user'];
				$userObject = _new('BMUser', array($userID));
				
				if($userObject)
				{
					$mailbox = _new('BMMailbox', array($userID, $userObject->_row['email'], $userObject));
					if($mailbox)
					{
						if($mailObject = $mailbox->GetMail($row['mail']))
						{
							if($row['confirmation_sent'] == 0)
							{
								//
								// send confirmation
								//
								$vars = array(
									'an'		=> $row['recipient'],
									'subject'	=> $mailObject->GetHeaderValue('subject'),
									'date'		=> date($userObject->_row['datumsformat'])
								);
								SystemMail($bm_prefs['passmail_abs'],
									$userObject->_row['email'],
									GetPhraseForUser($row['user'], 'lang_custom', 'cs_subject'),
									'cs_text',
									$vars,
									$row['user']);
								
								//
								// log
								//
								PutLog(sprintf('Certified mail <%d> read by <%s> (user: %d, mail: %d, key: %s, IP: %s)',
									$id,
									$row['recipient'],
									$row['user'],
									$row['mail'],
									$key,
									$_SERVER['REMOTE_ADDR']),
									PRIO_NOTE,
									__FILE__,
									__LINE__);
								
								//
								// forward "real" mail to recipient
								//
								$forwardTempID = RequestTempFile($row['user'], -1);
								$forwardTempFileName = TempFileName($forwardTempID);
								if($fp = fopen($forwardTempFileName, 'wb+'))
								{
									// add mail headers
									fprintf($fp, 'To: <%s>' . "\r\n", $row['recipient']);
									
									// copy original content
									$mailFP = $mailObject->GetMessageFP();
									$forwardInHeader = true;
									fseek($mailFP, 0, SEEK_SET);
									while(!feof($mailFP))
									{
										$line = rtrim(fgets2($mailFP)) . "\r\n";
										
										if($forwardInHeader && $line == "\r\n")
											$forwardInHeader = false;
											
										if(!$forwardInHeader ||
											(strtolower(substr($line, 0, 3)) != 'to:'
											&& strtolower(substr($line, 0, 3)) != 'cc:'
											&& strtolower(substr($line, 0, 4)) != 'bcc:'))
										{
											fwrite($fp, $line);
										}
									}
									
									// reset stream
									fseek($fp, 0, SEEK_SET);
									
									// send
									$sendMail = _new('BMSendMail');
									$sendMail->SetSender($mailObject->GetHeaderValue('from'));
									$sendMail->SetRecipients($row['recipient']);
									$sendMail->SetSubject(($subject = $mailObject->GetHeaderValue('subject')) != ''
											? $subject
											: '(no subject)');
									$sendMail->SetBodyStream($fp);
									
									// send, log
									if($sendMail->Send())
									{
										PutLog(sprintf('Sent certified mail <%d> to <%s>',
											$row['id'],
											$row['recipient']),
											PRIO_NOTE,
											__FILE__,
											__LINE__);
									}
									else 
									{
										PutLog(sprintf('Failed to send certified mail <%d> to <%s>',
											$row['id'],
											$row['recipient']),
											PRIO_WARNING,
											__FILE__,
											__LINE__);
									}
									
									// close file
									fclose($fp);
									fclose($mailFP);
								}
								
								//
								// update row
								//
								$db->Query('UPDATE {pre}certmails SET confirmation_sent=1 WHERE id=?',
									$id);
							}
							
							return($mailObject);
						}
					}
				}
			}
		}
		
		return(false);
	}
}
?>
