<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: mailbuilder.class.php,v 1.14 2009/02/16 17:16:50 patrick Exp $
 *
 */

if(!class_exists('BMSendMail'))
	include(B1GMAIL_DIR . 'serverlib/sendmail.class.php');

/**
 * mail builder class
 * 
 */
class BMMailBuilder
{
	var $_smimeSkipHeaders = array('from', 'to', 'cc', 'bcc', 'date', 'message-id', 'x-mailer',
									'reply-to', 'subject', 'x-priority', 'in-reply-to',
									'x-sender-ip', 'disposition-notification-to', 'references');
	var $_tempID;
	var $_fp;
	var $_headerFields;
	var $_parts;
	var $_trustedMail;
	var $_smimeUser = false;
	var $_smimeSign = false;
	var $_smimeEncrypt = false;
	var $_smimeHeaders = array();
	
	/**
	 * constructor
	 *
	 */
	function BMMailBuilder($trustedMail = false)
	{
		// init
		$this->_fp = false;
		$this->_headerFields = array();
		$this->_parts = array();
		$this->_initHeaderFields();
		$this->_trustedMail = $trustedMail;
	}
	
	/**
	 * add header field
	 *
	 * @param string $key
	 * @param string $value
	 */
	function AddHeaderField($key, $value)
	{
		$this->_headerFields[$key] = $value;
	}
	
	/**
	 * add text part
	 *
	 * @param mixed $data Data (file pointer or string)
	 * @param string $type Text type (plain or html)
	 * @param string $charset Text charset
	 */
	function AddText($data, $type = 'plain', $charset = 'iso-8859-1', $additionalParams = '')
	{
		if(!is_resource($data) && is_string($data))
			$data = str_replace("\n", "\r\n", str_replace("\r", '', $data));
		
		$this->_parts[] = array(
			'data'			=> $data,
			'base64'		=> false,
			'headerFields'	=> array(
				'Content-Type'					=> sprintf('text/%s; charset="%s"%s', $type, $charset, $additionalParams),
				'Content-Transfer-Encoding'		=> '8bit',
				'Content-Disposition'			=> 'inline'
			)
		);
	}
	
	/**
	 * add an attachment
	 *
	 * @param mixed $data Data (file pointer or string)
	 * @param string $contentType Content type
	 * @param string $fileName File name
	 * @param string $contentDisposition Content disposition (attachment or inline)
	 */
	function AddAttachment($data, $contentType, $fileName, $contentDisposition = 'attachment')
	{
		$this->_parts[] = array(
			'data'			=> $data,
			'base64'		=> $contentType != 'message/rfc822',
			'headerFields'	=> array(
				'Content-Type'					=> sprintf('%s; name="%s"', $contentType, $fileName),
				'Content-Transfer-Encoding'		=> $contentType == 'message/rfc822' ? '8bit' : 'base64',
				'Content-Disposition'			=> sprintf('%s; filename="%s"', $contentDisposition, $fileName)
			)
		);
	}
	
	/**
	 * build the mail
	 *
	 * @return resource
	 */
	function Build($forSending = false, $recipients = false)
	{
		// add trust signature?
		if($this->_trustedMail)
		{
			$this->AddHeaderField('X-b1gMail-Trusted',
				GenerateTrustToken($this->_headerFields['Message-ID'],
									$this->_headerFields['From'],
									$this->_headerFields['To'],
									$this->_headerFields['Subject']));
		}
		
		// create stream
		$this->_tempID = RequestTempFile(-1);
		$this->_fp = fopen(TempFileName($this->_tempID), 'wb+');
		assert('is_resource($this->_fp)');
		
		// multipart?
		if(count($this->_parts) > 1)
		{
			$multiPart = true;
			$boundary = '--Boundary-=_' .  GenerateRandomKey('mimeBoundary');
			$this->AddHeaderField('Content-Type', sprintf('multipart/mixed; boundary="%s"', $boundary));
		}
		else if(count($this->_parts) == 1)
		{
			$multiPart = false;
			$this->_headerFields = array_merge($this->_headerFields, $this->_parts[0]['headerFields']);
		}
		else
		{
			return(false);
		}
		
		// write header fields
		$this->_writePartHeader($this->_headerFields, true);
		
		// write parts
		if($multiPart)
		{
			fwrite($this->_fp, 'This is a multi-part message in MIME format.' . "\r\n");
			foreach($this->_parts as $part)
			{
				fwrite($this->_fp, "\r\n" . '--' . $boundary . "\r\n");
				$this->_writePartHeader($part['headerFields']);
				$this->_writePartBody($part);
			}
			fwrite($this->_fp, "\r\n" . '--' . $boundary . '--' . "\r\n");
		}
		else 
		{
			$this->_writePartBody($this->_parts[0]);
		}
		
		// s/mime?
		if($this->_smimeUser !== false && ($this->_smimeSign || $this->_smimeEncrypt))
		{
			if(!class_exists('BMSMIME'))
				include(B1GMAIL_DIR . 'serverlib/smime.class.php');
			
			$smime = _new('BMSMIME', array($this->_smimeUser->_id, &$this->_smimeUser));
			
			if($this->_smimeSign)
			{
				if(!$smime->SignMail($this))
				{
					PutLog(sprintf('Failed to sign S/MIME message for user <%s> (#%d)',
						$this->_smimeUser->_row['email'],
						$this->_smimeUser->_id),
						PRIO_NOTE,
						__FILE__,
						__LINE__);
					return(false);
				}
			}
			
			if($this->_smimeEncrypt)
			{
				if(!$smime->EncryptMail($this, $recipients))
				{
					PutLog(sprintf('Failed to encrypt S/MIME message for user <%s> (#%d)',
						$this->_smimeUser->_row['email'],
						$this->_smimeUser->_id),
						PRIO_NOTE,
						__FILE__,
						__LINE__);
					return(false);
				}
			}
		}
		
		// return file pointer
		fseek($this->_fp, 0, SEEK_SET);
		
		return($this->_fp);
	}
	
	/**
	 * send the mail
	 *
	 * @return resource File pointer of sent mail
	 */
	function Send()
	{
		global $bm_prefs;
		
		// prepare sender
		if(isset($this->_headerFields['Return-Path']))
			$sender = ExtractMailAddress($this->_headerFields['Return-Path']);
		else if(isset($this->_headerFields['From']))
			$sender = ExtractMailAddress($this->_headerFields['From']);
		else 
			$sender = '';
		
		// prepare recipients
		$recpString = '';
		if(isset($this->_headerFields['To']))
			$recpString .= $this->_headerFields['To'] . ' ';
		if(isset($this->_headerFields['Cc']))
			$recpString .= $this->_headerFields['Cc'] . ' ';
		if(isset($this->_headerFields['Bcc']))
			$recpString .= $this->_headerFields['Bcc'];
		$recipients = ExtractMailAddresses($recpString);
		unset($this->_headerFields['Bcc']);
		
		// build, if needed
		if($this->Build(true, $recipients) === false || !is_resource($this->_fp))
			return(false);
		
		// load class, if needed
		if(!class_exists('BMSendMail'))
			include(B1GMAIL_DIR . 'serverlib/sendmail.class.php');
		
		// send
		$sendMail = _new('BMSendMail');
		$sendMail->SetSender($sender);
		$sendMail->SetRecipients($recipients);
		$sendMail->SetSubject(isset($this->_headerFields['Subject'])
			? $this->_headerFields['Subject']
			: '(no subject)');
		$sendMail->SetBodyStream($this->_fp);
		
		// return
		fseek($this->_fp, 0, SEEK_SET);
		if($sendMail->Send())
		{
			fseek($this->_fp, 0, SEEK_SET);
			return($this->_fp);
		}
		else 
		{
			fseek($this->_fp, 0, SEEK_SET);
			return(false);
		}
	}
	
	/**
	 * clean up
	 *
	 */
	function CleanUp()
	{
		fclose($this->_fp);
		ReleaseTempFile(-1, $this->_tempID);
	}
	
	/**
	 * write part header
	 *
	 * @param array $headerFields Header fields
	 * @param bool $rootHeaders Root headers?
	 */
	function _writePartHeader($headerFields, $rootHeaders = false)
	{
		foreach($headerFields as $key=>$value)
		{
			if((!$this->_smimeEncrypt
				&& !$this->_smimeSign)
				|| !in_array(strtolower($key), $this->_smimeSkipHeaders))
				fwrite($this->_fp, sprintf('%s: %s' . "\r\n", $key, EncodeMailHeaderField($value)));
			else 
				$this->_smimeHeaders[$key] = EncodeMailHeaderField($value);
		}
		fwrite($this->_fp, "\r\n");
	}
	
	/**
	 * write part body
	 *
	 * @param array $part Part array
	 */
	function _writePartBody($part)
	{
		// encode
		if($part['base64'])
		{
			if(!is_resource($part['data']))
				fwrite($this->_fp, wordwrap(base64_encode($part['data']), 72, "\r\n", true));
			else 
			{
				fseek($part['data'], 0, SEEK_SET);
				while(!feof($part['data']))
				{
					$data = base64_encode(fread($part['data'], 54));
					fwrite($this->_fp, $data . "\r\n");
				}
			}
		}
		
		// just copy
		else 
		{
			if(!is_resource($part['data']))
				fwrite($this->_fp, $part['data']);
			else 
			{
				fseek($part['data'], 0, SEEK_SET);
				while(!feof($part['data']) && ($data = fread($part['data'], 4096)))
					fwrite($this->_fp, $data);
			}
		}
	}
	
	/**
	 * generate a message id
	 *
	 * @return string
	 */
	function _generateMessageID()
	{
		global $bm_prefs;
		return(sprintf('<%s@%s>',
			GenerateRandomKey('messageID'),
			$bm_prefs['b1gmta_host']));
	}
	
	/**
	 * initialize header fields
	 *
	 */
	function _initHeaderFields()
	{
		$this->AddHeaderField('Date',			date('r'));
		$this->AddHeaderField('MIME-Version',	'1.0');
		$this->AddHeaderField('Message-ID',		$this->_generateMessageID());
		$this->AddHeaderField('X-Mailer',		'b1gMail/' . B1GMAIL_VERSION);
	}
}
?>