<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: sendmail.class.php,v 1.11 2008/07/13 00:11:48 patrick Exp $
 *
 */
	
/**
 * mail sending system
 *
 */
class BMSendMail
{
	var $_recipients;
	var $_sender;
	var $_subject;
	var $_fp;
	
	/**
	 * constructor
	 *
	 * @return SendMail
	 */
	function SendMail()
	{
		$this->_recipients = array();
		$this->_sender = '';
		$this->_subject = '';
	}
	
	/**
	 * set recipients
	 *
	 * @param mixed $recipients Recipient(s) (array or string)
	 */
	function SetRecipients($recipients)
	{
		if(!is_array($recipients))
		{
			if(($recipient = ExtractMailAddress($recipients)) != '')
				$this->_recipients = array($recipient);
		}
		else 
		{
			$this->_recipients = array();
			foreach($recipients as $recipient)
				if(($recipient = ExtractMailAddress($recipient)) != '')
					$this->_recipients[] = $recipient;
		}
	}
	
	/**
	 * set sender
	 *
	 * @param string $sender
	 */
	function SetSender($sender)
	{
		$this->_sender = ExtractMailAddress($sender);
	}
	
	/**
	 * set subject
	 *
	 * @param string $subject
	 */
	function SetSubject($subject)
	{
		$this->_subject = $subject;
	}
	
	/**
	 * set body stream
	 *
	 * @param resource $fp
	 */
	function SetBodyStream($fp)
	{
		$this->_fp = $fp;
	}
	
	/**
	 * send the mail
	 *
	 * @return bool
	 */
	function Send()
	{
		global $bm_prefs;  
		
		if(count($this->_recipients) == 0)
			return(false); 
			
		// send using mail()...
		if($bm_prefs['send_method'] == 'php')
			return($this->_sendUsingPHPMail());	
		
		// ...or send using SMTP
		else if($bm_prefs['send_method'] == 'smtp')
			return($this->_sendUsingSMTP());
		
		// ...or send using sendmail
		else if($bm_prefs['send_method'] == 'sendmail')
			return($this->_sendUsingSendmail());
	}
	
	/**
	 * send mail using sendmail
	 *
	 * @return bool
	 */
	function _sendUsingSendmail()
	{
		global $bm_prefs;
		
		// build command
		$command = sprintf('%s -f "%s" %s',
			$bm_prefs['sendmail_path'],
			addslashes($this->_sender),
			addslashes(implode(' ', $this->_recipients)));
		
		// open
		$fp = popen($command, 'wb');
		if(!is_resource($fp))
		{
			PutLog(sprintf('Failed to execute sendmail command <%s>',
				$command),
				PRIO_WARNING,
				__FILE__,
				__LINE__);
			return(false);
		}
		
		// send
		fseek($this->_fp, 0, SEEK_SET);
		while(!feof($this->_fp) && ($line = fgets2($this->_fp)))
			fwrite($fp, rtrim($line) . "\n");
		
		// close
		return(pclose($fp) == 0);
	}
		
	/**
	 * send mail using PHP mail()
	 *
	 * @return bool
	 */
	function _sendUsingPHPMail()
	{
		// line separators
		if(substr(PHP_OS, 0, 3) == 'WIN')
			$headerEOL = "\r\n";
		else 
			$headerEOL = "\n";
		
		// get mail
		$messageHeader = $messageBody = '';
		$inBody = false;
		fseek($this->_fp, 0, SEEK_SET);
		while(!feof($this->_fp))
		{
			$line = rtrim(fgets2($this->_fp));
			if(!$inBody && $line == '')
				$inBody = true;
			else 
			{
				if($inBody)
					$messageBody .= $line . "\n";
				else if(substr($line, 0, 4) != 'To: '
					&& substr($line, 0, 9) != 'Subject: ')
					$messageHeader .= $line . $headerEOL;
			}
		}
		
		// send mail!
		if(ini_get('safe_mode'))
			$result = mail($this->_recipients[0],
				EncodeMailHeaderField($this->_subject),
				$messageBody,
				$messageHeader);
		else
			$result = mail($this->_recipients[0],
				EncodeMailHeaderField($this->_subject),
				$messageBody,
				$messageHeader,
				'-f "' . $this->_sender . '"');
		
		// return
		return($result);
	}
	
	/**
	 * send mail using SMTP
	 * 
	 * @return bool
	 */
	function _sendUsingSMTP()
	{
		global $bm_prefs;
		
		// load class, if needed
		if(!class_exists('BMSMTP'))
			include(B1GMAIL_DIR . 'serverlib/smtp.class.php');
		
		// send using SMTP
		$smtp = _new('BMSMTP', array($bm_prefs['smtp_host'], $bm_prefs['smtp_port'], $bm_prefs['b1gmta_host']));
		if($smtp->Connect())
		{
			// login
			if($bm_prefs['smtp_auth'] == 'yes')
				$smtp->Login($bm_prefs['smtp_user'], $bm_prefs['smtp_pass']);
			
			// submit mail
			if($smtp->StartMail($this->_sender, $this->_recipients))
				$ok = $smtp->SendMail($this->_fp);
			else 
				$ok = false;
		
			// disconnect
			$smtp->Disconnect();
			
			// return
			return($ok);
		}
		
		// return
		return(false);
	}
}
?>