<?php 
/*
 * b1gMail7
 * (c) 2002-2006 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: smime.class.php,v 1.4 2009/02/17 14:03:36 patrick Exp $
 *
 */

/**
 * smime class
 */
class BMSMIME
{
	var $_userID;
	var $_userObject;
	
	/**
	 * constructor
	 *
	 * @param int $userID User ID
	 * @param BMUser $userObject User object
	 * @return BMSMIME
	 */
	function BMSMIME($userID, &$userObject)
	{
		$this->_userID = (int)$userID;
		$this->_userObject = &$userObject;
	}
	
	/**
	 * write certs from array to disk
	 *
	 * @param array $certs Certs
	 * @return int Temp file ID
	 */
	function WriteCertsToDisk($certs)
	{
		// request temp file
		$tempFileID = RequestTempFile($this->_userID, time()+TIME_ONE_MINUTE);
		$tempFileName = TempFileName($tempFileID);
			
		// open file
		$fp = fopen($tempFileName, 'wb');
		
		// write certs
		foreach($certs as $cert)
			fwrite($fp, $cert."\r\n");
		
		// close
		fclose($fp);
		
		// return
		return($tempFileID);
	}
	
	function EncryptMail(&$mailBuilder, $recipients)
	{
		$result = false;
		
		// create temp file for message
		$tempID = RequestTempFile($this->_userID, time()+TIME_ONE_MINUTE);
		$tempFileName = TempFileName($tempID);
		$tempFP = fopen($tempFileName, 'wb');
		
		// write msg to temp file
		fseek($mailBuilder->_fp, 0, SEEK_SET);
		
		// copy message to temp file
		while(!feof($mailBuilder->_fp))
		{
			$buff = fread($mailBuilder->_fp, 4096);
			fwrite($tempFP, $buff);
		}
		
		// close file
		fclose($tempFP);
		
		// reset original mail file pointer
		fseek($mailBuilder->_fp, 0, SEEK_SET);
		
		// create dest temp file
		$destTempID = RequestTempFile($this->_userID, time()+TIME_ONE_MINUTE);
		$destTempFileName = TempFileName($destTempID);
		
		// get recipient certificates
		$certs = array();
		foreach($recipients as $recipient)
		{
			$cert = $this->_userObject->GetCertificateForAddress($recipient, CERTIFICATE_TYPE_PUBLIC);
			if(!$cert)
				return(false);
			$certs[] = $cert['pemdata'];
		}
		
		// add our own certificate (we should be able to read our own e-mail!)
		$cert = $this->_userObject->GetCertificateForAddress(ExtractMailAddress($mailBuilder->_headerFields['From']), CERTIFICATE_TYPE_PRIVATE);
		if($cert)
			$certs[] = $cert['pemdata'];
		
		// encrypt
		if(openssl_pkcs7_encrypt($tempFileName, $destTempFileName, $certs, $mailBuilder->_smimeHeaders))
		{
			// set original mail size to 0
			ftruncate($mailBuilder->_fp, 0);
			
			// open signed message
			$encryptedFP = fopen($destTempFileName, 'rb');
			
			// write encrypted message to original message file
			while(!feof($encryptedFP))
			{
				$buff = fread($encryptedFP, 4096);
				fwrite($mailBuilder->_fp, $buff);
			}
			
			// reset FP agan
			fseek($mailBuilder->_fp, 0, SEEK_SET);
			
			// close encrypted message
			fclose($encryptedFP);
			
			$result = true;	
		}
		
		// release
		ReleaseTempFile($this->_userID, $tempID);
		ReleaseTempFile($this->_userID, $destTempID);
		
		return($result);
	}
	
	/**
	 * Sign a mail
	 *
	 * @param BMMailBuilder $mailBuilder Mail builder object
	 * @return bool
	 */
	function SignMail(&$mailBuilder)
	{
		$result = false;
		
		// create temp file for message
		$tempID = RequestTempFile($this->_userID, time()+TIME_ONE_MINUTE);
		$tempFileName = TempFileName($tempID);
		$tempFP = fopen($tempFileName, 'wb');
		
		// write msg to temp file
		fseek($mailBuilder->_fp, 0, SEEK_SET);
		
		// copy message to temp file
		while(!feof($mailBuilder->_fp))
		{
			$buff = fread($mailBuilder->_fp, 4096);
			fwrite($tempFP, $buff);
		}
		
		// close file
		fclose($tempFP);
		
		// reset original mail file pointer
		fseek($mailBuilder->_fp, 0, SEEK_SET);
		
		// create dest temp file
		$destTempID = RequestTempFile($this->_userID, time()+TIME_ONE_MINUTE);
		$destTempFileName = TempFileName($destTempID);
		
		// find certificate
		$cert = $this->_userObject->GetCertificateForAddress(ExtractMailAddress($mailBuilder->_headerFields['From']), CERTIFICATE_TYPE_PRIVATE);
		if($cert === false)
			return(false);
		$certPEMData = $cert['pemdata'];
		$privKeyPEMData = $this->_userObject->GetPrivateKey($cert['hash']);
		$privKeyPass = $this->_userObject->GetPrivateKeyPassword($cert['hash']);
		
		// chain?
		$chainFileID = 0;
		$chainCerts = $this->_userObject->GetChainCerts($cert['hash']);
		if(is_array($chainCerts) && count($chainCerts) > 0)
			$chainFileID = $this->WriteCertsToDisk($chainCerts);
		
		// sign
		if($chainFileID > 0)
		{
			$signResult = @openssl_pkcs7_sign($tempFileName,
				$destTempFileName,
				$certPEMData,
				!empty($privKeyPass) ? array($privKeyPEMData, $privKeyPass) : $privKeyPEMData,
				$mailBuilder->_smimeHeaders,
				PKCS7_DETACHED,
				TempFileName($chainFileID));
			ReleaseTempFile($this->_userID, $chainFileID);
		}
		else 
		{
			$signResult = @openssl_pkcs7_sign($tempFileName,
				$destTempFileName,
				$certPEMData,
				!empty($privKeyPass) ? array($privKeyPEMData, $privKeyPass) : $privKeyPEMData,
				$mailBuilder->_smimeHeaders);
		}
			
		// success?
		if($signResult)
		{
			// set original mail size to 0
			ftruncate($mailBuilder->_fp, 0);
			
			// open signed message
			$signedFP = fopen($destTempFileName, 'rb');
			
			// write signed message to original message file
			while(!feof($signedFP))
			{
				$buff = fread($signedFP, 4096);
				fwrite($mailBuilder->_fp, $buff);
			}
			
			// reset FP agan
			fseek($mailBuilder->_fp, 0, SEEK_SET);
			
			// close signed message
			fclose($signedFP);
			
			$result = true;	
		}
		
		// release
		ReleaseTempFile($this->_userID, $tempID);
		ReleaseTempFile($this->_userID, $destTempID);
		
		return($result);
	}
	
	/**
	 * decrypt a signed mail
	 *
	 * @param BMMail $mail Mail object
	 * @return array Result code, mail object
	 */
	function DecryptMail(&$mail)
	{
		// check if mail is encrypted
		if(!$mail->IsEncrypted())
			return(array(SMIME_NOT_ENCRYPTED));
		
		// get available priv. certs
		$privateCerts = $this->_userObject->GetKeyRing('certificateid', 'ASC', CERTIFICATE_TYPE_PRIVATE);
		
		// create temp file for message
		$tempID = RequestTempFile($this->_userID, time()+TIME_ONE_MINUTE);
		$tempFileName = TempFileName($tempID);
		$tempFP = fopen($tempFileName, 'wb');
		$messageFP = $mail->GetMessageFP();
		
		// copy message to temp file
		while(!feof($messageFP))
		{
			$buff = fread($messageFP, 4096);
			fwrite($tempFP, $buff);
		}
		
		// close files
		fclose($messageFP);
		fclose($tempFP);
		
		// create dest temp file
		$destTempID = RequestTempFile($this->_userID, time()+TIME_ONE_MINUTE);
		$destTempFileName = TempFileName($destTempID);
		
		// try to dectypt it
		$success = false;
		foreach($privateCerts as $cert)
		{
			$privKeyPEMData = $this->_userObject->GetPrivateKey($cert['hash']);
			if(!$privKeyPEMData)
				continue;
			$privKeyPass = $this->_userObject->GetPrivateKeyPassword($cert['hash']);

			$result = @openssl_pkcs7_decrypt($tempFileName, $destTempFileName, $cert['pemdata'], !empty($privKeyPass) ? array($privKeyPEMData, $privKeyPass) : $privKeyPEMData);
			if($result)
			{
				$success = true;
				break;
			}
		}
		
		// success?
		if($success)
			$resultCode = SMIME_DECRYPTED;
		else
			$resultCode = SMIME_DECRYPTION_FAILED;
		
		// release
		ReleaseTempFile($this->_userID, $tempID);
		
		// create new mail object
		if($resultCode == SMIME_DECRYPTED && file_exists($destTempFileName))
		{
			$GLOBALS['tempFilesToReleaseAtShutdown'][] = array($this->_userID, $destTempID);
			
			$destFP = fopen($destTempFileName, 'rb');
			$newMail = _new('BMMail', array($this->_userID, $mail->_row, $destFP, false, $destTempFileName));
			$newMail->Parse();
			
			foreach($mail->_parsed->rootPart->header->items as $key=>$val)
				if(!isset($newMail->_parsed->rootPart->header->items[$key]))
					$newMail->_parsed->rootPart->header->items[$key] = $val;
		}
		else
			ReleaseTempFile($this->_userID, $tempID);
		
		// build result
		$result = array($resultCode);
		if(isset($newMail))
			$result[] = $newMail;
		else
			$result[] = false;
			
		// return
		return($result);
	}
	
	/**
	 * check mail signature
	 *
	 * @param BMMail $mail Mail object
	 * @return array Result code, mail object, cert hash
	 */
	function CheckMailSignature(&$mail)
	{
		// check if mail is signed
		if(!$mail->IsSigned())
			return(array(SMIME_SIGNATURE_NOT_SIGNED));
		
		// get root certs
		$rootCerts = array_values(array_merge(GetRootCertificates(), $this->_userObject->GetRootCertificates()));
		
		// create temp file for message
		$tempID = RequestTempFile($this->_userID, time()+TIME_ONE_MINUTE);
		$tempFileName = TempFileName($tempID);
		$tempFP = fopen($tempFileName, 'wb');
		$messageFP = $mail->GetMessageFP();
		
		// copy message to temp file
		while(!feof($messageFP))
		{
			$buff = fread($messageFP, 4096);
			fwrite($tempFP, $buff);
		}
		
		// close files
		fclose($messageFP);
		fclose($tempFP);
		
		// write certs to disk :(
		$certFileID = $this->WriteCertsToDisk($rootCerts);
		
		// create dest temp file
		$destTempID = RequestTempFile($this->_userID, time()+TIME_ONE_MINUTE);
		$destTempFileName = TempFileName($destTempID);
		$certTempID = RequestTempFile($this->_userID, time()+TIME_ONE_MINUTE);
		$certTempFileName = TempFileName($certTempID);
		
		// check it
		$result = openssl_pkcs7_verify($tempFileName,
			0,
			$certTempFileName,
			array(TempFileName($certFileID)),
			B1GMAIL_DIR . 'res/dummy.pem',
			$destTempFileName);
		if(!$result)
		{
			$result = openssl_pkcs7_verify($tempFileName,
				PKCS7_NOVERIFY,
				$certTempFileName,
				array(TempFileName($certFileID)),
				B1GMAIL_DIR . 'res/dummy.pem',
				$destTempFileName);
			$resultCode = $result === true ? SMIME_SIGNATURE_OK_NOVERIFY : SMIME_SIGNATURE_BAD;
		}
		else
		{
			$resultCode = $result === true ? SMIME_SIGNATURE_OK : SMIME_SIGNATURE_BAD;
		}
		
		// process cert
		if(file_exists($certTempFileName))
			$certHash = $this->_userObject->StoreCertificate(getFileContents($certTempFileName));
		
		// release
		ReleaseTempFile($this->_userID, $certFileID);
		ReleaseTempFile($this->_userID, $tempID);
		ReleaseTempFile($this->_userID, $certTempID);
		
		// create new mail object
		if(file_exists($destTempFileName))
		{
			$GLOBALS['tempFilesToReleaseAtShutdown'][] = array($this->_userID, $destTempID);
			
			$destFP = fopen($destTempFileName, 'rb');
			$newMail = _new('BMMail', array($this->_userID, $mail->_row, $destFP, false, $destTempFileName));
			$newMail->Parse();
			
			foreach($mail->_parsed->rootPart->header->items as $key=>$val)
				if(!isset($newMail->_parsed->rootPart->header->items[$key]))
					$newMail->_parsed->rootPart->header->items[$key] = $val;
		}
		else
			ReleaseTempFile($this->_userID, $tempID);
		
		// build result
		$result = array($resultCode);
		if(isset($newMail))
			$result[] = $newMail;
		else
			$result[] = false;
		if(isset($certHash))
			$result[] = $certHash;
		else
			$result[] = false;
			
		// return
		return($result);
	}
}
?>