<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: smtp.class.php,v 1.7 2007/12/17 15:23:07 patrick Exp $
 *
 */

/**
 * smtp class
 *
 */
class BMSMTP
{
	var $_host;
	var $_port;
	var $_sock;
	var $_helo;
	var $_my_host;
	
	/**
	 * constructor
	 *
	 * @param string $host
	 * @param int $port
	 * @return BMSMTP
	 */
	function BMSMTP($host, $port, $my_host)
	{
		$this->_host = $host;
		$this->_port = $port;
		$this->_helo = false;
		$this->_my_host = $my_host;
	}
	
	/**
	 * establish connection
	 *
	 * @return bool
	 */
	function Connect()
	{
		$this->_sock = @fsockopen($this->_host, $this->_port, $errNo, $errStr, SOCKET_TIMEOUT);
		
		if(!is_resource($this->_sock))
		{
			PutLog(sprintf('SMTP connection to <%s:%d> failed (%d, %s)',
				$this->_host,
				$this->_port,
				$errNo,
				$errStr),
				PRIO_WARNING,
				__FILE__,
				__LINE__);
			return(false);
		}
		else 
		{
			$responseLine = $this->_getResponse();
			if(substr($responseLine, 0, 3) != '220')
			{
				PutLog(sprintf('SMTP server <%s:%d> did not return +OK',
					$this->_host,
					$this->_port),
					PRIO_DEBUG,
					__FILE__,
					__LINE__);
				return(false);
			}
			return(true);
		}
	}
	
	/**
	 * log in
	 *
	 * @param string $user
	 * @param string $pass
	 * @return bool
	 */
	function Login($user, $pass)
	{
		fwrite($this->_sock, 'EHLO ' . $this->_my_host . "\r\n")
			&& substr($this->_getResponse(), 0, 3) == '250'
			&& $this->_helo = true;
		
		if(fwrite($this->_sock, 'AUTH LOGIN' . "\r\n")
			&& substr($this->_getResponse(), 0, 3) == '334')
		{
			if(fwrite($this->_sock, base64_encode($user) . "\r\n")
				&& substr($this->_getResponse(), 0, 3) == '334')
			{
				if(fwrite($this->_sock, base64_encode($pass) . "\r\n")
					&& substr($this->_getResponse(), 0, 3) == '235')
				{
					return(true);
				}
				else 
				{
					PutLog(sprintf('SMTP server <%s:%d> rejected username or password for user <%s>',
						$this->_host,
						$this->_port,
						$user),
						PRIO_WARNING,
						__FILE__,
						__LINE__);					
				}
			}
			else 
			{
				PutLog(sprintf('SMTP server <%s:%d> rejected username <%s>',
					$this->_host,
					$this->_port,
					$user),
					PRIO_WARNING,
					__FILE__,
					__LINE__);	
			}
		}
		else 
		{
			PutLog(sprintf('SMTP server <%s:%d> does not seem to support LOGIN authentication',
				$this->_host,
				$this->_port,
				$user),
				PRIO_WARNING,
				__FILE__,
				__LINE__);
		}
		
		return(false);
	}
	
	/**
	 * disconnect
	 *
	 * @return bool
	 */
	function Disconnect()
	{
		fwrite($this->_sock, 'QUIT' . "\r\n")
			&& $this->_getResponse();
		fclose($this->_sock);
		return(true);
	}
	
	/**
	 * initiate mail transfer
	 *
	 * @param string $from Sender address
	 * @param mixed $to Recipients (single address or array of addresses)
	 * @return bool
	 */
	function StartMail($from, $to)
	{
		// send helo, if not sent yet (e.g. at login)
		if(!$this->_helo)
			fwrite($this->_sock, 'HELO ' . $this->_my_host . "\r\n")
				&& substr($this->_getResponse(), 0, 3) == '250'
				&& $this->_helo = true;
		
		// send MAIL FROM
		if(fwrite($this->_sock, 'MAIL FROM:<' . $from . '>' . "\r\n")
			&& substr($this->_getResponse(), 0, 3) == '250')
		{
			if(!is_array($to))
				$to = array($to);
			
			// send RCPT TO
			foreach($to as $address)
				fwrite($this->_sock, 'RCPT TO:<' . $address . '>' . "\r\n")
					&& $this->_getResponse();
			
			// ok!
			return(true);
		}
		else 
		{
			PutLog(sprintf('SMTP server <%s:%d> did not accept sender address <%s>',
				$this->_host,
				$this->_port,
				$from),
				PRIO_DEBUG,
				__FILE__,
				__LINE__);			
		}
		
		return(false);
	}
	
	/**
	 * send mail data
	 *
	 * @param resource $fp File pointer
	 * @return bool
	 */
	function SendMail($fp)
	{
		// send DATA command
		if(fwrite($this->_sock, 'DATA' . "\r\n")
			&& substr($this->_getResponse(), 0, 3) == '354')
		{
			// send mail
			fseek($fp, 0, SEEK_SET);
			while(!feof($fp)
					&& ($line = fgets2($fp)) !== false
					&& fwrite($this->_sock, rtrim($line) . "\r\n") !== false);
		
			// finish
			return(fwrite($this->_sock, "\r\n" . '.' . "\r\n")
					&& substr($this->_getResponse(), 0, 3) == '250');
		}
		
		return(false);
	}
	
	/**
	 * reset session
	 *
	 * @return bool
	 */
	function Reset()
	{
		return(fwrite($this->_sock, 'RSET' . "\r\n")
				&& substr($this->_getResponse(), 0, 3) == '250');		
	}
	
	/**
	 * get smtp server response (may consist of multiple lines)
	 *
	 * @return string
	 */
	function _getResponse()
	{
		$response = '';
		while($line = fgets2($this->_sock))
		{
			$response .= $line;
			if($line[3] != '-')
				break;
		}
		return($response);
	}
}
?>