<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: todo.class.php,v 1.7 2007/12/05 20:54:12 patrick Exp $
 *
 */

/**
 * constants
 */
define('TASKS_NOTBEGUN', 		16);
define('TASKS_PROCESSING', 		32);
define('TASKS_DONE', 			64);
define('TASKS_POSTPONED',		128);

/**
 * todo interface class
 */
class BMTodo
{
	var $_userID;
	var $_prioTrans = array(
		'low'		=> -1,
		'normal'	=> 0,
		'high'		=> 1,
		-1			=> 'low',
		0			=> 'normal',
		1			=> 'high'
	);
	
	/**
	 * constructor
	 *
	 * @param int $userID User ID
	 * @return BMTodo
	 */
	function BMTodo($userID)
	{
		$this->_userID = $userID;
	}
	
	/**
	 * get list of tasks
	 *
	 * @param string $sortColumn Sort column 
	 * @param string $sortOrder Sort order
	 * @param int $limit Entry limit
	 * @return array
	 */
	function GetTodoList($sortColumn = 'faellig,beginn', $sortOrder = 'ASC', $limit = -1)
	{
		global $db;
		
		$result = array();
		$res = $db->Query('SELECT id,beginn,faellig,akt_status,titel,priority,erledigt,comments FROM {pre}tasks WHERE user=? ORDER BY ' . $sortColumn . ' '  . $sortOrder
							. ($limit != -1  ? ' LIMIT ' . $limit : ''),
							$this->_userID);
		while($row = $res->FetchArray())
		{
			$result[$row['id']] = array(
				'id'			=> $row['id'],
				'beginn'		=> $row['beginn'],
				'faellig'		=> $row['faellig'],
				'akt_status'	=> $row['akt_status'],
				'titel'			=> $row['titel'],
				'priority'		=> $this->_prioTrans[$row['priority']],
				'erledigt'		=> $row['erledigt'],
				'comments'		=> $row['comments']
			);
		}
		
		return($result);
	}
	
	/**
	 * get undone task count
	 *
	 * @return int
	 */
	function GetUndoneTaskCount()
	{
		global $db;
		
		$res = $db->Query('SELECT COUNT(*) FROM {pre}tasks WHERE user=? AND akt_status!=?',
			$this->_userID,
			TASKS_DONE);
		list($taskCount) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		return($taskCount);
	}
	
	/**
	 * add a task
	 *
	 * @param int $beginn Begin
	 * @param int $faellig Due
	 * @param int $akt_status Status
	 * @param string $titel Titel
	 * @param int $priority Priority
	 * @param int $erledigt Done
	 * @param string $comments Comments
	 * @return int
	 */
	function Add($beginn, $faellig, $akt_status, $titel, $priority, $erledigt, $comments)
	{
		global $db;
		
		// translate $priority, if neccessary
		if(is_numeric($priority))
			$priority = $this->_prioTrans[$priority];
		
		$db->Query('INSERT INTO {pre}tasks(user,beginn,faellig,akt_status,titel,priority,erledigt,comments) VALUES(?,?,?,?,?,?,?,?)',
			$this->_userID,
			(int)$beginn,
			(int)$faellig,
			(int)$akt_status,
			$titel,
			$priority,
			(int)$erledigt,
			$comments);
		return($db->InsertID());
	}
	
	/**
	 * change a task
	 *
	 * @param int $id Task ID
	 * @param int $beginn Begin
	 * @param int $faellig Due
	 * @param int $akt_status Status
	 * @param string $titel Titel
	 * @param int $priority Priority
	 * @param int $erledigt Done
	 * @param string $comments Comments
	 * @return bool
	 */
	function Change($id, $beginn, $faellig, $akt_status, $titel, $priority, $erledigt, $comments)
	{
		global $db;
		
		// translate $priority, if neccessary
		if(is_numeric($priority))
			$priority = $this->_prioTrans[$priority];
		
		$db->Query('UPDATE {pre}tasks SET beginn=?,faellig=?,akt_status=?,titel=?,priority=?,erledigt=?,comments=? WHERE id=? AND user=?',
			(int)$beginn,
			(int)$faellig,
			(int)$akt_status,
			$titel,
			$priority,
			(int)$erledigt,
			$comments,
			(int)$id,
			$this->_userID);
		return($db->AffectedRows() == 1);
	}
	
	/**
	 * update task status
	 *
	 * @param int $id Task ID
	 * @param int $status New status
	 * @return bool
	 */
	function SetStatus($id, $status)
	{
		global $db;
		
		$db->Query('UPDATE {pre}tasks SET akt_status=? WHERE id=? AND user=?',
			(int)$status,
			(int)$id,
			$this->_userID);
		return($db->AffectedRows() == 1);
	}
	
	/**
	 * delete a task
	 *
	 * @param int $id Task ID
	 * @return bool
	 */
	function Delete($id)
	{
		global $db;
	
		$db->Query('DELETE FROM {pre}tasks WHERE id=? AND user=?',
			(int)$id,
			$this->_userID);
		return($db->AffectedRows() == 1);
	}
	
	/**
	 * get task info
	 * 
	 * @param int $id Task ID
	 * @return array
	 */
	function GetTask($id)
	{
		global $db;
		
		$res = $db->Query('SELECT id,beginn,faellig,akt_status,titel,priority,erledigt,comments FROM {pre}tasks WHERE id=? AND user=?',
			(int)$id,
			$this->_userID);
		if($res->RowCount() == 0)
			return(false);
		$row = $res->FetchArray();
		$res->Free();
		
		return(array(
			'id'			=> $row['id'],
			'beginn'		=> $row['beginn'],
			'faellig'		=> $row['faellig'],
			'akt_status'	=> $row['akt_status'],
			'titel'			=> $row['titel'],
			'priority'		=> $this->_prioTrans[$row['priority']],
			'erledigt'		=> $row['erledigt'],
			'comments'		=> $row['comments']
		));
	}
}
?>