<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: toolinterface.class.php,v 1.13 2008/07/19 11:27:18 patrick Exp $
 *
 */

if(!class_exists('BMMailbox'))
	include(B1GMAIL_DIR . 'serverlib/mailbox.class.php');
if(!class_exists('BMAddressbook'))
	include(B1GMAIL_DIR . 'serverlib/addressbook.class.php');
if(!class_exists('BMMailBuilder'))
	include(B1GMAIL_DIR . 'serverlib/mailbuilder.class.php');

/**
 * interface for b1gMail tools (like mailchecker)
 *
 */
class BMToolInterface
{
	var $_sms, $_user, $_group;
	
	/**
	 * constructor
	 *
	 * @return BMToolInterface
	 */
	function BMToolInterface()
	{
	}
	
	/**
	 * check for new mails (mailchecker interface)
	 *
	 * @param string $userName
	 * @param string $passwordHash
	 * @return array
	 */
	function CheckForMails($userName, $passwordHash)
	{
		$result = array();
		$userInfo = $this->CheckLogin($userName, $passwordHash);
		
		if($userInfo['loginOK'] > 0)
		{
			$userID = $userInfo['userID'];
			$mailbox = _new('BMMailbox', array($userID, $userName, $this->_user));
			$result['status'] = 'OK';
			$result['recentMails'] = $mailbox->GetRecentMailCount();
		}
		else 
		{
			$result['status'] = 'Invalid login';
		}
		return($result);
	}
	
	/**
	 * send a SMS (smsmanager interface)
	 *
	 * @param string $userName
	 * @param string $passwordHash
	 * @param string $from
	 * @param string $to
	 * @param int $type
	 * @param string $text
	 * @return array
	 */
	function SendSMS($userName, $passwordHash, $from, $to, $type, $text)
	{
		$result = array('sendOK' => false);
		$userInfo = $this->CheckLogin($userName, $passwordHash);
		
		if($userInfo['loginOK'] > 0 && is_object($this->_sms))
		{
			$userID = $userInfo['userID'];
			
			// prepare from number
			if($this->_group->_row['sms_ownfrom'] == 'yes')
			{
				$from = str_replace('+', '00', $from);
				$from = ereg_replace('[^0-9]', '', $from);
			}
			else 
				$from = $this->_group->_row['sms_from'];
			
			// prepare to number
			$to = str_replace('+', '00', $to);
			$to = ereg_replace('[^0-9]', '', $to);
			
			// add signature
			if(strlen($text) > $this->_sms->GetMaxChars())
				$text = substr($text, 0, $this->_sms->GetMaxChars());
			$text .= $this->_group->_row['sms_sig'];

			if(!BMSMS::PreOK($to, $this->_group->_row['sms_pre'])
				|| ($this->_group->_row['sms_ownfrom'] == 'yes' && !BMSMS::PreOK($from, $this->_group->_row['sms_pre'])))
			{
				$result['sendOK'] = false;	
			}
			else 
			{
				$result['sendOK'] = $this->_sms->Send($from, $to, $text, (int)$type, true, true);
			}
			
			$this->_user->Fetch(-1, true);
			$result['balance'] = $this->_user->GetBalance();
		}
		else 
		{
			$result['status'] = 'Invalid login';
		}
		return($result);
	}
	
	/**
	 * get SMS outbox (smsmanager interface)
	 *
	 * @param string $userName
	 * @param string $passwordHash
	 * @return array
	 */
	function GetSMSOutbox($userName, $passwordHash)
	{
		$result = array('sendOK' => false);
		$userInfo = $this->CheckLogin($userName, $passwordHash);
		
		if($userInfo['loginOK'] > 0 && is_object($this->_sms))
		{
			$result['outbox'] = $this->_sms->GetOutbox('date', 'desc');
			$result['status'] = 'OK';
		}
		else 
		{
			$result['status'] = 'Invalid login';
		}
		return($result);
	}
	
	/**
	 * get SMS addressbook (smsmanager interface)
	 *
	 * @param string $userName
	 * @param string $passwordHash
	 * @return array
	 */
	function GetSMSAddressbook($userName, $passwordHash)
	{
		$result = array('sendOK' => false);
		$userInfo = $this->CheckLogin($userName, $passwordHash);
		
		if($userInfo['loginOK'] > 0 && is_object($this->_sms))
		{
			$userID = $userInfo['userID'];
		
			// addressbook
			$addresses = array();
			$book = _new('BMAddressbook', array($userID));
			$addressBook = $book->GetAddressBook('*', -1, 'nachname', 'asc');
			foreach($addressBook as $id=>$entry)
			{
				if(trim($entry['handy']) != '' || trim($entry['work_handy']) != '')
					$addresses[] = array('firstname' 		=> $entry['vorname'],
											'lastname'		=> $entry['nachname'],
											'handy'			=> $entry['handy'],
											'work_handy'	=> $entry['work_handy'],
											'id'			=> $entry['id']);
			}
	
			$result['addresses'] = $addresses;
			$result['status'] = 'OK';
		}
		else 
		{
			$result['status'] = 'Invalid login';
		}
		return($result);
	}

	/**
	 * get service info
	 *
	 * @return array
	 */
	function GetServiceInfo()
	{
		global $bm_prefs;

		$result = array(
			'title'		=> $bm_prefs['titel'],
			'domains'	=> explode(':', $bm_prefs['domains'])
		);

		return($result);
	}
	
	/**
	 * create a draft
	 *
	 * @return array
	 */
	function CreateDraft($userName, $passwordHash)
	{
		global $currentCharset;
		
		$result = array('draftID' => -1);
		$userInfo = $this->CheckLogin($userName, $passwordHash);
		
		if($userInfo['loginOK'] > 0)
		{
			$userID = $userInfo['userID'];
		
			if(!isset($_POST['msg']) || !is_array($_POST['msg']))
			{
				$result['status'] = 'Missing message';
			}
			else
			{
				$msg = $_POST['msg'];
				
				// build mail
				$mail = _new('BMMailBuilder');
				
				// subject
				if(isset($msg['subject']))
					$mail->AddHeaderField('Subject', $msg['subject']);
					
				// subject
				if(isset($msg['from']))
					$mail->AddHeaderField('From', $msg['from']);
				else 
					$mail->AddHeaderField('From', $this->_user->GetDefaultSender());
				
				// text
				if(!isset($msg['text']))
					$msg['text'] = '';
				$mail->AddText($msg['text'], 'plain', $currentCharset);				
				
				// attachments
				$tempFiles = array();
				if(isset($_FILES['msg']) && is_array($_FILES['msg']))
				{
					foreach($_FILES['msg']['name']['files'] as $key=>$val)
					{
						if($_FILES['msg']['error']['files'][$key] == 0)
						{
							$tempFileID = RequestTempFile($userID);
							$tempFileName = TempFileName($tempFileID);
							
							if(@move_uploaded_file($_FILES['msg']['tmp_name']['files'][$key],
								$tempFileName))
							{
								$tempFileFP = fopen($tempFileName, 'rb');
								$tempFiles[] = array(
									'id'	=> $tempFileID,
									'fp'	=> $tempFileFP
								);
								
								// add to mail
								$mail->AddAttachment($tempFileFP,
									'application/octet-stream',
									$val);
							}
							else 
								ReleaseTempFile($tempFileID);
						}
					}
				}
				
				// save to drafts folder
				$mailFP = $mail->Build();
				$mailbox = _new('BMMailbox', array($userID, $userName, $this->_user));
				$mailObj = _new('BMMail', array(0, false, $mailFP, false));
				$mailObj->Parse();
				$mailObj->ParseInfo();
				if($mailbox->StoreMail($mailObj, FOLDER_DRAFTS) == STORE_RESULT_OK)
				{
					$result['draftID'] = $mailbox->_lastInsertId;
					$result['status'] = 'OK';
				}
				
				// clean up
				foreach($tempFiles as $tempFile)
				{
					fclose($tempFile['fp']);
					ReleaseTempFile($userID, $tempFile['id']);
				}
				
				$result['status'] = 'Error';
			}
		}
		else 
		{
			$result['status'] = 'Invalid login';
		}
		return($result);
	}
	
	/**
	 * check user login, return user id on success, 0 otherwise
	 *
	 * @param string $userName User E-Mail
	 * @param string $passwordHash Password hash (MD5)
	 * @return int
	 */
	function CheckLogin($userName, $passwordHash)
	{
		global $db, $bm_prefs;
		
		$userID = BMUser::GetID($userName);
		if($userID != 0)
		{
			$res = $db->Query('SELECT passwort,gesperrt,gruppe,mail2sms_nummer FROM {pre}users WHERE id=?',
				$userID);
			if($res->RowCount() == 1)
			{
				$row = $res->FetchArray();
				
				$group = _new('BMGroup', array($row['gruppe']));
				$groupRow = $group->Fetch();
				$this->_group = $group;

				if(strtolower($row['passwort']) == strtolower($passwordHash)
					&& $row['gesperrt'] == 'no')
				{
					$user = _new('BMUser', array($userID));
					
					$result = array('loginOK' => 1, 'userID' => $userID,
						'hostName' => $bm_prefs['b1gmta_host'],
						'httpmailAccess' => $groupRow['httpmail'] == 'yes',
						'pop3Access' => $groupRow['pop3'] == 'yes',
						'imapAccess' => $groupRow['imap'] == 'yes',
						'smsAccess' => $user->SMSEnabled(),
						'balance' => $user->GetBalance());
					
					if($result['smsAccess'])
					{
						$sms = _new('BMSMS', array($userID, &$user));
						$result['smsMaxChars'] = $sms->GetMaxChars();
						$result['smsTypes'] = $sms->GetTypes();
						$result['smsPre'] = trim($groupRow['sms_pre']) == ''
							? '0'
							: $groupRow['sms_pre'];
						
						if($groupRow['sms_ownfrom'] == 'yes')
						{
							$result['smsOwnFrom'] = true;
							$result['smsFrom'] = $row['mail2sms_nummer'];
						}
						else 
						{
							$result['smsOwnFrom'] = false;
							$result['smsFrom'] = $groupRow['sms_from'];
						}
						
						$this->_sms = $sms;
					}
					
					$this->_user = $user;
					return($result);
				}
			}
			$res->Free();
		}
		return(array('loginOK' => 0));
	}
}
?>