<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: workgroup.class.php,v 1.7 2008/12/18 10:41:32 patrick Exp $
 *
 */

/**
 * workgroup class
 */
class BMWorkgroup
{
	var $_id;
	var $_row;
	
	/**
	 * constructor
	 *
	 * @param int $id
	 * @return BMWorkgroup
	 */
	function BMWorkgroup($id)
	{
		$this->_id = $id;
		$this->_row = $this->Fetch();
	}
	
	/**
	 * fetch a group row (assoc)
	 * 
	 * @param int $id
	 * @return $array
	 */
	function Fetch($id = -1)
	{
		global $db;
		
		if($id == -1)
		{
			$id = $this->_id;
			if(is_array($this->_row))
				return($this->_row);
		}
		
		$res = $db->Query('SELECT * FROM {pre}workgroups WHERE id=?',
			$id);
		if($res->RowCount() == 0)
			return(false);
		$row = $res->FetchArray(MYSQL_ASSOC);
		$res->Free();
		
		return($row);
	}
	
	/**
	 * retrieve a simple id/title workgroup list for user
	 *
	 * @param int $userID
	 * @param bool $withMembers Include members?
	 * @param bool $excludeDeleted Exclude deleted members?
	 * @return array
	 */
	function GetSimpleWorkgroupList($userID, $withMembers = false, $excludeDeleted = true)
	{
		global $db;
		
		$groups = array();
		$res = $db->Query('SELECT {pre}workgroups.id AS id,{pre}workgroups.title AS title,{pre}workgroups.email AS email,{pre}workgroups.webdisk AS webdisk FROM {pre}workgroups,{pre}workgroups_member WHERE {pre}workgroups.id={pre}workgroups_member.workgroup AND {pre}workgroups_member.user=? ORDER BY title ASC',
			(int)$userID);
		while($row = $res->FetchArray(MYSQL_ASSOC))
		{
			if($withMembers)
			{
				$members = array();
				$res2 = $db->Query('SELECT {pre}users.vorname AS vorname,{pre}users.nachname AS nachname,{pre}users.email AS email,{pre}users.id AS id FROM {pre}users,{pre}workgroups_member WHERE {pre}users.id={pre}workgroups_member.user AND {pre}workgroups_member.workgroup=? ' . ($excludeDeleted ? 'AND {pre}users.gesperrt!=\'delete\' ' : '') . 'ORDER BY nachname ASC',
					$row['id']);
				while($memberRow = $res2->FetchArray(MYSQL_ASSOC))
					$members[$memberRow['id']] = $memberRow;
				$res2->Free();
				
				$row['members'] = $members;
				$row['memberCount'] = count($members);
			}
			
			$groups[$row['id']] = $row;
		}
		$res->Free();
		
		return($groups);
	}
	
	/**
	 * get workgroup id by mail address
	 *
	 * @param string $email Mail address
	 * @return int
	 */
	function GetIDbyMail($email)
	{
		global $db;
		
		list(, $domainPart) = explode('@', $email);
		$res = $db->Query('SELECT id FROM {pre}workgroups WHERE email=? OR email=? LIMIT 1',
			$email,
			'*@' . $domainPart);
		if($res->RowCount() > 0)
		{
			list($id) = $res->FetchArray(MYSQL_NUM);
			$res->Free();
			return($id);
		}
		
		return(0);
	}
	
	/**
	 * get group members (id, email)
	 *
	 * @param int $id Workgroup ID
	 * @param bool $excludeDeleted Exclude deleted users?
	 * @return array
	 */
	function GetMembers($id = -1, $excludeDeleted = true)
	{
		global $db;
		
		if($id == -1)
			$id = $this->_id;
		
		$members = array();
		$res = $db->Query('SELECT {pre}users.id AS id, {pre}users.email AS email FROM {pre}users,{pre}workgroups_member WHERE {pre}users.id={pre}workgroups_member.user AND {pre}workgroups_member.workgroup=?'
			. ($excludeDeleted ? ' AND {pre}users.gesperrt!=\'delete\'' : ''),
			$id);
		while($row = $res->FetchArray(MYSQL_ASSOC))
			$members[$row['id']] = $row;
		$res->Free();
		
		return($members);
	}
	
	/**
	 * check if user is in a workgroup
	 *
	 * @param int $userID
	 * @param int $groupID
	 * @return bool
	 */
	function UserInGroup($userID, $groupID)
	{
		global $db, $__inGroupCache;
		
		// init cache
		if(!isset($__inGroupCache) || !is_array($__inGroupCache))
			$__inGroupCache = array();
		
		// cached?
		if(isset($__inGroupCache[$userID.'_'.$groupID]))
			return($__inGroupCache[$userID.'_'.$groupID]);
		
		// get from db
		$res = $db->Query('SELECT COUNT(*) FROM {pre}workgroups_member WHERE workgroup=? AND user=?',
			(int)$groupID,
			(int)$userID);
		list($rowCount) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		// cache, return
		$__inGroupCache[$userID.'_'.$groupID] = $rowCount == 1;
		return($__inGroupCache[$userID.'_'.$groupID]);
	}
	
	/**
	 * get title by id
	 *
	 * @param int $id
	 * @return string
	 */
	function GetTitle($id)
	{
		global $db;
		
		$res = $db->Query('SELECT title FROM {pre}workgroups WHERE id=?',
			$id);
		list($title) = $res->FetchArray(MYSQL_NUM);
		$res->Free();
		
		return($title);
	}
}
?>