<?php 
/*
 * b1gMail7
 * (c) 2002-2008 B1G Software
 * 
 * Redistribution of this code without explicit permission
 * is forbidden!
 *
 * $Id: zip.class.php,v 1.4 2008/12/19 13:24:47 patrick Exp $
 *
 */

/**
 * ZIP class
 *
 */
class BMZIP
{
	/**
	 * b1gZIP stream (used if b1gZIP is installed)
	 *
	 * @var resource
	 */
	var $_b1gzip_stream;
	
	/**
	 * output stream
	 *
	 * @var resource
	 */
	var $_fp;
	
	/**
	 * central directory structure
	 *
	 * @var array
	 */
	var $_centralDirStruct;
	
	/**
	 * constructor
	 *
	 * @param resource $fp Output stream
	 * @return BMZIP
	 */
	function BMZIP($fp)
	{
		// output stream
		$this->_fp = $fp;
		
		// use b1gZIP?
		if(function_exists('b1gzip_create') 
			&& function_exists('b1gzip_add')
			&& function_exists('b1gzip_final'))
		{
			$this->_b1gzip_stream = b1gzip_create();
		}
		else 
		{
			$this->_b1gzip_stream = false;
			$this->_centralDirStruct = array();
		}
	}
	
	/**
	 * add a file to ZIP file
	 *
	 * @param string $fileName File name
	 * @param string $zipFileName File name in ZIP file
	 * @return bool
	 */
	function AddFile($fileName, $zipFileName = false)
	{
		if(!$zipFileName)
			$zipFileName = basename($fileName);
		
		// read file
		$fileFP = @fopen($fileName, 'rb');
		if($fileFP)
		{
			$fileData = @fread($fileFP, filesize($fileName));
			fclose($fileFP);
		}
		else 
			return(false);
			
		// use b1gZIP
		if($this->_b1gzip_stream)
		{
			b1gzip_add($this->_b1gzip_stream, $fileData, $zipFileName);
			return(true);
		}
		
		// or own implementation
		else 
		{
			// compute crc32
			$crc32 = crc32($fileData);
			$compressedData = gzcompress($fileData);
			unset($fileData);
			$compressedData = substr($compressedData, 2, -4);
			$compressedSize = strlen($compressedData);
			$uncompressedSize = filesize($fileName);
			
			// write file header
			$this->_beginFile($crc32, $compressedSize, $uncompressedSize, $zipFileName);
			fwrite($this->_fp, $compressedData);
		}
		
		return(false);
	}
	
	/**
	 * begin file
	 *
	 * @param int $crc32
	 * @param int $compressedSize
	 * @param int $uncompressedSize
	 * @param string $fileName
	 */
	function _beginFile($crc32, $compressedSize, $uncompressedSize, $fileName)
	{
		// local header
		$header = pack('VvvvvvVVVvv',
			0x04034b50,
			0x0014,
			0x0,
			0x0008,
			0x0,
			0x0,
			$crc32,
			$compressedSize,
			$uncompressedSize,
			strlen($fileName),
			0x0);
		$offset = ftell($this->_fp);
		fwrite($this->_fp, $header);
		fwrite($this->_fp, $fileName);
		
		// central dir struct entry
		$entry = pack('VvvvvvvVVVvvvvvVV',
			0x02014b50,
			0x0,
			0x0014,
			0x0,
			0x0008,
			0x0,
			0x0,
			$crc32,
			$compressedSize,
			$uncompressedSize,
			strlen($fileName),
			0x0,
			0x0,
			0x0,
			0x0,
			32,
			$offset);
		$entry .= $fileName;
		$this->_centralDirStruct[] = $entry;
	}
	
	/**
	 * finish zip file
	 *
	 * @return int Size
	 */
	function Finish()
	{
		// use b1gZIP?
		if($this->_b1gzip_stream)
		{
			$zipData = b1gzip_final($this->_b1gzip_stream);
			fwrite($this->_fp, $zipData);
			fseek($this->_fp, 0, SEEK_SET);
			return(strlen($zipData));
		}
		
		// or own implementation
		else 
		{
			// write central dir struct
			$offset = ftell($this->_fp);
			$dLength = 0;
			foreach($this->_centralDirStruct as $item)
			{
				fwrite($this->_fp, $item);
				$dLength += strlen($item);
			}
			
			// write footer
			$footer = pack('VvvvvVVv',
				0x06054b50,
				0x0,
				0x0,
				count($this->_centralDirStruct),
				count($this->_centralDirStruct),
				$dLength,
				$offset,
				0x0);
			fwrite($this->_fp, $footer); 
			
			// return
			$len = ftell($this->_fp);
			fseek($this->_fp, 0, SEEK_SET);
			return($len);
		}
	}
}
?>